#ifndef ALLOCDEBUG_H_
#define ALLOCDEBUG_H_
	#ifndef __APPLE__
		#ifdef _DEBUG
		#include <winsock2.h>
		#include <windows.h>
		#include <stdlib.h>
			#define DEBUG_NEW new(__FILE__, __LINE__)

			class AllocList
			{
			public:
				AllocList()
				{
					m_pNext = NULL;
					m_pPrev = NULL;
				}

				DWORD		m_dwAddress;
				DWORD		m_dwSize;
				char		m_strFile[256];
				DWORD		m_dwLine;
				int			m_iIsNewed;
				int			m_iIsArray;
				AllocList	*m_pNext;
				AllocList	*m_pPrev;
			};

			extern AllocList g_AllocList;

			void AddTrack(DWORD dwAddress, DWORD dwSize, const char *strFile, DWORD dwLine, bool bIsNewCall, bool bIsNewArrayCall = 0);
			void RemoveTrack(DWORD dwAddress, bool bIsDeleteCall, bool bIsDeleteArrayCall = 0);
			void DumpUnfreed();

			inline void * __cdecl operator new[](unsigned int size, const char *file, int line)
			{
				void *ptr = (void *)malloc(size);
				AddTrack((DWORD)ptr, size, file, line, true, true);
				memset(ptr, 0xdeadb33f, size);
				return(ptr);
			};

			inline void __cdecl operator delete[](void *p)
			{
				if(p)
					RemoveTrack((DWORD)p, true, true);

				free(p);
				p = NULL;
			};

			inline void * __cdecl operator new(unsigned int size, const char *file, int line)
			{
				void *ptr = (void *)malloc(size);
				AddTrack((DWORD)ptr, size, file, line, true);
				memset(ptr, 0xdeadb33f, size);
				return(ptr);
			};

			inline void __cdecl operator delete(void *p)
			{
				RemoveTrack((DWORD)p, true);
				free(p);
				p = NULL;
			};

			inline void* mymalloc(unsigned int size, const char *file, int line)
			{
				void *ptr = (void *)malloc(size);
				AddTrack((DWORD)ptr, size, file, line, false);
				memset(ptr, 0xdeadb33f, size);
				return(ptr);
			}

			inline void myfree(void *p)
			{
				RemoveTrack((DWORD)p, false);
				free(p);
				p = NULL;
			};
		#else
		#define DEBUG_NEW new
		#endif

		#define new DEBUG_NEW
	#endif
#endif