#include <unistd.h>

#include "Paper.h"
#include "Main.h"

xPaper	g_xPaperAPI;
WindowPtr			gGameWindow			= NULL;
EventLoopTimerRef	gMainLoopTimer		= NULL;
EventHandlerUPP		gEventHandlerUPP	= NULL;
EventHandlerRef		gEventHandlerRef	= 0;

extern void GameInit();
extern void GamePreUpdate(DWORD dwTime);
extern void GameUpdate(double dTime);
extern void GameShutdown();

int main(void)
{
#ifdef __APPLE__
	{
		char path[255];
		CFURLRef url = CFBundleCopyBundleURL(CFBundleGetMainBundle());
		CFStringRef str = CFURLCopyFileSystemPath(url,kCFURLPOSIXPathStyle);
		CFStringGetCString(str,path,255,CFStringGetSystemEncoding());
		CFRelease(str);
		CFRelease(url);
		sprintf(path, "%s/", path);
		chdir(path);
	}
#endif

	xFileIO fileio;

#ifdef _DEBUG
	char temp[256] = "FreePlay", output[256];
	uint32_t encrypted[100] = {0};
	int i;

	//xEncryptString(temp, encrypted, PRIVATE_KEY_KEY0, PRIVATE_KEY_KEY1, PRIVATE_KEY_KEY2, PRIVATE_KEY_KEY3);
	//xDecryptString(encrypted, 4, temp, PRIVATE_KEY_KEY0, PRIVATE_KEY_KEY1, PRIVATE_KEY_KEY2, PRIVATE_KEY_KEY3);

/*	strcpy(temp, "BreakBlocks.lic");
	memset(output, 0, 255);
	memset(encrypted, 0, 400);
	xEncryptString(temp, encrypted, PRIVATE_KEY_KEY0, PRIVATE_KEY_KEY1, PRIVATE_KEY_KEY2, PRIVATE_KEY_KEY3);
	for(i = 0; i < 4; i++)
		sprintf(output, "%s, %u", output, encrypted[i]);
*/
#endif

	g_xPaperAPI.Init();

	if(g_xPaperAPI.m_bRunning)
	{
		//Create Game Objects
		//Load Maps/Menus/etc.
		GameInit();

		g_xPaperAPI.m_xInput.m_lWindowOffsetX = g_xPaperAPI.m_xPlatform.GetScreenRect().left;
		g_xPaperAPI.m_xInput.m_lWindowOffsetY = g_xPaperAPI.m_xPlatform.GetScreenRect().top;

		g_xPaperAPI.m_dwLastTime = g_xPaperAPI.m_xPlatform.GetTime();

		InitEventHandler();
	
		SetMainLoopEventTimer(GameLoop);
	
		RunApplicationEventLoop();

		GameShutdown();
	}

	g_xPaperAPI.Shutdown();

	if(g_xPaperAPI.m_strExitCommand[0] != '\0')
		system(g_xPaperAPI.m_strExitCommand);

	return 0;
}

void InitEventHandler(void)
{
	EventTypeSpec	events[9] = {kEventClassCommand, kEventCommandProcess,	// command events
								 kEventClassKeyboard, kEventRawKeyDown,		// key down events
								 kEventClassKeyboard, kEventRawKeyUp,		// key up events
								 kEventClassKeyboard, kEventRawKeyModifiersChanged,	// modifiers changed event
								 kEventClassMouse, kEventMouseMoved,
								 kEventClassMouse, kEventMouseDragged,
								 kEventClassMouse, kEventMouseUp,
								 kEventClassMouse, kEventMouseDown,
								 kEventClassMouse, kEventMouseWheelMoved};
	
	gEventHandlerUPP = NewEventHandlerUPP(EventHandler);
	
	InstallEventHandler(GetApplicationEventTarget(), gEventHandlerUPP, 9, events, nil, &gEventHandlerRef);
}

static pascal OSStatus EventHandler(EventHandlerCallRef pEventHandler, EventRef pEvent, void *pUserData)
{
	UInt32 eventclass, eventkind;
	HICommand command;
	char charcode;
	UInt32 modifiers;
	static UInt32 oldmodifiers;
	short button;
	HIPoint mousecoord;

	eventclass = GetEventClass(pEvent);
	eventkind  = GetEventKind(pEvent);
	
	switch(eventclass)
	{
		case kEventClassCommand:
		{
			GetEventParameter(pEvent, kEventParamDirectObject, typeHICommand, nil, sizeof(HICommand), nil, &command);
			
			switch(command.commandID)
			{	 
				case 'quit':
					g_xPaperAPI.m_bRunning = false;
					return eventNotHandledErr;
			}
			
			break;
		}

		case kEventClassKeyboard:
		{
			switch(eventkind)
			{
				case kEventRawKeyDown:
				{
					GetEventParameter(pEvent, kEventParamKeyMacCharCodes, typeChar, nil, sizeof(charcode), nil, &charcode);
					g_xPaperAPI.m_xInput.HandleKeyboardKeyPressed(charcode, true);
					break;
				}		

				case kEventRawKeyUp:
				{
					GetEventParameter(pEvent, kEventParamKeyMacCharCodes, typeChar, nil, sizeof(charcode), nil, &charcode);
					g_xPaperAPI.m_xInput.HandleKeyboardKeyPressed(charcode, false);
					break;
				}		

				case kEventRawKeyModifiersChanged:
				{
					GetEventParameter(pEvent, kEventParamKeyModifiers, typeUInt32, nil, sizeof(modifiers), nil, &modifiers);
					
					// Command Key
					if((modifiers & cmdKey) != (oldmodifiers & cmdKey))
						g_xPaperAPI.m_xInput.HandleKeyboardKeyPressed(0x5B, modifiers & cmdKey);
					
					// Shift Key
					if((modifiers & shiftKey) != (oldmodifiers & shiftKey))
						g_xPaperAPI.m_xInput.HandleKeyboardKeyPressed(0xA0, modifiers & shiftKey);
					
					// Alt/Option Key
					if((modifiers & optionKey) != (oldmodifiers & optionKey))
						g_xPaperAPI.m_xInput.HandleKeyboardKeyPressed(0xA4, modifiers & optionKey);
					
					// Control Key
					if((modifiers & controlKey) != (oldmodifiers & controlKey))
						g_xPaperAPI.m_xInput.HandleKeyboardKeyPressed(0xA2, modifiers & controlKey);

					// Caps Lock
					if((modifiers & alphaLock) != (oldmodifiers & alphaLock))
						g_xPaperAPI.m_xInput.HandleKeyboardKeyPressed(0x14, modifiers & alphaLock);
					
					oldmodifiers = modifiers;
					
					break;
				}
			}
			
			break;
		}		

		case kEventClassMouse:
		{
			switch(eventkind)
			{
				case kEventMouseDown:
				{
					GetEventParameter(pEvent, kEventParamMouseButton, typeMouseButton, nil, sizeof(button), nil, &button);
					
					switch(button)
					{
						case kEventMouseButtonPrimary:
						{
							g_xPaperAPI.m_xInput.HandleMouseButtonPressed(0, true);
							break;
						}
			
						case kEventMouseButtonSecondary:
						{
							g_xPaperAPI.m_xInput.HandleMouseButtonPressed(1, true);
							break;
						}
			
						case kEventMouseButtonTertiary:
						{
							g_xPaperAPI.m_xInput.HandleMouseButtonPressed(2, true);
							break;
						}
					}
					
					break;
				}
					
				case kEventMouseUp:
				{
					GetEventParameter(pEvent, kEventParamMouseButton, typeMouseButton, nil, sizeof(button), nil, &button);
					
					switch(button)
					{
						case kEventMouseButtonPrimary:
						{
							g_xPaperAPI.m_xInput.HandleMouseButtonPressed(0, false);
							break;
						}
					
						case kEventMouseButtonSecondary:
						{
							g_xPaperAPI.m_xInput.HandleMouseButtonPressed(1, false);
							break;
						}
			
						case kEventMouseButtonTertiary:
						{
							g_xPaperAPI.m_xInput.HandleMouseButtonPressed(2, false);
							break;
						}
					}
					
					break;
				}
			
				case kEventMouseMoved:
				case kEventMouseDragged:
				{
					GetEventParameter(pEvent, kEventParamMouseLocation, typeHIPoint, nil, sizeof(mousecoord), nil, &mousecoord);
					g_xPaperAPI.m_xInput.m_lMouseX = (float)(mousecoord.x - g_xPaperAPI.m_xInput.m_lWindowOffsetX - g_xPaperAPI.m_iScreenOffsetX) / g_xPaperAPI.m_fScreenRatioX;
					g_xPaperAPI.m_xInput.m_lMouseY = (float)(mousecoord.y - g_xPaperAPI.m_xInput.m_lWindowOffsetY - g_xPaperAPI.m_iScreenOffsetY) / g_xPaperAPI.m_fScreenRatioY;

					break;
				}
			}
			
			break;
		}
	}
	
	return noErr;
/*	switch(msg)
	{
		case WM_PE_WSASYNC:
		{
			HandleAsyncNetworkMessage(&g_xPaperAPI.m_xNetwork, lparam);
			break;
		}

		case WM_PE_HTTPASYNC:
		{
			HandleAsyncNetworkMessage(&g_xPaperAPI.m_xHTTPNetwork, lparam);
			break;
		}

		case WM_PE_PINGASYNC:
		{
			HandleAsyncNetworkMessage(&g_xPaperAPI.m_xPingNetwork, lparam);
			break;
		}

		case WM_PE_FTPASYNC:
		{
			HandleAsyncNetworkMessage(&g_xPaperAPI.m_xFTPNetwork, lparam);
			break;
		}

		case WM_PE_FTPTRANSFERASYNC:
		{
			HandleAsyncNetworkMessage(&g_xPaperAPI.m_xFTPFileTransfer, lparam);
			break;
		}

		case WM_MOVE:
		{
			RECT tmp;
			GetWindowRect(g_xPaperAPI.m_hWnd, &tmp);
			g_xPaperAPI.m_xInput.m_lWindowOffsetX = tmp.left + g_xPaperAPI.m_xPlatform.m_iBorderSize;
			g_xPaperAPI.m_xInput.m_lWindowOffsetY = tmp.top  + g_xPaperAPI.m_xPlatform.m_iTitleBarHeight;
			g_xPaperAPI.m_bDragged = true;
			break;
		}

		case WM_ACTIVATE:
		{
			g_xPaperAPI.m_bHasFocus = wparam;
			break;
		}

		case WM_DESTROY:
		{
			g_xPaperAPI.m_bRunning = false;
			break;
		}

		case WM_SIZE:
		{
			g_xPaperAPI.m_bMinimized = wparam == SIZE_MINIMIZED;
			g_xPaperAPI.m_bHasFocus = !g_xPaperAPI.m_bMinimized;
			return DefWindowProc(hwnd, msg, wparam, lparam);
		}

		//Let Windows handle it, b/c we dont do anything with it
		default:
			return DefWindowProc(hwnd, msg, wparam, lparam);
	}
*/
}

void SetMainLoopEventTimer(EventLoopTimerProcPtr myMainLoopFunc)
{
	if(gMainLoopTimer != nil)
		return;
	
	EventLoopRef mainLoop = GetMainEventLoop();
	
	EventLoopTimerUPP eventLoopTimerUPP = NewEventLoopTimerUPP(myMainLoopFunc);
	
	InstallEventLoopTimer(mainLoop, kEventDurationNoWait, kEventDurationMillisecond, eventLoopTimerUPP, nil, &gMainLoopTimer);
	
	DisposeEventLoopTimerUPP(eventLoopTimerUPP);
}

static pascal void GameLoop(EventLoopTimerRef theTimer, void* userData)
{
	if(g_xPaperAPI.m_bHasFocus || g_xPaperAPI.m_iNetMode != PE_NETMODE_SINGLEPLAYER)
	{
		g_xPaperAPI.PreUpdate();
		GamePreUpdate(g_xPaperAPI.m_dwCurrentTime);
	}

	GameUpdate((double)(g_xPaperAPI.m_dwCurrentTime - g_xPaperAPI.m_dwLastTime) * 0.001);

	g_xPaperAPI.Update(); //Updates GameTime and Renders/Flips Screen

	g_xPaperAPI.m_xInput.PostUpdate();
	
	if(!g_xPaperAPI.m_bHasFocus)
		sleep(100);
}

void HandleAsyncNetworkMessage(xNetworkClient* pNetwork, DWORD lparam)
{
/*	switch(WSAGETSELECTEVENT(lparam))
	{
		case FD_CONNECT:
		{
			pNetwork->SetConnected(true);
			break;
		}

		case FD_READ:
		{
			if(int retvalu = WSAGETSELECTERROR(lparam))
			{
				if(retvalu != 0 && retvalu != 10035)
				{
					return;
				}
			}

			pNetwork->Read();
			break;
		}

		case FD_WRITE:
		{
			if(int retvalu = WSAGETSELECTERROR(lparam))
			{
				if(retvalu != 0 && retvalu != 10035)
				{
					return;
				}
			}

			pNetwork->SetWritable(true);
			break;
		}

		case FD_ACCEPT:
		{
			if(int retvalu = WSAGETSELECTERROR(lparam))
			{
				if(retvalu != 0 && retvalu != 10035)
				{
					return;
				}
			}

			if(!pNetwork->Accept())
				pNetwork->SetConnected(true);

			break;
		}

		case FD_CLOSE:
		{
			pNetwork->Close();
			break;
		}
	}
*/}
