/////////////////////////////////////////////////////////////////
// Name: xAudio.h
/////////////////////////////////////////////////////////////////
// Description:
// - Core audio interface, manages creation of Audio Streams and
//   Sounds, though it doesn't handle playing them
/////////////////////////////////////////////////////////////////
// - 7.6.2004, Update, Arto Ruotsalainen(Arex)
//   Comment stuff added to begin of the file.
//   Add new updates/fixes above this, so the old updates
//   will be at the bottom.
/////////////////////////////////////////////////////////////////
#ifndef XAUDIO_H_
#define XAUDIO_H_

#define PE_SOUNDSOURCE_NONE 999999999

#include "openal/al.h"
#include "openal/alc.h"
#include "vorbis/vorbisfile.h"
#include "xResults.h"
#include "xSound.h"
#include "xAudioStream.h"

enum xAudioSources
{
	PE_SOURCE_MUSIC = 0,
	PE_SOURCE_FIRSTSOUND,
	PE_SOURCE_MAX = 128
};

class xAudio
{
public:
	xAudio()
	{
		m_pDevice				= 0;
		m_pContext				= 0;
		m_pSources				= 0;
		m_iMaxSourcesOverride	= 0;
	}

	int Init();
	void Shutdown();

	xSound*       LoadSound(const char* strPathName);
	xSound*       LoadSoundWithRandom(const char* strPathName, bool bDontRepeat = 0);
	xAudioStream* LoadMusic(const char* strPathName, bool bLooping = 0, bool bAnalyzing = 0);
	void          UnloadSound(xSound* pSound);
	void          UnloadMusic(xAudioStream* pMusic);

	ALuint GetSource(bool bStream);

	void StopAllSounds();

	int m_iMaxSourcesOverride;

protected:
	xPool<xSound>	m_xSoundPool;

	// Sources Pool
	ALuint	*m_pSources;
	int		 m_iMaxSources;
	int		 m_iNextSoundSource;

	ALCdevice	*m_pDevice;
	ALCcontext	*m_pContext;

	int m_iResult;
};

#endif