#include "Paper.h"

xEmitter g_xEmitterPool;

void xParticle::Update(double dTime)
{
	m_dCurLife += dTime;
	if(m_dCurLife > m_dLifespan)
	{
		if(m_fFadeOutTime >= m_fScaleOutTime)
		{
			if(m_dCurLife > m_dLifespan + m_fFadeOutTime)
			{
				xRenderable2DQuadUpdateVisibility(m_pRenderable, 0);
				m_bDead = true;
				return;
			}

			if(m_fFadeOutTime - (m_dCurLife - m_dLifespan) < m_fScaleOutTime)
			{
				m_fScale = m_fFullScale * ((m_fFadeOutTime - (m_dCurLife - m_dLifespan)) / m_fScaleOutTime);
				xRenderable2DQuadSetScale(m_pRenderable, m_fScale, m_fScale);
			}

			xRenderable2DQuadUpdateAlpha(m_pRenderable, m_fFullAlpha * (1.0 - ((m_dCurLife - m_dLifespan) / m_fFadeOutTime)));
		}
		else
		{
			if(m_dCurLife > m_dLifespan + m_fScaleOutTime)
			{
				xRenderable2DQuadUpdateVisibility(m_pRenderable, 0);
				m_bDead = true;
				return;
			}

			if(m_fScaleOutTime - (m_dCurLife - m_dLifespan) < m_fFadeOutTime)
				xRenderable2DQuadUpdateAlpha(m_pRenderable, m_fFullAlpha * ((m_fScaleOutTime - (m_dCurLife - m_dLifespan)) / m_fFadeOutTime));

			m_fScale = m_fFullScale * (1.0 - ((m_dCurLife - m_dLifespan) / m_fScaleOutTime));
			xRenderable2DQuadSetScale(m_pRenderable, m_fScale, m_fScale);
		}
	}
	else
	{
		if(m_dCurLife < m_fFadeInTime)
			m_xColor.A = m_fFullAlpha * (m_dCurLife / m_fFadeInTime);
		else
			m_xColor.A = m_fFullAlpha;

		if(m_dCurLife < m_fScaleInTime)
		{
			m_fScale = m_fStartingScale + (m_fFullScale - m_fStartingScale) * (m_dCurLife / m_fScaleInTime);
			xRenderable2DQuadSetScale(m_pRenderable, m_fScale, m_fScale);
		}
		else
			xRenderable2DQuadSetScale(m_pRenderable, m_fFullScale, m_fFullScale);

		m_xColor.R = m_xStartColor.R + ((m_xEndColor.R - m_xStartColor.R)) * (m_dCurLife / m_dLifespan);
		m_xColor.G = m_xStartColor.G + ((m_xEndColor.G - m_xStartColor.G)) * (m_dCurLife / m_dLifespan);
		m_xColor.B = m_xStartColor.B + ((m_xEndColor.B - m_xStartColor.B)) * (m_dCurLife / m_dLifespan);

		xRenderable2DQuadUpdateColor(m_pRenderable, &m_xColor);
	}

	m_xVelocity.y += m_fGravity * dTime * 30.0;

	m_fX += m_xVelocity.x * dTime * 30.0;
	m_fY += m_xVelocity.y * dTime * 30.0;

	m_fRotation += m_fRotationSpeed * dTime * 30.0;

	xRenderable2DQuadSetPosition(m_pRenderable, m_fX, m_fY);
	xRenderable2DQuadSetRotation(m_pRenderable, m_fRotation);
	xRenderable2DQuadUpdate(m_pRenderable);
}

void xParticleSystem::Init(xTexture *pTexture, float fDepth)
{
	int i;

	m_dNextSpawnTime = 0.0;

	m_dMinSpawnTime = (float)(256 - m_iMaxParticlesPerSecond) / 255.0;
	m_dMaxSpawnTime = (float)(256 - m_iMinParticlesPerSecond) / 255.0;

	if(m_fMaxScaleOutTime >= m_fMaxFadeOutTime)
		m_iMaxParticles = ceilf((float)m_iMaxParticlesPerSecond * (m_dMaxLifespan + m_fMaxScaleOutTime));
	else
		m_iMaxParticles = ceilf((float)m_iMaxParticlesPerSecond * (m_dMaxLifespan + m_fMaxFadeOutTime));

	if(g_xPaperAPI.m_bLimitParticles && m_iMaxParticles > 50)
	{
		m_iMaxParticlesPerSecond = (float)m_iMaxParticlesPerSecond / ((float)m_iMaxParticles / 50.0f);
		m_iMaxParticles = 50;
	}

	if(m_pSceneGraph)
	{
		m_pParticles	= new xParticle[m_iMaxParticles];
		m_iCurParticle	= 0;

		for(i = 0; i < m_iMaxParticles; i++)
		{
			m_pParticles[i].m_pRenderable = xSceneGraphCreateQuad(pTexture);
			xSceneGraphInsertQuad(m_pSceneGraph, m_pParticles[i].m_pRenderable, 1, fDepth, 0, m_bAnimated);
			xRenderable2DQuadUpdateVisibility(m_pParticles[i].m_pRenderable, 0);

			xColorInit(&m_pParticles[i].m_xColor, 1.0, 1.0, 1.0, 1.0);
			xColorInit(&m_pParticles[i].m_xStartColor, 1.0, 1.0, 1.0, 1.0);
			xColorInit(&m_pParticles[i].m_xEndColor, 1.0, 1.0, 1.0, 1.0);
		}
	}
}

void xParticleSystem::InitAnimated(xTexture **pTextures, float fDepth, int iNumTextures, int iFPS)
{
	m_bAnimated = 1;
	m_pTextures = pTextures;
	m_iNumFrames = iNumTextures;
	m_iFPS = iFPS;

	Init(pTextures[0], fDepth);
}

void xParticleSystem::CopyParticleSettings(xParticleSystem *pCopy)
{
	m_iMinParticlesPerSecond = pCopy->m_iMinParticlesPerSecond;
	m_iMaxParticlesPerSecond = pCopy->m_iMaxParticlesPerSecond;
	
	m_dMinLifespan = pCopy->m_dMinLifespan;
	m_dMaxLifespan = pCopy->m_dMaxLifespan;
	
	m_fMinDirection = pCopy->m_fMinDirection;
	m_fMaxDirection = pCopy->m_fMaxDirection;
	m_fMinDirection2 = pCopy->m_fMinDirection2;
	m_fMaxDirection2 = pCopy->m_fMaxDirection2;
	m_fMinDirectionSpeed = pCopy->m_fMinDirectionSpeed;
	m_fMaxDirectionSpeed = pCopy->m_fMaxDirectionSpeed;
	
	m_fMinSpeed = pCopy->m_fMinSpeed;
	m_fMaxSpeed = pCopy->m_fMaxSpeed;
	
	m_fMinFadeOutTime = pCopy->m_fMinFadeOutTime;
	m_fMaxFadeOutTime = pCopy->m_fMaxFadeOutTime;
	m_fMinFadeInTime = pCopy->m_fMinFadeInTime;
	m_fMaxFadeInTime = pCopy->m_fMaxFadeInTime;
	
	m_fMinGravity = pCopy->m_fMinGravity;
	m_fMaxGravity = pCopy->m_fMaxGravity;
	
	m_fMinStartingScale = pCopy->m_fMinStartingScale;
	m_fMaxStartingScale = pCopy->m_fMaxStartingScale;
	m_fMinScale = pCopy->m_fMinScale;
	m_fMaxScale = pCopy->m_fMaxScale;
	
	m_fMinScaleInTime = pCopy->m_fMinScaleInTime;
	m_fMaxScaleInTime = pCopy->m_fMaxScaleInTime;
	m_fMinScaleOutTime = pCopy->m_fMinScaleOutTime;
	m_fMaxScaleOutTime = pCopy->m_fMaxScaleOutTime;
	
	m_fMinRotation = pCopy->m_fMinRotation;
	m_fMaxRotation = pCopy->m_fMaxRotation;
	
	m_fMinRotationSpeed = pCopy->m_fMinRotationSpeed;
	m_fMaxRotationSpeed = pCopy->m_fMaxRotationSpeed;
	
	m_fMinFullAlpha = pCopy->m_fMinFullAlpha;
	m_fMaxFullAlpha = pCopy->m_fMaxFullAlpha;
	
	m_fMinStartX = pCopy->m_fMinStartX;
	m_fMaxStartX = pCopy->m_fMaxStartX;
	m_fMinStartY = pCopy->m_fMinStartY;
	m_fMaxStartY = pCopy->m_fMaxStartY;

	m_bColorRanges = pCopy->m_bColorRanges;
	m_bColorTransitions = pCopy->m_bColorTransitions;
	m_iColorArrayLength = pCopy->m_iColorArrayLength;

	m_pColorArray = new xColor[m_iColorArrayLength];
	for(int i = 0; i < m_iColorArrayLength; i++)
		xColorCopy(&m_pColorArray[i], &pCopy->m_pColorArray[i]);
}

void xParticleSystem::Update(double dTime)
{
	if(m_dNextSpawnTime == 0.0)
		m_dNextSpawnTime = m_dCurTime;
	else
		m_bIsDone = true;

	for(int i = 0; i < m_iMaxParticles; i++)
	{
		if(!m_pParticles[i].m_bDead)
		{
			m_pParticles[i].Update(dTime);
			m_bIsDone = false;

			if(m_bAnimated)
			{
				m_pParticles[i].m_dFrameTime += dTime;
				if(m_pParticles[i].m_dFrameTime >= (1.0 / m_iFPS))
				{
					m_pParticles[i].m_iCurFrame++;

					if(m_pParticles[i].m_iCurFrame >= m_iNumFrames)
						m_pParticles[i].m_iCurFrame = 0;

					xRenderable2DQuadUpdateTexture(m_pParticles[i].m_pRenderable, m_pTextures[m_pParticles[i].m_iCurFrame]);

					m_pParticles[i].m_dFrameTime -= (1.0 / m_iFPS);
				}
			}
		}
	}

	m_dCurTime += dTime;

	//DEBUGLOG("Update: %f %f %f\n", dTime, m_dCurTime, m_dNextSpawnTime);

	if(m_bEnabled)
	{
		while(m_dCurTime >= m_dNextSpawnTime)
		{
			CreateNextParticle();
#if _DEBUG && 0
			double additionaltime = m_dMinSpawnTime + (((double)rand() / (double)RAND_MAX) * (m_dMaxSpawnTime - m_dMinSpawnTime));
			DEBUGLOG("\nNext Particle: %f %f %f\n", m_dMinSpawnTime, m_dMaxSpawnTime, additionaltime);
			m_dNextSpawnTime += additionaltime;
#else
			m_dNextSpawnTime += m_dMinSpawnTime + (((double)rand() / (double)RAND_MAX) * (m_dMaxSpawnTime - m_dMinSpawnTime));
#endif
		}
	}
}

void xParticleSystem::Delete()
{
	int i;

	for(i = 0; i < m_iMaxParticles; i++)
		xSceneGraphDeleteQuad(m_pSceneGraph, m_pParticles[i].m_pRenderable, 0);

	delete [] m_pParticles;
}

void xParticleSystem::CreateNextParticle(float fX, float fY)
{
	if(fX == -99999.0)
	{
		m_pParticles[m_iCurParticle].m_fX = m_fX + m_fWidth * (m_fMinStartX + (((float)rand() / (float)RAND_MAX) * (m_fMaxStartX - m_fMinStartX)));
		m_pParticles[m_iCurParticle].m_fY = m_fY + m_fHeight * (m_fMinStartY + (((float)rand() / (float)RAND_MAX) * (m_fMaxStartY - m_fMinStartY)));
	}
	else
	{
		m_pParticles[m_iCurParticle].m_fX = fX + m_fWidth * (m_fMinStartX + (((float)rand() / (float)RAND_MAX) * (m_fMaxStartX - m_fMinStartX)));
		m_pParticles[m_iCurParticle].m_fY = fY + m_fHeight * (m_fMinStartY + (((float)rand() / (float)RAND_MAX) * (m_fMaxStartY - m_fMinStartY)));
	}

	m_pParticles[m_iCurParticle].m_bDead			= false;
	m_pParticles[m_iCurParticle].m_dCurLife			= 0.0;
	m_pParticles[m_iCurParticle].m_iCurFrame		= 0;
	m_pParticles[m_iCurParticle].m_dFrameTime		= 0.0;
	m_pParticles[m_iCurParticle].m_dLifespan		= m_dMinLifespan + (((double)rand() / (double)RAND_MAX) * (m_dMaxLifespan - m_dMinLifespan));
	m_pParticles[m_iCurParticle].m_fFadeInTime		= m_fMinFadeInTime + (((float)rand() / (float)RAND_MAX) * (m_fMaxFadeInTime - m_fMinFadeInTime));
	m_pParticles[m_iCurParticle].m_fFadeOutTime		= m_fMinFadeOutTime + (((float)rand() / (float)RAND_MAX) * (m_fMaxFadeOutTime - m_fMinFadeOutTime));
	m_pParticles[m_iCurParticle].m_xVelocity		= CreateVelocityVector();
	m_pParticles[m_iCurParticle].m_fGravity			= m_fMinGravity + (((float)rand() / (float)RAND_MAX) * (m_fMaxGravity - m_fMinGravity));
	m_pParticles[m_iCurParticle].m_fStartingScale	= m_fMinStartingScale + (((float)rand() / (float)RAND_MAX) * (m_fMaxStartingScale - m_fMinStartingScale));
	m_pParticles[m_iCurParticle].m_fFullScale		= m_fMinScale + (((float)rand() / (float)RAND_MAX) * (m_fMaxScale - m_fMinScale));
	m_pParticles[m_iCurParticle].m_fScaleInTime		= m_fMinScaleInTime + (((float)rand() / (float)RAND_MAX) * (m_fMaxScaleInTime - m_fMinScaleInTime));
	m_pParticles[m_iCurParticle].m_fScaleOutTime	= m_fMinScaleOutTime + (((float)rand() / (float)RAND_MAX) * (m_fMaxScaleOutTime - m_fMinScaleOutTime));
	m_pParticles[m_iCurParticle].m_fRotation		= m_fMinRotation + (((float)rand() / (float)RAND_MAX) * (m_fMaxRotation - m_fMinRotation));
	m_pParticles[m_iCurParticle].m_fRotationSpeed	= m_fMinRotationSpeed + (((float)rand() / (float)RAND_MAX) * (m_fMaxRotationSpeed - m_fMinRotationSpeed));
	m_pParticles[m_iCurParticle].m_fFullAlpha		= m_fMinFullAlpha + (((float)rand() / (float)RAND_MAX) * (m_fMaxFullAlpha - m_fMinFullAlpha));

	// Convert Rotations to Degrees
	m_pParticles[m_iCurParticle].m_fRotation		*= 180.0f / 3.141593f;
	m_pParticles[m_iCurParticle].m_fRotationSpeed	*= 180.0f / 3.141593f;

	if(m_iColorArrayLength > 0)
	{
		if(m_bColorRanges)
		{
			if(m_bColorTransitions)
			{
				int index;
				
				if(m_iColorIndex < 0)
				{
					index = rand() % m_iColorArrayLength;
					index -= index % 4;
				}
				else
					index = m_iColorIndex;

				m_pParticles[m_iCurParticle].m_xStartColor	= GenerateColorInRange(m_pColorArray[index], m_pColorArray[index + 1]);
				m_pParticles[m_iCurParticle].m_xEndColor	= GenerateColorInRange(m_pColorArray[index + 2], m_pColorArray[index + 3]);
			}
			else
			{
				int index;
				
				if(m_iColorIndex < 0)
				{
					index = rand() % m_iColorArrayLength;
					index -= index % 2;
				}
				else
					index = m_iColorIndex;

				m_pParticles[m_iCurParticle].m_xStartColor	= GenerateColorInRange(m_pColorArray[index], m_pColorArray[index + 1]);
				m_pParticles[m_iCurParticle].m_xEndColor	= m_pParticles[m_iCurParticle].m_xStartColor;
			}
		}
		else
		{
			if(m_bColorTransitions)
			{
				int index;
				
				if(m_iColorIndex < 0)
				{
					index = rand() % m_iColorArrayLength;
					index -= index % 2;
				}
				else
					index = m_iColorIndex;

				m_pParticles[m_iCurParticle].m_xStartColor	= m_pColorArray[index];
				m_pParticles[m_iCurParticle].m_xEndColor	= m_pColorArray[index + 1];
			}
			else
			{
				int index;
				
				if(m_iColorIndex < 0)
					index = rand() % m_iColorArrayLength;
				else
					index = m_iColorIndex;

				m_pParticles[m_iCurParticle].m_xStartColor	= m_pColorArray[index];
				m_pParticles[m_iCurParticle].m_xEndColor	= m_pParticles[m_iCurParticle].m_xStartColor;
			}
		}
	}

	xRenderable2DQuadUpdateVisibility(m_pParticles[m_iCurParticle].m_pRenderable, 1);

	m_pParticles[m_iCurParticle].Update(0.0);

	if(m_iCurParticle >= m_iMaxParticles - 1)
		m_iCurParticle = 0;
	else
		m_iCurParticle++;
}

void xParticleSystem::UpdatedSpawnTimes()
{
	m_dMinSpawnTime = (float)(256 - m_iMaxParticlesPerSecond) / 255.0;
	m_dMaxSpawnTime = (float)(256 - m_iMinParticlesPerSecond) / 255.0;

	int		   oldlength = m_iMaxParticles;
	xParticle *oldarray	 = m_pParticles;

	if(m_fMaxScaleOutTime >= m_fMaxFadeOutTime)
		m_iMaxParticles = ceil((float)m_iMaxParticlesPerSecond * (m_dMaxLifespan + m_fMaxScaleOutTime));
	else
		m_iMaxParticles = ceil((float)m_iMaxParticlesPerSecond * (m_dMaxLifespan + m_fMaxFadeOutTime));

	m_pParticles = new xParticle[m_iMaxParticles];

	if(oldlength > 0)
	{
		if(oldlength <= m_iMaxParticles)
			memcpy(m_pParticles, oldarray, sizeof(xParticle) * oldlength);
		else
		{
			memcpy(m_pParticles, oldarray, sizeof(xParticle) * m_iMaxParticles);

			if(m_iCurParticle >= m_iMaxParticles)
				m_iCurParticle = 0;
		}
	}

	//DEBUGLOG("UpdatedSpawnTimes: %f %f\n", m_dMinSpawnTime, m_dMaxSpawnTime);
}

xVector xParticleSystem::CreateVelocityVector()
{
	float speed = m_fMinSpeed + (((float)rand() / (float)RAND_MAX) * (m_fMaxSpeed - m_fMinSpeed));
	
	if(m_fMaxDirectionSpeed > 0.0)
	{
		//DEBUGLOG("CreateVelocityVector: %f %f %f %f %f %i\n", m_fMinDirectionSpeed, m_fMaxDirectionSpeed, m_fLastDirection, m_fDirection1, m_fDirection2, m_bMovingFromDirection1to2);

		if(m_fLastDirection > 0.0 || m_fDirection1 > 0.0 || m_fDirection2 > 0.0)
		{
			m_fDirection1 = m_fMinDirection + (((float)rand() / (float)RAND_MAX) * (m_fMaxDirection - m_fMinDirection));
			m_fDirection2 = m_fMinDirection2 + (((float)rand() / (float)RAND_MAX) * (m_fMaxDirection2 - m_fMinDirection2));

			if(m_fDirection1 < m_fDirection2)
			{
				m_fLastDirection = m_fDirection2;
				m_fDirection2 = m_fDirection1;
				m_fDirection1 = m_fLastDirection;
			}
			else
				m_fLastDirection = m_fDirection1;
		}
		else if(m_bMovingFromDirection1to2)
		{
			m_fLastDirection -= m_fMinDirectionSpeed + (((float)rand() / (float)RAND_MAX) * (m_fMaxDirectionSpeed - m_fMinDirectionSpeed));

			if(m_fLastDirection < m_fDirection2)
			{
				m_bMovingFromDirection1to2 = false;
				m_fDirection1 = m_fMinDirection + (((float)rand() / (float)RAND_MAX) * (m_fMaxDirection - m_fMinDirection));
				
				if(m_fDirection1 < m_fDirection2)
				{
					m_fLastDirection = m_fDirection2;
					m_fDirection2 = m_fDirection1;
					m_fDirection1 = m_fLastDirection;
				}
				else
					m_fLastDirection = m_fDirection2;
			}
		}
		else
		{
			m_fLastDirection += m_fMinDirectionSpeed + (((float)rand() / (float)RAND_MAX) * (m_fMaxDirectionSpeed - m_fMinDirectionSpeed));

			if(m_fLastDirection > m_fDirection1)
			{
				m_bMovingFromDirection1to2 = true;
				m_fDirection2 = m_fMinDirection2 + (((float)rand() / (float)RAND_MAX) * (m_fMaxDirection2 - m_fMinDirection2));

				if(m_fDirection2 > m_fDirection1)
				{
					m_fLastDirection = m_fDirection1;
					m_fDirection1 = m_fDirection2;
					m_fDirection2 = m_fLastDirection;
				}
				else
					m_fLastDirection = m_fDirection1;
			}
		}
	}
	else if(m_fMinDirection2 > -6.29 && m_fMaxDirection2 > -6.29 && rand() <= RAND_MAX / 2)
		m_fLastDirection = m_fMinDirection2 + (((float)rand() / (float)RAND_MAX) * (m_fMaxDirection2 - m_fMinDirection2));
	else
		m_fLastDirection = m_fMinDirection + (((float)rand() / (float)RAND_MAX) * (m_fMaxDirection - m_fMinDirection));

	return xVector(cos(m_fLastDirection) * speed, sin(m_fLastDirection) * speed, 0.0);
}

xColor xParticleSystem::GenerateColorInRange(xColor &xColor1, xColor &xColor2)
{
	xColor ret;

	if(xColor1.R > xColor2.R)
		ret.R = xColor2.R + (((float)rand() / (float)RAND_MAX) * (xColor1.R - xColor2.R));
	else
		ret.R = xColor1.R + (((float)rand() / (float)RAND_MAX) * (xColor2.R - xColor1.R));

	if(xColor1.G > xColor2.G)
		ret.G = xColor2.G + (((float)rand() / (float)RAND_MAX) * (xColor1.G - xColor2.G));
	else
		ret.G = xColor1.G + (((float)rand() / (float)RAND_MAX) * (xColor2.G - xColor1.G));

	if(xColor1.B > xColor2.B)
		ret.B = xColor2.B + (((float)rand() / (float)RAND_MAX) * (xColor1.B - xColor2.B));
	else
		ret.B = xColor1.B + (((float)rand() / (float)RAND_MAX) * (xColor2.B - xColor1.B));

	if(xColor1.A > xColor2.A)
		ret.A = xColor2.A + (((float)rand() / (float)RAND_MAX) * (xColor1.A - xColor2.A));
	else
		ret.A = xColor1.A + (((float)rand() / (float)RAND_MAX) * (xColor2.A - xColor1.A));

	return ret;
}

void xEmitter::Init(const char *strFilePath, float fDepth, bool bAnimated, xSceneGraph *pSceneGraph)
{
	xEmitter *curemitter, *copyemitter = 0;
	char strFile[256];
	xFileIO objectfile;
	xP2DFileHeader fileheader;
	xP2DSpriteHeader spriteheader;
	xParticleSystemList *curparticle, *copyparticle;
	xParticleSystem unusedparticle;
	RECT temprect;
	uint8_t exptype;
	int fps, i;

	curemitter = &g_xEmitterPool;
	while(curemitter->m_pPoolNext)
	{
		if(!stricmp(curemitter->m_pPoolNext->m_strObjectPath, strFilePath))
		{
			copyemitter = curemitter->m_pPoolNext;
			break;
		}

		curemitter = curemitter->m_pPoolNext;
	}

	if(copyemitter)
	{
		m_pTextures = copyemitter->m_pTextures;

		i = 0;
		curparticle = &m_xParticleList;
		copyparticle = copyemitter->m_xParticleList.m_pNext;

		while(copyparticle)
		{
			curparticle->m_pNext				= new xParticleSystemList;
			curparticle->m_pNext->m_pPrev		= curparticle;
			curparticle							= curparticle->m_pNext;
			curparticle->m_pData				= new xParticleSystem;
			curparticle->m_pData->m_pSceneGraph	= pSceneGraph;
			curparticle->m_pData->m_fX			= m_fX;
			curparticle->m_pData->m_fY			= m_fY;
			curparticle->m_pData->m_fWidth		= m_fWidth;
			curparticle->m_pData->m_fHeight		= m_fHeight;
			curparticle->m_pData->m_bAnimated	= copyparticle->m_pData->m_bAnimated;
			curparticle->m_pData->m_iFPS		= copyparticle->m_pData->m_iFPS;
			curparticle->m_pData->CopyParticleSettings(copyparticle->m_pData);

			if(bAnimated)
				curparticle->m_pData->InitAnimated(m_pTextures, fDepth, copyparticle->m_pData->m_iNumFrames, copyparticle->m_pData->m_iFPS);
			else
				curparticle->m_pData->Init(m_pTextures[i++], fDepth);

			copyparticle = copyparticle->m_pNext;
		}

		copyemitter->m_iReferenceCount++;
	}
	else
	{
		curemitter->m_pPoolNext = this;
		m_pPoolPrev = curemitter;
		strcpy(m_strObjectPath, strFilePath);

		if(objectfile.OpenFile(strFilePath, true, true))
			assert(0);

		if(objectfile.ReadData(&fileheader, sizeof(xP2DFileHeader)))
			assert(0);

		m_pTextures = (xTexture**)dlmalloc(sizeof(xTexture*) * fileheader.m_iNumSprites);

		if(bAnimated)
		{
			m_xParticleList.m_pNext							= new xParticleSystemList;
			m_xParticleList.m_pNext->m_pPrev				= &m_xParticleList;
			m_xParticleList.m_pNext->m_pData				= new xParticleSystem;
			m_xParticleList.m_pNext->m_pData->m_pSceneGraph	= pSceneGraph;
			m_xParticleList.m_pNext->m_pData->m_fX			= m_fX;
			m_xParticleList.m_pNext->m_pData->m_fY			= m_fY;
			m_xParticleList.m_pNext->m_pData->m_fWidth		= m_fWidth;
			m_xParticleList.m_pNext->m_pData->m_fHeight		= m_fHeight;

			for(i = 0; i < fileheader.m_iNumSprites; i++)
			{
				if(objectfile.ReadData(&spriteheader, sizeof(xP2DSpriteHeader)))
					assert(0);

				if(i == 0)
					fps = spriteheader.m_iFPS;

				while(spriteheader.m_iNumExtensions > 0)
				{
					objectfile.ReadData(&exptype, sizeof(uint8_t));

					switch(exptype)
					{
						case PE_P2D_EXTTYPE_PARTICLE:
							if(i == 0)
								LoadParticleParameters(m_xParticleList.m_pNext->m_pData, &objectfile);
							else
								LoadParticleParameters(&unusedparticle, &objectfile);

							break;
					}

					spriteheader.m_iNumExtensions--;
				}

				sprintf(strFile, "%s%i", strFilePath, i);
				m_pTextures[i] = xGraphicsTexturePoolNew(strFile);

				if(!m_pTextures[i]->m_bLoaded)
				{
					xTextureCreateTexture(m_pTextures[i], &objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);
					strcpy(m_pTextures[i]->m_strObjectPath, strFile);
					m_pTextures[i]->m_bLoaded = true;
				}
				else
					xTextureSkipTexture(&objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);

				objectfile.ReadData(&temprect, sizeof(RECT));
			}

			m_xParticleList.m_pNext->m_pData->InitAnimated(m_pTextures, fDepth, fileheader.m_iNumSprites, fps);
		}
		else
		{
			curparticle = &m_xParticleList;

			for(i = 0; i < fileheader.m_iNumSprites; i++)
			{
				curparticle->m_pNext			= new xParticleSystemList;
				curparticle->m_pNext->m_pPrev	= curparticle;
				curparticle						= curparticle->m_pNext;

				curparticle->m_pData				= new xParticleSystem;
				curparticle->m_pData->m_pSceneGraph	= pSceneGraph;
				curparticle->m_pData->m_fX			= m_fX;
				curparticle->m_pData->m_fY			= m_fY;
				curparticle->m_pData->m_fWidth		= m_fWidth;
				curparticle->m_pData->m_fHeight		= m_fHeight;

				if(objectfile.ReadData(&spriteheader, sizeof(xP2DSpriteHeader)))
					assert(0);

				while(spriteheader.m_iNumExtensions > 0)
				{
					objectfile.ReadData(&exptype, sizeof(uint8_t));

					switch(exptype)
					{
						case 0:
							LoadParticleParameters(curparticle->m_pData, &objectfile);
							break;
					}

					spriteheader.m_iNumExtensions--;
				}

				sprintf(strFile, "%s%i", strFilePath, i);
				m_pTextures[i] = xGraphicsTexturePoolNew(strFile);

				if(!m_pTextures[i]->m_bLoaded)
				{
					xTextureCreateTexture(m_pTextures[i], &objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);
					strcpy(m_pTextures[i]->m_strObjectPath, strFile);
					m_pTextures[i]->m_bLoaded = true;
				}
				else
					xTextureSkipTexture(&objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);

				objectfile.ReadData(&temprect, sizeof(RECT));

				curparticle->m_pData->Init(m_pTextures[i], fDepth);
			}
		}

		objectfile.CloseFile();
	}

	m_dLifeTime = 0.0;
	m_bIsLoaded = 1;
}

void xEmitter::Update(double dTime)
{
	m_bIsDone = 1;

	m_dLifeTime += dTime;

	xParticleSystemList *curparticle = m_xParticleList.m_pNext;
	while(curparticle)
	{
		if(!curparticle->m_pData->m_bIsDone)
		{
			curparticle->m_pData->Update(dTime);
			m_bIsDone = false;
		}

		curparticle = curparticle->m_pNext;
	}
}

void xEmitter::RefreshRenderables(const char *strFilePath)
{
	xParticleSystemList *curparticle;
	int i;

	//xGraphicsGetTextures(strFilePath, &m_pTextures);

	curparticle = m_xParticleList.m_pNext;
	while(curparticle)
	{
		for(i = 0; i < curparticle->m_pData->m_iMaxParticles; i++)
		{
			if(curparticle->m_pData->m_pParticles[i].m_pRenderable)
			{
				xRenderable2DQuadUpdateTexture(curparticle->m_pData->m_pParticles[i].m_pRenderable, curparticle->m_pData->m_pParticles[i].m_pRenderable->m_pTextures[0]);
				xRenderable2DQuadUpdate(curparticle->m_pData->m_pParticles[i].m_pRenderable);
			}
		}

		curparticle = curparticle->m_pNext;
	}
}

void xEmitter::Reset()
{
	xParticleSystemList *curparticle = m_xParticleList.m_pNext;

	while(curparticle)
	{
		curparticle->m_pData->m_bEnabled = true;
		curparticle->m_pData->m_bIsDone = false;
		curparticle->m_pData->m_dNextSpawnTime = 0.0;

		for(int i = 0; i < curparticle->m_pData->m_iMaxParticles; i++)
		{
			curparticle->m_pData->m_pParticles[i].m_bDead = true;
			xRenderable2DQuadUpdateVisibility(curparticle->m_pData->m_pParticles[i].m_pRenderable, 0);
		}

		curparticle = curparticle->m_pNext;
	}

	m_dLifeTime = 0.0;
	m_bIsDone = false;
}

void xEmitter::Kill()
{
	xParticleSystemList *curparticle = m_xParticleList.m_pNext;

	while(curparticle)
	{
		curparticle->m_pData->m_bEnabled = false;
		curparticle = curparticle->m_pNext;
	}
}

void xEmitter::SetPosition(float fX, float fY)
{
	if(m_fWidth != 0.0)
		fX -= m_fWidth / 2.0;

	if(m_fHeight != 0.0)
		fY -= m_fHeight / 2.0;

	m_fX = fX;
	m_fY = fY;

	xParticleSystemList *curparticle = m_xParticleList.m_pNext;
	while(curparticle)
	{
		curparticle->m_pData->m_fX = m_fX;
		curparticle->m_pData->m_fY = m_fY;
		curparticle = curparticle->m_pNext;
	}
}

void xEmitter::SetSize(float fWidth, float fHeight)
{
	if(m_fWidth != 0.0)
		m_fX += m_fWidth / 2.0;

	if(m_fHeight != 0.0)
		m_fY += m_fHeight / 2.0;

	m_fWidth = fWidth;
	m_fHeight = fHeight;

	if(m_fWidth != 0.0)
		m_fX -= m_fWidth / 2.0;

	if(m_fHeight != 0.0)
		m_fY -= m_fHeight / 2.0;

	xParticleSystemList *curparticle = m_xParticleList.m_pNext;
	while(curparticle)
	{
		curparticle->m_pData->m_fX = m_fX;
		curparticle->m_pData->m_fY = m_fY;
		curparticle->m_pData->m_fWidth = m_fWidth;
		curparticle->m_pData->m_fHeight = m_fHeight;
		curparticle = curparticle->m_pNext;
	}
}

void xEmitter::SetColorIndex(int iIndex)
{
	xParticleSystemList *curparticle = m_xParticleList.m_pNext;
	while(curparticle)
	{
		if(curparticle->m_pData->m_bColorRanges && curparticle->m_pData->m_bColorTransitions)
		{
			if(iIndex < curparticle->m_pData->m_iColorArrayLength / 4)
				curparticle->m_pData->m_iColorIndex = iIndex * 4;
		}
		else if(curparticle->m_pData->m_bColorRanges || curparticle->m_pData->m_bColorTransitions)
		{
			if(iIndex < curparticle->m_pData->m_iColorArrayLength / 2)
				curparticle->m_pData->m_iColorIndex = iIndex * 2;
		}
		else
		{
			if(iIndex < curparticle->m_pData->m_iColorArrayLength)
				curparticle->m_pData->m_iColorIndex = iIndex;
		}

		curparticle = curparticle->m_pNext;
	}
}

void xEmitter::Delete()
{
	xParticleSystemList *curparticle = m_xParticleList.m_pNext;
	while(curparticle)
	{
		xParticleSystemList *oldparticle = curparticle;
		
		curparticle->m_pData->Delete();
		curparticle = curparticle->m_pNext;

		delete oldparticle->m_pData;
		delete oldparticle;
	}
}

void xEmitter::LoadParticleParameters(xParticleSystem *pParticleSystem, xFileIO *pFile)
{
	uint8_t		version;
	uint8_t		tempint8;
	uint16_t	tempint16;

	// Read the Version
	pFile->ReadData((void*)&version, sizeof(uint8_t));

	// Read the Particle Properties
	pFile->ReadData((void*)&tempint8, sizeof(uint8_t));
	pParticleSystem->m_iMinParticlesPerSecond = tempint8;

	pFile->ReadData((void*)&tempint8, sizeof(uint8_t));
	pParticleSystem->m_iMaxParticlesPerSecond = tempint8;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_dMinLifespan = ((double)tempint16 / 65535.0) * 10.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_dMaxLifespan = ((double)tempint16 / 65535.0) * 10.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinDirection = ((float)tempint16 / 65535.0) * -6.28319;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxDirection = ((float)tempint16 / 65535.0) * -6.28319;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinDirection2 = ((float)tempint16 / 65535.0) * -6.3006433;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxDirection2 = ((float)tempint16 / 65535.0) * -6.3006433;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinDirectionSpeed = ((float)tempint16 / 65535.0) * 1.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxDirectionSpeed = ((float)tempint16 / 65535.0) * 1.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinSpeed = ((double)tempint16 / 65535.0) * 20.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxSpeed = ((double)tempint16 / 65535.0) * 20.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinFadeOutTime = ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxFadeOutTime = ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinFadeInTime = ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxFadeInTime = ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinGravity = ((double)tempint16 / 65535.0) * 1.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxGravity = ((double)tempint16 / 65535.0) * 1.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinStartingScale = ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxStartingScale = ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinScale = ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxScale = ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinScaleInTime = ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxScaleInTime = ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinScaleOutTime = ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxScaleOutTime = ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinRotation = ((float)tempint16 / 65535.0) * -6.28319;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxRotation = ((float)tempint16 / 65535.0) * -6.28319;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinRotationSpeed = -1.0 + ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxRotationSpeed = -1.0 + ((double)tempint16 / 65535.0) * 2.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinFullAlpha = ((double)tempint16 / 65535.0) * 1.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxFullAlpha = ((double)tempint16 / 65535.0) * 1.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinStartX = ((double)tempint16 / 65535.0) * 1.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxStartX = ((double)tempint16 / 65535.0) * 1.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMinStartY = ((double)tempint16 / 65535.0) * 1.0;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_fMaxStartY = ((double)tempint16 / 65535.0) * 1.0;

	// Read in the color settings out
	pFile->ReadData((void*)&tempint8, sizeof(uint8_t));
	pParticleSystem->m_bColorRanges = tempint8;

	pFile->ReadData((void*)&tempint8, sizeof(uint8_t));
	pParticleSystem->m_bColorTransitions = tempint8;

	pFile->ReadData((void*)&tempint16, sizeof(uint16_t));
	pParticleSystem->m_iColorArrayLength = tempint16;

	pParticleSystem->m_pColorArray = new xColor[tempint16];
	for(int i = 0; i < tempint16; i++)
	{
		pFile->ReadData((void*)&tempint8, sizeof(uint8_t));
		pParticleSystem->m_pColorArray[i].R = (float)tempint8 / 255.0;

		pFile->ReadData((void*)&tempint8, sizeof(uint8_t));
		pParticleSystem->m_pColorArray[i].G = (float)tempint8 / 255.0;

		pFile->ReadData((void*)&tempint8, sizeof(uint8_t));
		pParticleSystem->m_pColorArray[i].B = (float)tempint8 / 255.0;
	}
}
