/////////////////////////////////////////////////////////////////
// Name: xField.h
/////////////////////////////////////////////////////////////////
// Description:
// - Keeps up with blocks that have already been
//   added to the field.
/////////////////////////////////////////////////////////////////
// - 7.15.2004, Update, Dayle Flowers(Xienen)
//   File Created
/////////////////////////////////////////////////////////////////
// - 7.30.2004, Update. Greg Feldman
//	 added functions to get the left and right extents of the field
/////////////////////////////////////////////////////////////////
// - 8.07.2004, Update, Alayton Norgard (Adept)
//   Added exploding piece goodness if it lands without using all additions
/////////////////////////////////////////////////////////////////
#ifndef XFIELD_H_
#define XFIELD_H_

#include "xPiece.h"
#include "xBlock.h"
#include "xGameDefines.h"

#define BB_PLACETAILS_MAX	15

enum xDir
{
	DOWN,
	RIGHT,
	UP,
	LEFT
};

class xField
{
public:
	xField()
	{
		m_pBackgroundObject	= 0;
		m_pBuildOverlay	= 0;
	}

	bool Init();
	bool Update(double dTime);
	void RefreshAllRenderables();
	void Shutdown();

	void ForceUpdateDroppingPiece();
	void NewPiece();
	void RotateFieldClockwise();
	void RotateFieldCounterClockwise();
	void DropPiece();
	bool CanMovePieceDown();
	void MovePieceDown();
	void PieceTouchedDown();
	void UpdateGlowAlphas(bool bDoClearableWhiteFlash);
	void UpdateShadow();
	int	 ClearBlocks(int *pWhiteBlocks, int *pRingsCleared);
	int  CheckClearBlocks(int iX, int iY, int iColorIndex, int iAltColorIndex = -10, int iAltColorIndex2 = -10);
	bool SearchForColor(int iX, int iY, int iStartColorIndex, int iSearchColorIndex);
	void ForceFinishRotation();

	int		m_iClearableBlockCount;
	DWORD m_dwLastDropTime;

	xRenderable2DQuad	*m_pBackgroundObject;
	xRenderable2DQuad	*m_pBuildOverlay;
	xRenderable2DQuad	*m_pPlacementFailureWarning;
	xRenderable2DQuad	*m_pRotateTails[BB_FIELDWIDTH / 2][BB_FIELDHEIGHT];
	xRenderable2DQuad	*m_pPlaceTails[BB_PLACETAILS_MAX];
	xBlock				 m_pFieldBlocks[BB_FIELDWIDTH][BB_FIELDHEIGHT];
	xEmitterList		 m_xBlockClearedEmitters;

	int					 m_iShadowPieceX[9];
	int					 m_iShadowPieceY[9];

	xRenderable2DQuad	*m_pBlockAnimationObject[BB_FIELDHEIGHT];

	float	m_fRotationTime;
	bool	m_bRotateClockwise;
	int		m_iCurrentPlaceTail;
};

#endif