#include "Paper.h"

int xGUI::Init(xSceneGraph *pSceneGraph)
{
	m_pSceneGraph = pSceneGraph;

	return PE_RES_NOERROR;
}

int xGUI::PreUpdate(DWORD dwTime)
{
	m_bHasFocus = false;
	xGUIWindowList *curwin = m_pWindowList;

	while(curwin)
	{
		bool hadfocus = curwin->m_pData->m_bHasFocus;

		if(!curwin->m_pData->GetHidden())
		{
			if(curwin->m_pData->pfnPreUpdate)
				curwin->m_pData->pfnPreUpdate(curwin->m_pData, dwTime);
		}
		else
		{
			curwin->m_pData->m_bHasFocus = false;
		}

		if(curwin->m_pData->m_bHasFocus)
		{
			//If it didnt have focus before, 
			//set it to have focus now
			if(!hadfocus)
			{
				if(m_pWindowList->m_pData->m_bAlwaysOnTop)
				{
					m_pWindowList->m_pData->m_bHasFocus	= true;
					curwin->m_pData->m_bHasFocus		= false;
				}
				else if(curwin->m_pData->m_bAlwaysOnBottom)
					curwin->m_pData->m_bHasFocus = true;
				else
					SetFocus(curwin);
			}

			curwin = curwin->m_pNext;
			while(curwin)
			{
				if(curwin->m_pData->pfnClearState)
					curwin->m_pData->pfnClearState(curwin->m_pData, dwTime);

				curwin = curwin->m_pNext;
			}

			//This GUI has focus now
			m_bHasFocus = true;
			
			break;
		}

		curwin = curwin->m_pNext;
	}

	if(m_pDragObject)
	{
		if(g_xPaperAPI.m_xInput.m_buttonMouseLeft.IsDown())
		{
			xRenderable2DQuadSetPosition(m_pDragObject, m_pDragObject->m_xRenderable2D.m_fX + g_xPaperAPI.m_xInput.m_lMouseX - m_lLastMouseX, m_pDragObject->m_xRenderable2D.m_fY + g_xPaperAPI.m_xInput.m_lMouseY - m_lLastMouseY);
			xRenderable2DQuadUpdate(m_pDragObject);

			m_lLastMouseX = g_xPaperAPI.m_xInput.m_lMouseX;
			m_lLastMouseY = g_xPaperAPI.m_xInput.m_lMouseY;
		}
		else
		{
			xGUIObjectList *curobject = &m_pWindowList->m_pData->m_xChildObjectList;
			while(curobject->m_pNext)
			{
				curobject = curobject->m_pNext;

				if(curobject->m_pData->m_strDropCommand &&
				   g_xPaperAPI.m_xInput.m_lMouseX >= curobject->m_pData->GetAbsoluteX() && g_xPaperAPI.m_xInput.m_lMouseX <= curobject->m_pData->GetAbsoluteX() + curobject->m_pData->GetWidth() &&
				   g_xPaperAPI.m_xInput.m_lMouseY >= curobject->m_pData->GetAbsoluteY() && g_xPaperAPI.m_xInput.m_lMouseY <= curobject->m_pData->GetAbsoluteY() + curobject->m_pData->GetHeight())
				{
					pfnRunDropCommand(curobject->m_pData->m_strDropCommand, curobject->m_pData->m_strDropParams, m_pWindowList, curobject->m_pData);
					break;
				}
			}

			xRenderable2DQuadUpdateVisibility(m_pDragObject, 0);
			m_pDragObject = NULL;
		}
	}
	else if(m_pHoveredObject)
	{
		if(!m_pHoveredObject->m_bHovered)
		{
			m_pHoverObject->SetHidden(true);
			m_pHoverObject->UpdateRenderables();
			xGUIObjectSyncChildRenderables(m_pHoverObject);
			m_pHoveredObject = NULL;
			m_pHoverObject = NULL;
		}
	}

	return PE_RES_NOERROR;
}

int xGUI::Update(double dTime)
{
	if(m_pWindowList)
	{
		//Render the windows in thier reverse order
		xGUIWindowList *curwin = m_pWindowList;
		while(curwin->m_pNext)
			curwin = curwin->m_pNext;

		while(curwin)
		{
			if(!curwin->m_pData->GetHidden() && curwin->m_pData->pfnUpdate)
				curwin->m_pData->pfnUpdate(curwin->m_pData, dTime);

			if(curwin->m_pData->m_bPendingClose)
			{
				if(curwin->m_pPrev)
				{
					curwin = curwin->m_pPrev;

					RemoveWindow(curwin->m_pNext);
				}
				else
				{
					xGUIWindowList *temp = curwin->m_pNext;

					RemoveWindow(curwin);

					curwin = temp;
				}
			}
			else
				curwin = curwin->m_pPrev;
		}
	}

//	if(m_pDragObject)
//		m_pDragObject->Render(m_iDragFrame);

	return PE_RES_NOERROR;
}

void xGUI::RefreshAllRenderables()
{
	xFont *curfont;
	xGUIWindowList *curwin;
	
	curfont = m_xFontPool.m_pNext;

	while(curfont)
	{
		xFontRefreshRenderables(curfont);

		curfont = curfont->m_pNext;
	}

	curwin = m_pWindowList;

	while(curwin)
	{
		curwin->m_pData->RefreshRenderables();
		curwin = curwin->m_pNext;
	}
}

int xGUI::Shutdown()
{
	xGUIWindowList *curwin = m_pWindowList;

	if(curwin)
	{
		while(curwin->m_pNext)
		{
			curwin = curwin->m_pNext;
			curwin->m_pPrev->m_pData->Shutdown();
			delete curwin->m_pPrev;
		}

		curwin->m_pData->Shutdown();
		delete curwin;
	}

	return PE_RES_NOERROR;
}

int xGUI::AddWindow(const char *strWindowName)
{
	float depth;
	char winfile[256] = {0};

	xGUIWindow *data = m_xWindowPool.CreateNewObject(strWindowName);

	xGUIWindowList *list = GetWindow(strWindowName);

#if 0
	DEBUGLOG("AddWindow %s %i %i %i\n", strWindowName, data->m_bLoaded, data->m_bPendingClose, data->m_bHidden);
	DEBUGLOG("WindowList Before:\n");
	xGUIWindowList *templist = m_pWindowList;
	while(templist)
	{
		DEBUGLOG("\t%s\n", templist->m_pData->m_strName);
		templist = templist->m_pNext;
	}
#endif

	if(!data->m_bLoaded || !list)
	{
		if(!m_pWindowList)
		{
			m_pWindowList = new xGUIWindowList;
			depth = 1.0;
		}
		else
		{
			depth = m_pWindowList->m_pData->GetAbsoluteDepth() + 1.0;

			//DEBUGLOG("HeadWindow %s\n", m_pWindowList->m_pData->m_strName);
			m_pWindowList->m_pPrev			= new xGUIWindowList;
			m_pWindowList->m_pPrev->m_pNext	= m_pWindowList;
			m_pWindowList					= m_pWindowList->m_pPrev;
		}

		list = m_pWindowList;
		list->m_pData = data;

		if(!data->m_bLoaded)
		{
			data->SetName(strWindowName);

			sprintf(winfile, "Data/Windows/%s.pew", strWindowName);
			if(m_iResult = data->Init(m_pSceneGraph, this, list, winfile, depth))
				assert(0);

			data->m_bLoaded = true;
		}

		//Make sure a new window wasn't added during Init before bringing us to the front
		if(list == m_pWindowList)
			SetFocus(list, true);
	}
	else
	{
		data->m_bPendingClose = false;
		data->SetHidden(false);

		SetFocus(list, true);

		if(data->m_strPostLoadCommand)
			pfnRunPostLoadCommand(data->m_strPostLoadCommand, data->m_strPostLoadParams, data->m_pOwningWindow);

		data->UpdateRenderables();
	}

#if 0
	DEBUGLOG("WindowList After:\n");
	xGUIWindowList *templist2 = m_pWindowList;
	while(templist2)
	{
		DEBUGLOG("\t%s\n", templist2->m_pData->m_strName);
		templist2 = templist2->m_pNext;
	}
#endif

	return PE_RES_NOERROR;
}

xGUIWindowList* xGUI::GetWindow(const char *strWindowName)
{
	xGUIWindowList *curwin = m_pWindowList;

	while(curwin)
	{
		if(!stricmp(curwin->m_pData->m_strName, strWindowName))
			return curwin;

		curwin = curwin->m_pNext;
	}

	return NULL;
}

xGUIWindowList*	xGUI::GetWindowList(xGUIWindow *pWindow)
{
	xGUIWindowList *curwin = m_pWindowList;

	while(curwin)
	{
		if(pWindow == curwin->m_pData)
			return curwin;

		curwin = curwin->m_pNext;
	}

	return NULL;
}

void xGUI::DelayedRemoveAllWindows()
{
	xGUIWindowList *curwin = m_pWindowList;

	while(curwin)
	{
		curwin->m_pData->m_bPendingClose = true;
		curwin = curwin->m_pNext;
	}
}

int	xGUI::RemoveWindow(xGUIWindowList *pWindow)
{
	if(!pWindow)
		return 0;

	if(pWindow == m_pWindowList)
	{
		m_pWindowList = m_pWindowList->m_pNext;
		
		if(m_pWindowList)
			m_pWindowList->m_pData->m_bHasFocus = true;
	}

	if(pWindow->m_pNext)
		pWindow->m_pNext->m_pPrev = pWindow->m_pPrev;
	if(pWindow->m_pPrev)
		pWindow->m_pPrev->m_pNext = pWindow->m_pNext;

	pWindow->m_pData->Shutdown();

	m_xWindowPool.DeleteObject(pWindow->m_pData);

	delete pWindow;

	return PE_RES_NOERROR;
}

int xGUI::RemoveAllWindows()
{
	xGUIWindowList *curwin = m_pWindowList;

	if(curwin)
	{
		while(curwin->m_pNext)
		{
			curwin = curwin->m_pNext;
			RemoveWindow(curwin->m_pPrev);
		} 

		RemoveWindow(curwin);
	}

	return PE_RES_NOERROR;
}

void xGUI::SetFocus(xGUIWindowList *pWindow, bool bForce)
{
	float highdepth;

	//If it already has focus, return
	if(!bForce && m_pWindowList == pWindow)
		return;

	if(m_pWindowList != pWindow)
	{
		//Remove pWindow from its current position
		if(pWindow->m_pNext)
			pWindow->m_pNext->m_pPrev = pWindow->m_pPrev;
		if(pWindow->m_pPrev)
			pWindow->m_pPrev->m_pNext = pWindow->m_pNext;

		//Set pWindow as the m_pPrev to the current first Window
		m_pWindowList->m_pPrev = pWindow;

		//Set pWindow's new m_pNext and NULL out it's m_pPrev
		pWindow->m_pNext = m_pWindowList;
		pWindow->m_pPrev = NULL;

		//Set m_pWindowList as the new first Window
		m_pWindowList = pWindow;
	}

	//This window has focus
	pWindow->m_pData->m_bHasFocus = true;

	//The GUI has focus now
	m_bHasFocus = true;

	//Lets find the highest depth a window currently has
	highdepth = 0.0;

	while(pWindow->m_pNext)
	{
		pWindow = pWindow->m_pNext;
		pWindow->m_pData->m_bHasFocus = false;
		pWindow->m_pData->m_bDragging = false;

		if(pWindow->m_pData->GetDepth() > highdepth)
			highdepth = pWindow->m_pData->GetDepth();
	}

	if(highdepth >= m_pWindowList->m_pData->GetDepth())
		m_pWindowList->m_pData->UpdateWindowDepth(floor(highdepth) + 1.0f);
}

xFont* xGUI::GetFont(const char *strFontName)
{
	//Add it to the FontPool
	xFont *font = xGUIFontPoolNew(this, strFontName);

	//If it hasn't already been loaded, load the font
	if(!font->m_bLoaded)
	{
		xFontLoad(font, strFontName);

		strcpy(font->m_strObjectPath, strFontName);

		//It's loaded now
		font->m_bLoaded = true;
	}

	return font;
}

void xGUI::RunHoverCommand(xGUIObject *pObject)
{
	pfnRunHoverCommand(pObject->m_strHoverCommand, pObject->m_strHoverParams, pObject->m_pOwningWindow, pObject, m_pHoverObject);

	pObject->m_bRanHoveredCommand = 1;

	if(m_pHoverObject)
	{
		m_pHoveredObject = pObject;
		m_pHoverObject->SetHidden(0);
		m_pHoverObject->UpdateRenderables();
		xGUIObjectSyncChildRenderables(m_pHoverObject);
	}
}

void xGUI::RunUnHoverCommand(xGUIObject *pObject)
{
	pfnRunUnHoverCommand(pObject->m_strUnHoverCommand, pObject->m_strUnHoverParams, pObject->m_pOwningWindow, pObject);

	m_pHoverObject = 0;
}

void xGUI::RunDragCommand(xGUIObject *pObject)
{
	pfnRunDragCommand(pObject->m_strDragCommand, pObject->m_strDragParams, pObject->m_pOwningWindow, pObject, m_pDragObject);

	if(m_pDragObject)
	{
		m_lLastMouseX = g_xPaperAPI.m_xInput.m_lMouseX;
		m_lLastMouseY = g_xPaperAPI.m_xInput.m_lMouseY;
	}
}