/////////////////////////////////////////////////////////////////
// Name: xGUI.h
/////////////////////////////////////////////////////////////////
// Description:
// - Main interface for loading UI Windows(not HUD)
/////////////////////////////////////////////////////////////////
// - 07.11.2004, Update, Dayle Flowers (Xienen)
//   Restarted the whole GUI.  Arex has not advanced much on
//   the GUI lately, so he is off the team.  Everything will
//   be working within the next couple days.
//
// - 25.10.2004, Note, Arto Ruotsalainen (Arex)
//   ListBox and ListTree are ready, xGUIStringList can be used
//   as a list box. Also added xGUIStringBox, which isn't fully 
//   ready yet, but you can already write onto it. :)
//
// - 25.10.2004, Update, Arto Ruotsalainen (Arex)
//   Added xGUIStringTree, it's working, but it doesn't contain
//   mouse hit checks. 
//
// - 7.8.2004, Update, Arto Ruotsalainen (Arex)
//   Added xGUIStringList, xGUIStringListChild
//
// - 7.6.2004, Update, Arto Ruotsalainen(Arex)
//   Comment stuff added to begin of the file.
//   Add new updates/fixes above this, so the old updates
//   will be at the bottom.
/////////////////////////////////////////////////////////////////
#ifndef XGUI_H_
#define XGUI_H_

#include "xFont.h"
#include "xGUIObject.h"
#include "xGUIImage.h"
#include "xGUIModel.h"
#include "xGUIButton.h"
#include "xGUICheckBox.h"
#include "xGUIComboBox.h"
#include "xGUIListBox.h"
#include "xGUIColumnListBox.h"
#include "xGUIScrollBar.h"
#include "xGUISlider.h"
#include "xGUIText.h"
#include "xGUITextBox.h"
#include "xGUITreeList.h"
#include "xGUIWindow.h"

class xGUI
{
public:
	xGUI()
	{
		m_pWindowList		= NULL;
		m_bHasFocus			= true;
		m_pDragObject		= NULL;
		m_pHoveredObject	= NULL;
		m_pHoverObject		= NULL;
	}

	int  Init(xSceneGraph *pSceneGraph);
	int  PreUpdate(DWORD dwTime);
	int  Update(double dTime);
	void RefreshAllRenderables();
	int  Shutdown();

	int				AddWindow(const char *strFile);
	xGUIWindowList* GetWindow(const char *strWindowName);
	xGUIWindowList*	GetWindowList(xGUIWindow *pWindow);
	void			DelayedRemoveWindow(xGUIWindowList *pWindow) { if(pWindow) pWindow->m_pData->m_bPendingClose = true; }
	void			DelayedRemoveWindow(const char *strWindowName) { DelayedRemoveWindow(GetWindow(strWindowName)); }
	void			DelayedRemoveAllWindows();
	int				RemoveWindow(const char *strWindowName) { return RemoveWindow(GetWindow(strWindowName)); }
	int				RemoveWindow(xGUIWindow *pWindow) { return RemoveWindow(GetWindowList(pWindow)); }
	int				RemoveWindow(xGUIWindowList *pWindow);
	int				RemoveAllWindows();
	xFont*			GetFont(const char *strFontName);
	void			SetFocus(const char *strWindowName, bool bForce = false) { SetFocus(GetWindow(strWindowName), bForce); }
	void			SetFocus(xGUIWindowList *pWindow, bool bForce = false);
	bool			GUIHasFocus() { return m_bHasFocus; }

	void RunHoverCommand(xGUIObject *pObject);
	void RunUnHoverCommand(xGUIObject *pObject);
	void RunDragCommand(xGUIObject *pObject);
	bool IsDragging() { return m_pDragObject; }

	void (*pfnRunPostLoadCommand)(char *strCommand, char **strParams, xGUIWindowList *pWindow);
	void (*pfnRunInputCommand)(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject);
	void (*pfnRunKeyCommand)(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject);
	void (*pfnRunHoverCommand)(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject, xGUIObject* &pOutHoverObject);
	void (*pfnRunUnHoverCommand)(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject);
	void (*pfnRunDragCommand)(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject, xRenderable2DQuad* &pOutDragObject);
	void (*pfnRunDropCommand)(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject);
	void (*pfnHandlePaste)(char *strContents, xGUIWindowList *pWindow, xGUIObject *pObject);

	int  m_iResult;
	bool m_bHasFocus;

	// Drag and Drop Helpers
	long				 m_lLastMouseX;
	long				 m_lLastMouseY;
	xRenderable2DQuad	*m_pDragObject;
	xGUIObject			*m_pHoveredObject;
	xGUIObject			*m_pHoverObject;

	xSceneGraph			*m_pSceneGraph;
	xGUIWindowList		*m_pWindowList;
	xPool<xGUIWindow>	 m_xWindowPool;
	xFont				 m_xFontPool;
};

FORCE_INLINE xFont* xGUIFontPoolNew(xGUI *pGUI, const char* strFilePath)
{
	xFont *current = &pGUI->m_xFontPool;

	while(current->m_pNext)
	{
		if(!stricmp(current->m_strObjectPath, strFilePath))
		{
			current->m_iReferenceCount++;
			return current;
		}

		current = current->m_pNext;
	}

	if(!stricmp(current->m_strObjectPath, strFilePath))
	{
		current->m_iReferenceCount++;
		return current;
	}

	current->m_pNext = (xFont*)dlmalloc(sizeof(xFont));
	xFontInit(current->m_pNext);
	current->m_pNext->m_pPrev = current;
	
	strcpy(current->m_pNext->m_strObjectPath, strFilePath);
	return current->m_pNext;
}

#endif