/////////////////////////////////////////////////////////////////
// Name: xGUIButton.h
/////////////////////////////////////////////////////////////////
// Description:
// - This handles buttons for the GUI.
/////////////////////////////////////////////////////////////////
// - 07.11.2004, Note, DayleFlowers (Xienen)
//   File created
/////////////////////////////////////////////////////////////////
#ifndef XGUIBUTTON_H_
#define XGUIBUTTON_H_

#include "xGUIObject.h"
#include "xGUIText.h"

enum xGUIButtonState
{
	PE_GUI_BUTTONSTATE_NONE = 0,
	PE_GUI_BUTTONSTATE_HOVER,
	PE_GUI_BUTTONSTATE_CLICKED,
	PE_GUI_BUTTONSTATE_DISABLED,
	PE_GUI_BUTTONSTATE_TOGGLED
};

void xGUIButtonPreUpdate(xGUIObject* pThis, DWORD dwTime);
void xGUIButtonClearState(xGUIObject* pThis, DWORD dwTime);
void xGUIButtonUpdate(xGUIObject* pThis, double dTime);
void xGUIButtonUpdateRenderables(xGUIObject* pThis);
void xGUIButtonUpdateDepth(xGUIObject* pThis);
void xGUIButtonRefreshRenderables(xGUIObject* pThis);

class xGUIButton : public xGUIObject
{
public:
	xGUIButton()
	{
		pfnPreUpdate			= &xGUIButtonPreUpdate;
		pfnClearState			= &xGUIButtonClearState;
		pfnUpdate				= &xGUIButtonUpdate;
		pfnUpdateRenderables	= &xGUIButtonUpdateRenderables;
		pfnUpdateDepth			= &xGUIButtonUpdateDepth;
		pfnRefreshRenderables	= &xGUIButtonRefreshRenderables;

		m_iButtonState	= PE_GUI_BUTTONSTATE_NONE;
		m_bWasClicked	= 0;
		m_bToggled		= 0;

		m_pTextObject	= 0;
		m_pIconObject	= 0;
	}

	void InitRenderable(const char *strThemeName, const char* strBackground = 0, const char* strIconImage = 0, int iIconFrame = 0, const char* strText = 0);
	int  LoadObject(const char *strThemeName);
	void Shutdown();

	char* GetText() { return m_pTextObject->m_strBuffer; }
	void  SetText(const char *strText) { m_pTextObject->SetText(strText); }
	void  SetState(xGUIButtonState iState) { m_iButtonState = iState; xRenderable2DQuadUpdateTexture(m_pBackground, m_pTextures[iState]); }
	void  SetIconFrame(int iFrameIndex) { xRenderable2DQuadUpdateTexture(m_pIconObject, m_pIconTextures[iFrameIndex]); }

	char				  m_strBackgroundPath[256];
	xRenderable2DQuad	 *m_pBackground;
	xTexture			**m_pTextures;
	xGUIText			 *m_pTextObject;
	xGUIButtonState		  m_iButtonState;
	bool				  m_bWasClicked;
	bool				  m_bToggled;

	//Icon System
	xRenderable2DQuad	*m_pIconObject;
	xTexture			**m_pIconTextures;
	float				 m_fBorder;
};

#endif