#include "Paper.h"

void xGUIColumnListBox::InitRenderable(const char *strThemeName, const char *strBackground, const char *strSelectedBackground, const char *strHeaderBackground, const char *strScrollbarBG, const char *strScrollbarThumb, const char *strScrollbarUpArrow, const char *strScrollbarDownArrow)
{
	char backgroundpath[256] = {0};
	float x;
	int i, j;

	if(!strBackground || strBackground[0] == '\0')
		sprintf(backgroundpath, "Data/Themes/%s/ListBoxBG.p2d", strThemeName);
	else
		strcpy(backgroundpath, strBackground);

	if(m_bStretch)
	{
		m_pBackground = xSceneGraphCreateQuad(backgroundpath, 0);
		xRenderable2DQuadSetPosition(m_pBackground, GetAbsoluteX(), GetAbsoluteY());
		xRenderable2DQuadSetSize(m_pBackground, GetWidth(), GetHeight());
		xSceneGraphInsertQuad(m_pSceneGraph, m_pBackground, 9, GetAbsoluteDepth());
		xRenderable2DQuad9Update(m_pBackground);
	}
	else
	{
		m_pBackground = xSceneGraphCreateQuad(backgroundpath, 0);
		xRenderable2DQuadSetPosition(m_pBackground, GetAbsoluteX(), GetAbsoluteY());
		xSceneGraphInsertQuad(m_pSceneGraph, m_pBackground, 1, GetAbsoluteDepth());
		xRenderable2DQuadUpdate(m_pBackground);
	}

	if(!strSelectedBackground || strSelectedBackground[0] == '\0')
		sprintf(backgroundpath, "Data/Themes/%s/ListBoxSelectionBG.p2d", strThemeName);
	else
		strcpy(backgroundpath, strSelectedBackground);

	m_pSelectedBackground = xSceneGraphCreateQuad(backgroundpath, 0);
	xSceneGraphInsertQuad(m_pSceneGraph, m_pSelectedBackground, 1, GetAbsoluteDepth() + 0.001);
	xRenderable2DQuadUpdateVisibility(m_pSelectedBackground, false);
	xRenderable2DQuadUpdate(m_pSelectedBackground);

	if(!strHeaderBackground || strHeaderBackground[0] == '\0')
		sprintf(backgroundpath, "Data/Themes/%s/ListBoxHeader.p2d", strThemeName);
	else
		strcpy(backgroundpath, strHeaderBackground);

	m_iNumItems = 0;
	m_iNumVisibleItems = GetHeight() / (((float)xFontGetSize(m_pFont)) + m_fItemSpacing);
	m_iTopVisibleItem = 0;

	x = GetAbsoluteX() + m_fBorder;
	for(i = 0; i < m_iColumnCount; i++)
	{
		m_pHeaderBackgrounds[i] = xSceneGraphCreateQuad(backgroundpath, 0);
		xRenderable2DQuadSetPosition(m_pHeaderBackgrounds[i], x + (m_iColumnWidths[i] - xFontGetWidth(m_pFont, m_strColumnHeaders[i])) / 2, GetAbsoluteY());
		xRenderable2DQuadSetSize(m_pHeaderBackgrounds[i], m_iColumnWidths[i] - (m_fBorder / 2.0), xFontGetSize(m_pFont) + m_fItemSpacing);
		xSceneGraphInsertQuad(m_pSceneGraph, m_pHeaderBackgrounds[i], 1, GetAbsoluteDepth() + 0.001);
		xRenderable2DQuadUpdate(m_pHeaderBackgrounds[i]);

		xRenderable2DTextInit(&m_pHeaderTextObjects[i], GetAbsoluteDepth() + 0.002);
		xRenderable2DTextSetPosition(&m_pHeaderTextObjects[i], x + (m_iColumnWidths[i] - xFontGetWidth(m_pFont, m_strColumnHeaders[i])) / 2, GetAbsoluteY() + (m_fItemSpacing / 2.0));
		xRenderable2DTextSetFont(&m_pHeaderTextObjects[i], m_pFont);
		xRenderable2DTextSetText(&m_pHeaderTextObjects[i], m_pSceneGraph, m_strColumnHeaders[i]);

		m_pTextObjects[i] = (xRenderable2DText*)dlmalloc(sizeof(xRenderable2DText) * m_iNumVisibleItems);

		for(j = 0; j < m_iNumVisibleItems; j++)
			xRenderable2DTextInit(&m_pTextObjects[i][j], GetAbsoluteDepth() + 0.002);

		x += m_iColumnWidths[i];
	}

	if(m_bScrollbar)
	{
		m_pScrollbar = new xGUIScrollbar;
		m_pScrollbar->Init();
		m_pScrollbar->m_pParent = this;
		m_pScrollbar->m_pGUI = m_pGUI;
		m_pScrollbar->m_pOwningWindow = m_pOwningWindow;
		m_pScrollbar->m_pSceneGraph = m_pSceneGraph;
		m_pScrollbar->m_iVisibleObjects = m_iNumVisibleItems;
		m_pScrollbar->m_fScrollButtonWidth = m_iScrollbarWidth;
		m_pScrollbar->m_fScrollButtonHeight = m_iScrollbarWidth;
		m_pScrollbar->SetDepth(0.005);
		m_pScrollbar->SetX(GetWidth() + m_fBorder);
		m_pScrollbar->SetY(m_fBorder);
		m_pScrollbar->SetWidth(m_iScrollbarWidth);
		m_pScrollbar->SetHeight(GetHeight() - m_fBorder * 2);
		m_pScrollbar->InitRenderable(strThemeName, strScrollbarBG, strScrollbarThumb, strScrollbarUpArrow, strScrollbarDownArrow);
	}
}

int xGUIColumnListBox::LoadObject(const char *strThemeName)
{
	xGUIObject::LoadObject(strThemeName);

	m_fBorder			= m_pParser->ToFlt("m_fBorder");
	m_fItemSpacing		= m_pParser->ToFlt("m_fItemSpacing");
	m_iColumnCount		= m_pParser->ToInt("m_iColumnCount");
	m_bScrollbar		= m_pParser->ToBool("m_bScrollbar");
	m_iScrollbarWidth	= m_pParser->ToInt("m_iScrollbarWidth");
	
	char temp[20];
	for(int i = 0; i < m_iColumnCount; i++)
	{
		sprintf(temp, "m_strColumnHeader%i", i + 1);
		char *tempvalue = m_pParser->GetVariable(temp);
		m_strColumnHeaders[i] = new char[strlen(tempvalue) + 1];
		strcpy(m_strColumnHeaders[i], tempvalue);

		sprintf(temp, "m_iColumnWidth%i", i + 1);
		m_iColumnWidths[i] = m_pParser->ToInt(temp);
	}

	InitRenderable(strThemeName, m_pParser->GetVariable("m_strBackground"), m_pParser->GetVariable("m_strSelectedBackground"), m_pParser->GetVariable("m_strHeaderBackground"), m_pParser->GetVariable("m_strScrollbarBG"), m_pParser->GetVariable("m_strScrollbarThumb"), m_pParser->GetVariable("m_strScrollbarUpArrow"), m_pParser->GetVariable("m_strScrollbarDownArrow"));

	return PE_RES_NOERROR;
}

void xGUIColumnListBoxPreUpdate(xGUIObject* pThis, DWORD dwTime)
{
	int oldtopobject;

	if(!pThis->m_pParent->GetAbsoluteFocus())
		pThis->m_bHasFocus = false;

	xGUIObjectPreUpdate(pThis, dwTime);

	if(((xGUIColumnListBox*)pThis)->m_bScrollbar)
	{
		oldtopobject = ((xGUIColumnListBox*)pThis)->m_iTopVisibleItem;

		((xGUIColumnListBox*)pThis)->m_pScrollbar->pfnPreUpdate(((xGUIColumnListBox*)pThis)->m_pScrollbar, dwTime);

		((xGUIColumnListBox*)pThis)->m_iTopVisibleItem = ((xGUIColumnListBox*)pThis)->m_pScrollbar->m_iTopObject;

		if(((xGUIColumnListBox*)pThis)->m_iTopVisibleItem != oldtopobject)
			xGUIColumnListBoxUpdateRenderables(pThis);
	}

	if(g_xPaperAPI.m_xInput.m_lMouseX > pThis->GetAbsoluteX() && g_xPaperAPI.m_xInput.m_lMouseX < pThis->GetAbsoluteX() + pThis->GetWidth() &&
	   g_xPaperAPI.m_xInput.m_lMouseY > pThis->GetAbsoluteY() && g_xPaperAPI.m_xInput.m_lMouseY < pThis->GetAbsoluteY() + pThis->GetHeight())
	{
		if(g_xPaperAPI.m_xInput.m_buttonMouseLeft.JustPressed(true))
		{
			g_xPaperAPI.m_xInput.m_buttonMouseLeft.Eaten();

			pThis->m_bHasFocus = true;

			if(!((xGUIColumnListBox*)pThis)->m_bScrollbar || g_xPaperAPI.m_xInput.m_lMouseX < ((xGUIColumnListBox*)pThis)->m_pScrollbar->GetAbsoluteX())
			{
				int relativemousey = g_xPaperAPI.m_xInput.m_lMouseY - pThis->GetAbsoluteY();

				if(relativemousey >= xFontGetSize(pThis->m_pFont) + ((xGUIColumnListBox*)pThis)->m_fItemSpacing)
				{
					((xGUIColumnListBox*)pThis)->m_iSelected = ((xGUIColumnListBox*)pThis)->m_iTopVisibleItem - 1 + (relativemousey / (xFontGetSize(pThis->m_pFont) + ((xGUIColumnListBox*)pThis)->m_fItemSpacing));

					xRenderable2DQuadSetPosition(((xGUIColumnListBox*)pThis)->m_pSelectedBackground, ((xGUIColumnListBox*)pThis)->m_pSelectedBackground->m_xRenderable2D.m_fX, pThis->GetAbsoluteY() + (((xGUIColumnListBox*)pThis)->m_fBorder / 2.0) + ((xGUIColumnListBox*)pThis)->GetItemY(((xGUIColumnListBox*)pThis)->m_iSelected));
					xRenderable2DQuadUpdateVisibility(((xGUIColumnListBox*)pThis)->m_pSelectedBackground, true);
					xRenderable2DQuadUpdate(((xGUIColumnListBox*)pThis)->m_pSelectedBackground);
				}
				else
					((xGUIColumnListBox*)pThis)->SortListByHeaderColumnClicked(g_xPaperAPI.m_xInput.m_lMouseX - pThis->GetAbsoluteX());

				if(dwTime - pThis->PressTime() < 300) //Set Clicked only on Double Click
				{
					pThis->m_bClicked = true;

					if(pThis->m_strDblClickCommand)
						pThis->m_pGUI->pfnRunInputCommand(pThis->m_strDblClickCommand, pThis->m_strDblClickParams, pThis->m_pOwningWindow, pThis);

					if(pThis->m_pDblClickSound)
						pThis->m_pDblClickSound->Play(g_xPaperAPI.m_fVolume);
				}
				else
					pThis->SetPressTime(dwTime);
			}
		}
		else
			pThis->m_bClicked = false;
	}
	else
	{
		pThis->m_bClicked = false;

		if(g_xPaperAPI.m_xInput.m_buttonMouseLeft.JustPressed())
			pThis->m_bHasFocus = false;
	}

	if(g_xPaperAPI.m_bHasFocus && pThis->m_bHasFocus)
	{
		if(g_xPaperAPI.m_xInput.m_keyEnter.IsDown() || g_xPaperAPI.m_xInput.m_keyNumEnter.IsDown())
		{
			if(pThis->m_strEnterPressCommand)
			{
				if(g_xPaperAPI.m_xInput.m_keyEnter.JustPressed(true) || g_xPaperAPI.m_xInput.m_keyNumEnter.JustPressed(true))
				{
					pThis->m_pGUI->pfnRunInputCommand(pThis->m_strEnterPressCommand, pThis->m_strEnterPressParams, pThis->m_pOwningWindow, pThis);
					g_xPaperAPI.m_xInput.m_keyEnter.Eaten();
					g_xPaperAPI.m_xInput.m_keyNumEnter.Eaten();

					if(pThis->m_pEnterPressSound)
						pThis->m_pEnterPressSound->Play(g_xPaperAPI.m_fVolume);
				}
			}
		}
	}
}

void xGUIColumnListBoxUpdate(xGUIObject* pThis, double dTime)
{
	xColor tempcolor;

	if(!pThis->m_pParent->GetAbsoluteFocus())
		pThis->m_bHasFocus = false;

	if(!pThis->m_xAnimation.m_bDone)
		xAnimationUpdate(&pThis->m_xAnimation, dTime);

	if(((xGUIColumnListBox*)pThis)->m_bScrollbar)
		((xGUIColumnListBox*)pThis)->m_pScrollbar->pfnUpdate(((xGUIColumnListBox*)pThis)->m_pScrollbar, dTime);

	xGUIObjectUpdate(pThis, dTime);
}

void xGUIColumnListBoxUpdateRenderables(xGUIObject* pThis)
{
	xColor tempcolor;
	int i, j;
	float x, y;
	float listboxx = pThis->GetAbsoluteX();
	float listboxy = pThis->GetAbsoluteY();
	bool visible = !pThis->GetAbsoluteHidden();

	xRenderable2DQuadSetPosition(((xGUIColumnListBox*)pThis)->m_pBackground, listboxx, listboxy);
	xRenderable2DQuadUpdateAlpha(((xGUIColumnListBox*)pThis)->m_pBackground, pThis->GetAbsoluteColor(&tempcolor)->A);
	xRenderable2DQuadUpdateVisibility(((xGUIColumnListBox*)pThis)->m_pBackground, visible);

	if(pThis->m_bStretch)
		xRenderable2DQuad9Update(((xGUIColumnListBox*)pThis)->m_pBackground);
	else
		xRenderable2DQuadUpdate(((xGUIColumnListBox*)pThis)->m_pBackground);

	x = listboxx + ((xGUIColumnListBox*)pThis)->m_fBorder;
	for(i = 0; i < ((xGUIColumnListBox*)pThis)->m_iColumnCount; i++)
	{
		xRenderable2DQuadSetPosition(((xGUIColumnListBox*)pThis)->m_pHeaderBackgrounds[i], x - (((xGUIColumnListBox*)pThis)->m_fBorder / 2.0), pThis->GetAbsoluteY());
		xRenderable2DQuadUpdate(((xGUIColumnListBox*)pThis)->m_pHeaderBackgrounds[i]);
		xRenderable2DTextSetPosition(&((xGUIColumnListBox*)pThis)->m_pHeaderTextObjects[i], x + (((xGUIColumnListBox*)pThis)->m_iColumnWidths[i] - xFontGetWidth(pThis->m_pFont, ((xGUIColumnListBox*)pThis)->m_strColumnHeaders[i])) / 2, pThis->GetAbsoluteY() + (((xGUIColumnListBox*)pThis)->m_fItemSpacing / 2.0));
		xRenderable2DTextSetText(&((xGUIColumnListBox*)pThis)->m_pHeaderTextObjects[i], pThis->m_pSceneGraph, ((xGUIColumnListBox*)pThis)->m_strColumnHeaders[i]);

		xList<char> *curentryitem = ((xGUIColumnListBox*)pThis)->m_xStringItemLists[i].m_pNext;
		for(j = 0; curentryitem && j < ((xGUIColumnListBox*)pThis)->m_iTopVisibleItem; j++)
		{
			if(j == ((xGUIColumnListBox*)pThis)->m_iSelected)
				xRenderable2DQuadUpdateVisibility(((xGUIColumnListBox*)pThis)->m_pSelectedBackground, false);

			curentryitem = curentryitem->m_pNext;
		}

		y = listboxy + ((xGUIColumnListBox*)pThis)->m_fBorder + xFontGetSize(pThis->m_pFont) + ((xGUIColumnListBox*)pThis)->m_fItemSpacing;
		for(j = 0; curentryitem && j < ((xGUIColumnListBox*)pThis)->m_iNumVisibleItems && j < ((xGUIColumnListBox*)pThis)->m_iNumItems; j++)
		{
			xRenderable2DTextSetPosition(&((xGUIColumnListBox*)pThis)->m_pTextObjects[i][j], x, y);
			xRenderable2DTextUpdateAlpha(&((xGUIColumnListBox*)pThis)->m_pTextObjects[i][j], tempcolor.A);
			xRenderable2DTextUpdateVisibility(&((xGUIColumnListBox*)pThis)->m_pTextObjects[i][j], visible);
			xRenderable2DTextSetText(&((xGUIColumnListBox*)pThis)->m_pTextObjects[i][j], pThis->m_pSceneGraph, curentryitem->m_pData);

			if(j + ((xGUIColumnListBox*)pThis)->m_iTopVisibleItem == ((xGUIColumnListBox*)pThis)->m_iSelected)
			{
				xRenderable2DQuadSetPosition(((xGUIColumnListBox*)pThis)->m_pSelectedBackground, listboxx + (((xGUIColumnListBox*)pThis)->m_fBorder / 2.0), y);
				xRenderable2DQuadUpdateVisibility(((xGUIColumnListBox*)pThis)->m_pSelectedBackground, true);
			}

			curentryitem = curentryitem->m_pNext;

			y += xFontGetSize(pThis->m_pFont) + ((xGUIColumnListBox*)pThis)->m_fItemSpacing;
		}

		x += ((xGUIColumnListBox*)pThis)->m_iColumnWidths[i];
	}

	if(!visible || ((xGUIColumnListBox*)pThis)->m_iSelected >= ((xGUIColumnListBox*)pThis)->m_iTopVisibleItem + ((xGUIColumnListBox*)pThis)->m_iNumVisibleItems)
		xRenderable2DQuadUpdateVisibility(((xGUIColumnListBox*)pThis)->m_pSelectedBackground, false);

	xRenderable2DQuadUpdate(((xGUIColumnListBox*)pThis)->m_pSelectedBackground);
	xRenderable2DQuadUpdateAlpha(((xGUIColumnListBox*)pThis)->m_pSelectedBackground, tempcolor.A);

	if(((xGUIColumnListBox*)pThis)->m_bScrollbar)
		xGUIScrollbarUpdateRenderables(((xGUIColumnListBox*)pThis)->m_pScrollbar);
}

void xGUIColumnListBoxUpdateDepth(xGUIObject* pThis)
{
	int i, j;

	xRenderable2DQuadUpdateDepth(((xGUIColumnListBox*)pThis)->m_pBackground, pThis->m_pSceneGraph, pThis->GetAbsoluteDepth());
	xRenderable2DQuadUpdateDepth(((xGUIColumnListBox*)pThis)->m_pSelectedBackground, pThis->m_pSceneGraph, pThis->GetAbsoluteDepth() + 0.001);

	for(i = 0; i < ((xGUIColumnListBox*)pThis)->m_iColumnCount; i++)
	{
		xRenderable2DQuadUpdateDepth(((xGUIColumnListBox*)pThis)->m_pHeaderBackgrounds[i], pThis->m_pSceneGraph, pThis->GetAbsoluteDepth() + 0.001);
		xRenderable2DTextUpdateDepth(&((xGUIColumnListBox*)pThis)->m_pHeaderTextObjects[i], pThis->m_pSceneGraph, pThis->GetAbsoluteDepth() + 0.002);

		for(j = 0; j < ((xGUIColumnListBox*)pThis)->m_iNumVisibleItems && j < ((xGUIColumnListBox*)pThis)->m_iNumItems; j++)
			xRenderable2DTextUpdateDepth(&((xGUIColumnListBox*)pThis)->m_pTextObjects[i][j], pThis->m_pSceneGraph, pThis->GetAbsoluteDepth() + 0.002);
	}
}

void xGUIColumnListBoxRefreshRenderables(xGUIObject* pThis)
{
	int i, j;

	xRenderable2DQuadUpdateTexture(((xGUIColumnListBox*)pThis)->m_pBackground, ((xGUIColumnListBox*)pThis)->m_pBackground->m_pTextures[0]);
	xRenderable2DQuadUpdateTexture(((xGUIColumnListBox*)pThis)->m_pSelectedBackground, ((xGUIColumnListBox*)pThis)->m_pSelectedBackground->m_pTextures[0]);

	for(i = 0; i < ((xGUIColumnListBox*)pThis)->m_iColumnCount; i++)
	{
		xRenderable2DQuadUpdateTexture(((xGUIColumnListBox*)pThis)->m_pHeaderBackgrounds[i], ((xGUIColumnListBox*)pThis)->m_pHeaderBackgrounds[i]->m_pTextures[0]);
		xRenderable2DTextRefreshTextures(&((xGUIColumnListBox*)pThis)->m_pHeaderTextObjects[i]);

		for(j = 0; j < ((xGUIColumnListBox*)pThis)->m_iNumVisibleItems && j < ((xGUIColumnListBox*)pThis)->m_iNumItems; j++)
			xRenderable2DTextRefreshTextures(&((xGUIColumnListBox*)pThis)->m_pTextObjects[i][j]);
	}

	xGUIColumnListBoxUpdateRenderables(pThis);
}

void xGUIColumnListBox::Shutdown()
{
	int i, j;

	xGUIObject::Shutdown();

	if(m_pBackground)
		xSceneGraphDeleteQuad(m_pSceneGraph, m_pBackground, false);

	if(m_pSelectedBackground)
		xSceneGraphDeleteQuad(m_pSceneGraph, m_pSelectedBackground, false);

	for(i = 0; i < m_iColumnCount; i++)
	{
		if(m_pHeaderBackgrounds[i])
		{
			xSceneGraphDeleteQuad(m_pSceneGraph, m_pHeaderBackgrounds[i], false);
			xRenderable2DTextDelete(&m_pHeaderTextObjects[i], m_pSceneGraph);
		}

		for(j = 0; j < m_iNumVisibleItems && j < m_iNumItems; j++)
			xRenderable2DTextDelete(&m_pTextObjects[i][j], m_pSceneGraph);

		if(m_pTextObjects)
			dlfree(m_pTextObjects[i]);
	}

	if(m_pScrollbar)
	{
		m_pScrollbar->Shutdown();
		delete m_pScrollbar;
	}
}

void xGUIColumnListBox::AddItem(const char* Data, const char* NewItem1, const char* NewItem2, const char* NewItem3, const char* NewItem4, const char* NewItem5, const char* NewItem6, const char* NewItem7, const char* NewItem8, const char* NewItem9, const char* NewItem10)
{
	xList<char> *curitem, *sortitem = 0;
	const char *newitem;
	int i;

	curitem = &m_xStringDataList;
	while(curitem->m_pNext)
		curitem = curitem->m_pNext;

	curitem->m_pNext = new xList<char>;
	curitem->m_pNext->m_pPrev = curitem;
	curitem->m_pNext->m_pData = new char[strlen(Data) + 1];
	strcpy(curitem->m_pNext->m_pData, Data);

	for(i = 0; i < m_iColumnCount; i++)
	{
		curitem = &m_xStringItemLists[i];

		while(curitem->m_pNext)
			curitem = curitem->m_pNext;

		switch(i)
		{
			case 0:
				newitem = NewItem1;
				break;
			case 1:
				newitem = NewItem2;
				break;
			case 2:
				newitem = NewItem3;
				break;
			case 3:
				newitem = NewItem4;
				break;
			case 4:
				newitem = NewItem5;
				break;
			case 5:
				newitem = NewItem6;
				break;
			case 6:
				newitem = NewItem7;
				break;
			case 7:
				newitem = NewItem8;
				break;
			case 8:
				newitem = NewItem9;
				break;
			case 9:
				newitem = NewItem10;
				break;
		}

		curitem->m_pNext = new xList<char>;
		curitem->m_pNext->m_pPrev = curitem;
		curitem->m_pNext->m_pData = new char[strlen(newitem) + 1];
		strcpy(curitem->m_pNext->m_pData, newitem);

		if(i == m_iCurrentSortColumn)
			sortitem = curitem->m_pNext;

		if(m_iNumItems < m_iNumVisibleItems)
		{
			if(m_iSelected < 0)
			{
				m_iSelected = 0;
				xRenderable2DQuadSetPosition(m_pSelectedBackground, GetAbsoluteX() + (m_fBorder / 2.0), GetAbsoluteY() + (m_fBorder / 2.0));
				xRenderable2DQuadSetSize(m_pSelectedBackground, GetWidth() - m_fBorder, xFontGetSize(m_pFont) + (m_fItemSpacing / 2.0));
				xRenderable2DQuadUpdateVisibility(m_pSelectedBackground, true);
				xRenderable2DQuadUpdate(m_pSelectedBackground);
	}

			xRenderable2DTextSetPosition(&m_pTextObjects[i][m_iNumItems], GetAbsoluteX() + m_fBorder, GetAbsoluteY() + m_fBorder + GetItemY(m_iNumItems));
			xRenderable2DTextSetFont(&m_pTextObjects[i][m_iNumItems], m_pFont);
			xRenderable2DTextSetVisibility(&m_pTextObjects[i][m_iNumItems], true);
			xRenderable2DTextSetText(&m_pTextObjects[i][m_iNumItems], m_pSceneGraph, newitem);
		}
	}

	if(sortitem)
		InsertItem(sortitem);

	m_iNumItems++;

	if(m_bScrollbar)
	{
		m_pScrollbar->m_iNumObjects = m_iNumItems;
		xGUIScrollbarUpdateRenderables(m_pScrollbar);
	}
}

void xGUIColumnListBox::RemoveItem(int iIndex)
{
	int i;

	xList<char> *curitem = m_xStringDataList.m_pNext;
	for(i = 0; i <= m_iColumnCount; i++)
	{
		int index = 0;
		while(curitem && index < iIndex)
		{
			curitem = curitem->m_pNext;
			index++;
		}

		if(curitem)
		{
			delete [] curitem->m_pData;

			if(curitem->m_pNext)
				curitem->m_pNext->m_pPrev = curitem->m_pPrev;

			if(curitem->m_pPrev)
				curitem->m_pPrev->m_pNext = curitem->m_pNext;

			delete curitem;
		}

		if(i == m_iColumnCount)
			break;

		if(m_iNumItems <= m_iNumVisibleItems)
			xRenderable2DTextSetVisibility(&m_pTextObjects[i][m_iNumItems - 1], false);

		curitem = m_xStringItemLists[i].m_pNext;
	}

	if(m_iNumItems > 0)
	{
	m_iNumItems--;

		if(m_iNumItems == 0)
			m_iSelected = -1;
	}
}

void xGUIColumnListBox::RemoveAllItems()
{
	xList<char> *curitem = m_xStringDataList.m_pNext;
	m_xStringDataList.m_pNext = NULL;

	for(int i = 0; i <= m_iColumnCount; i++)
	{
		while(curitem)
		{
			delete [] curitem->m_pData;

			if(curitem->m_pNext)
			{
				curitem = curitem->m_pNext;
				delete curitem->m_pPrev;
			}
			else
			{
				delete curitem;
				break;
			}
		}

		if(m_iNumItems > 0 && m_iNumItems <= m_iNumVisibleItems)
			xRenderable2DTextSetVisibility(&m_pTextObjects[i][m_iNumItems - 1], false);

		if(i == m_iColumnCount)
			break;

		curitem = m_xStringItemLists[i].m_pNext;
		m_xStringItemLists[i].m_pNext = NULL;

		m_iNumItems--;
	}

	m_iNumItems = 0;
}

void xGUIColumnListBox::SetItemValue(int iColumn, int iIndex, char* strNewValue)
{
	int index = 0;
	xList<char> *curitem = m_xStringItemLists[iColumn].m_pNext;
	while(curitem && index < iIndex)
	{
		curitem = curitem->m_pNext;
		index++;
	}

	if(curitem)
	{
		delete [] curitem->m_pData;

		curitem->m_pData = new char[strlen(strNewValue) + 1];
		strcpy(curitem->m_pData, strNewValue);

		index = iIndex - m_iTopVisibleItem;
		if(index >= 0 && index < m_iNumItems)
			xRenderable2DTextSetText(&m_pTextObjects[iColumn][index], m_pSceneGraph, strNewValue);
	}
}

char* xGUIColumnListBox::GetSelectedData()
{
	xList<char> *curitem = &m_xStringDataList;
	for(int i = 0; curitem->m_pNext; i++)
	{
		curitem = curitem->m_pNext;

		if(i == m_iSelected)
			return curitem->m_pData;
	}

	return NULL;
}

char* xGUIColumnListBox::GetData(int iIndex)
{
	xList<char> *curitem = &m_xStringDataList;
	for(int i = 0; curitem->m_pNext; i++)
	{
		curitem = curitem->m_pNext;

		if(i == iIndex)
			return curitem->m_pData;
	}

	return NULL;
}

char* xGUIColumnListBox::GetItemValue(int iColumn, int iIndex)
{
	xList<char> *curitem = &m_xStringItemLists[iColumn];
	for(int i = 0; curitem->m_pNext; i++)
	{
		curitem = curitem->m_pNext;

		if(i == iIndex)
			return curitem->m_pData;
	}

	return NULL;
}

void xGUIColumnListBox::SortListByHeaderColumnClicked(DWORD dwRelativeX)
{
	int	column = -1;
	DWORD curcolumnx = 0;

	for(int i = 0; i < m_iColumnCount; i++)
	{
		if(dwRelativeX - curcolumnx < m_iColumnWidths[i])
		{
			column = i;
			break;
		}

		curcolumnx += m_iColumnWidths[i];
	}

	if(column != -1 && m_xStringItemLists[column].m_pNext)
	{
		if(column == m_iCurrentSortColumn)
			m_bReverseOrder = !m_bReverseOrder;
		else
		{
			m_bReverseOrder = false;
			m_iCurrentSortColumn = column;
		}

		xList<char> *curitem[PE_GUI_LIST_MAX_COLUMNS];

		for(int i = 0; i < m_iColumnCount; i++)
			curitem[i] = m_xStringItemLists[i].m_pNext->m_pNext;

		xList<char> *nextitem[PE_GUI_LIST_MAX_COLUMNS];
		xList<char> *sorteditems[PE_GUI_LIST_MAX_COLUMNS];

		xList<char> *curdata = m_xStringDataList.m_pNext->m_pNext;
		xList<char> *nextdata, *sorteddata;

		while(curitem[column])
		{
			for(int i = 0; i < m_iColumnCount; i++)
			{
				nextitem[i] = curitem[i]->m_pNext;
				sorteditems[i] = m_xStringItemLists[i].m_pNext;
			}

			nextdata = curdata->m_pNext;
			sorteddata = m_xStringDataList.m_pNext;

			while(sorteditems[column])
			{
				bool swap = false;

				if(m_bReverseOrder)
				{
					if(curitem[column]->m_pData[0] >= '0' && curitem[column]->m_pData[0] <= '9')
					{
						int lenitem = strlen(curitem[column]->m_pData);
						int lensorted = strlen(sorteditems[column]->m_pData);
						if(lenitem == lensorted)
							swap = stricmp(curitem[column]->m_pData, sorteditems[column]->m_pData) > 0;
						else if(lenitem > lensorted)
							swap = true;
					}
					else
						swap = stricmp(curitem[column]->m_pData, sorteditems[column]->m_pData) > 0;
				}
				else
				{
					if(curitem[column]->m_pData[0] >= '0' && curitem[column]->m_pData[0] <= '9')
					{
						int lenitem = strlen(curitem[column]->m_pData);
						int lensorted = strlen(sorteditems[column]->m_pData);
						if(lenitem == lensorted)
							swap = stricmp(curitem[column]->m_pData, sorteditems[column]->m_pData) < 0;
						else if(lenitem < lensorted)
							swap = true;
					}
					else
						swap = stricmp(curitem[column]->m_pData, sorteditems[column]->m_pData) < 0;
				}

				if(swap)
				{
					for(int i = 0; i < m_iColumnCount; i++)
					{
						if(curitem[i]->m_pPrev)
							curitem[i]->m_pPrev->m_pNext = curitem[i]->m_pNext;

						if(curitem[i]->m_pNext)
							curitem[i]->m_pNext->m_pPrev = curitem[i]->m_pPrev;

						if(sorteditems[i]->m_pPrev)
							sorteditems[i]->m_pPrev->m_pNext = curitem[i];

						curitem[i]->m_pPrev = sorteditems[i]->m_pPrev;
						curitem[i]->m_pNext = sorteditems[i];

						sorteditems[i]->m_pPrev = curitem[i];
					}

					if(curdata->m_pPrev)
						curdata->m_pPrev->m_pNext = curdata->m_pNext;

					if(curdata->m_pNext)
						curdata->m_pNext->m_pPrev = curdata->m_pPrev;

					if(sorteddata->m_pPrev)
						sorteddata->m_pPrev->m_pNext = curdata;

					curdata->m_pPrev = sorteddata->m_pPrev;
					curdata->m_pNext = sorteddata;

					sorteddata->m_pPrev = curdata;

					break;
				}

				if(sorteditems[column] == curitem[column])
				{
					break;
				}

				if(sorteditems[column]->m_pNext == NULL)
				{
					for(int i = 0; i < m_iColumnCount; i++)
					{
						sorteditems[i]->m_pNext	= curitem[i];
						curitem[i]->m_pPrev		= sorteditems[i]->m_pNext;
						curitem[i]->m_pNext		= NULL;
					}

					sorteddata->m_pNext	= curdata;
					curdata->m_pPrev	= sorteddata->m_pNext;
					curdata->m_pNext	= NULL;

					break;
				}

				for(int i = 0; i < m_iColumnCount; i++)
					sorteditems[i] = sorteditems[i]->m_pNext;

				sorteddata = sorteddata->m_pNext;
			}

			for(int i = 0; i < m_iColumnCount; i++)
				curitem[i] = nextitem[i];

			curdata = nextdata;
		}
	}

	xGUIColumnListBoxUpdateRenderables(this);
}

void xGUIColumnListBox::InsertItem(xList<char> *pCurItem)
{
	if(m_xStringItemLists[m_iCurrentSortColumn].m_pNext && m_xStringItemLists[m_iCurrentSortColumn].m_pNext->m_pNext)
	{
		xList<char> *curitem[PE_GUI_LIST_MAX_COLUMNS];
		xList<char> *curdata = m_xStringDataList.m_pNext->m_pNext;

		for(int i = 0; i < m_iColumnCount; i++)
			curitem[i] = m_xStringItemLists[i].m_pNext->m_pNext;

		while(curitem[m_iCurrentSortColumn] != pCurItem)
		{
			for(int i = 0; i < m_iColumnCount; i++)
				curitem[i] = curitem[i]->m_pNext;

			curdata = curdata->m_pNext;
		}

		xList<char> *nextitem[PE_GUI_LIST_MAX_COLUMNS];
		xList<char> *sorteditems[PE_GUI_LIST_MAX_COLUMNS];

		xList<char> *nextdata, *sorteddata;

		while(curitem[m_iCurrentSortColumn])
		{
			for(int i = 0; i < m_iColumnCount; i++)
			{
				nextitem[i] = curitem[i]->m_pNext;
				sorteditems[i] = m_xStringItemLists[i].m_pNext;
			}

			nextdata = curdata->m_pNext;
			sorteddata = m_xStringDataList.m_pNext;

			while(sorteditems[m_iCurrentSortColumn])
			{
				bool swap = false;

				if(m_bReverseOrder)
				{
					if(curitem[m_iCurrentSortColumn]->m_pData[0] >= '0' && curitem[m_iCurrentSortColumn]->m_pData[0] <= '9')
					{
						int lenitem = strlen(curitem[m_iCurrentSortColumn]->m_pData);
						int lensorted = strlen(sorteditems[m_iCurrentSortColumn]->m_pData);
						if(lenitem == lensorted)
							swap = stricmp(curitem[m_iCurrentSortColumn]->m_pData, sorteditems[m_iCurrentSortColumn]->m_pData) > 0;
						else if(lenitem > lensorted)
							swap = true;
					}
					else
						swap = stricmp(curitem[m_iCurrentSortColumn]->m_pData, sorteditems[m_iCurrentSortColumn]->m_pData) > 0;
				}
				else
				{
					if(curitem[m_iCurrentSortColumn]->m_pData[0] >= '0' && curitem[m_iCurrentSortColumn]->m_pData[0] <= '9')
					{
						int lenitem = strlen(curitem[m_iCurrentSortColumn]->m_pData);
						int lensorted = strlen(sorteditems[m_iCurrentSortColumn]->m_pData);
						if(lenitem == lensorted)
							swap = stricmp(curitem[m_iCurrentSortColumn]->m_pData, sorteditems[m_iCurrentSortColumn]->m_pData) < 0;
						else if(lenitem < lensorted)
							swap = true;
					}
					else
						swap = stricmp(curitem[m_iCurrentSortColumn]->m_pData, sorteditems[m_iCurrentSortColumn]->m_pData) < 0;
				}

				if(swap)
				{
					for(int i = 0; i < m_iColumnCount; i++)
					{
						if(curitem[i]->m_pPrev)
							curitem[i]->m_pPrev->m_pNext = curitem[i]->m_pNext;

						if(curitem[i]->m_pNext)
							curitem[i]->m_pNext->m_pPrev = curitem[i]->m_pPrev;

						if(sorteditems[i]->m_pPrev)
							sorteditems[i]->m_pPrev->m_pNext = curitem[i];

						curitem[i]->m_pPrev = sorteditems[i]->m_pPrev;
						curitem[i]->m_pNext = sorteditems[i];

						sorteditems[i]->m_pPrev = curitem[i];
					}

					if(curdata->m_pPrev)
						curdata->m_pPrev->m_pNext = curdata->m_pNext;

					if(curdata->m_pNext)
						curdata->m_pNext->m_pPrev = curdata->m_pPrev;

					if(sorteddata->m_pPrev)
						sorteddata->m_pPrev->m_pNext = curdata;

					curdata->m_pPrev = sorteddata->m_pPrev;
					curdata->m_pNext = sorteddata;

					sorteddata->m_pPrev = curdata;

					break;
				}

				if(sorteditems[m_iCurrentSortColumn] == curitem[m_iCurrentSortColumn])
				{
					break;
				}

				if(sorteditems[m_iCurrentSortColumn]->m_pNext == NULL)
				{
					for(int i = 0; i < m_iColumnCount; i++)
					{
						sorteditems[i]->m_pNext	= curitem[i];
						curitem[i]->m_pPrev		= sorteditems[i]->m_pNext;
						curitem[i]->m_pNext		= NULL;
					}

					sorteddata->m_pNext	= curdata;
					curdata->m_pPrev	= sorteddata->m_pNext;
					curdata->m_pNext	= NULL;

					break;
				}

				for(int i = 0; i < m_iColumnCount; i++)
					sorteditems[i] = sorteditems[i]->m_pNext;

				sorteddata = sorteddata->m_pNext;
			}

			for(int i = 0; i < m_iColumnCount; i++)
				curitem[i] = nextitem[i];

			curdata = nextdata;
		}
	}

	xGUIColumnListBoxUpdateRenderables(this);
}
