/////////////////////////////////////////////////////////////////
// Name: xGUIColumnListBox.h
/////////////////////////////////////////////////////////////////
// Description:
// - This handles multi-column list boxes for the GUI.
/////////////////////////////////////////////////////////////////
// - 04.07.2010, Note, DayleFlowers (Xienen)
//   File created
/////////////////////////////////////////////////////////////////
#ifndef XGUICOLUMNLISTBOX_H_
#define XGUICOLUMNLISTBOX_H_

#include "xList.h"
#include "xGUIObject.h"
#include "xGUIScrollbar.h"

void xGUIColumnListBoxPreUpdate(xGUIObject* pThis, DWORD dwTime);
void xGUIColumnListBoxUpdate(xGUIObject* pThis, double dTime);
void xGUIColumnListBoxUpdateRenderables(xGUIObject* pThis);
void xGUIColumnListBoxUpdateDepth(xGUIObject* pThis);
void xGUIColumnListBoxRefreshRenderables(xGUIObject* pThis);

#define PE_GUI_LIST_MAX_COLUMNS	10

class xGUIColumnListBox : public xGUIObject
{
public:
	xGUIColumnListBox()
	{
		pfnPreUpdate	= &xGUIColumnListBoxPreUpdate;
		pfnUpdate		= &xGUIColumnListBoxUpdate;
		pfnUpdateRenderables	= &xGUIColumnListBoxUpdateRenderables;
		pfnUpdateDepth			= &xGUIColumnListBoxUpdateDepth;
		pfnRefreshRenderables	= &xGUIColumnListBoxRefreshRenderables;

		m_pBackground			= 0;
		m_pSelectedBackground	= 0;
		m_iCurrentSortColumn = -1;
		m_iColumnCount = 0;
		m_iSelected				= -1;
		m_pScrollbar			= 0;
		m_fBorder				= 0.0;
		m_fItemSpacing			= 0.0;
		m_iScrollbarWidth		= 16;

		m_xFontColor.R = m_xFontColor.G = m_xFontColor.B = m_xFontColor.A = 1.0;

		for(int i = 0; i < PE_GUI_LIST_MAX_COLUMNS; i++)
		{
			m_pTextObjects[i]		= 0;
			m_pHeaderBackgrounds[i]	= 0;
	}
	}

	void InitRenderable(const char *strThemeName, const char *strBackground, const char *strHeaderBackground, const char *strSelectedBackground, const char *strScrollbarBG, const char *strScrollbarThumb, const char *strScrollbarUpArrow, const char *strScrollbarDownArrow);
	int  LoadObject(const char *strThemeName);
	void Shutdown();

	void  AddItem(const char* Data, const char* NewItem1, const char* NewItem2, const char* NewItem3 = NULL, const char* NewItem4 = NULL, const char* NewItem5 = NULL, const char* NewItem6 = NULL, const char* NewItem7 = NULL, const char* NewItem8 = NULL, const char* NewItem9 = NULL, const char* NewItem10 = NULL);
	void  RemoveItem(int iIndex);
	void  RemoveAllItems();
	void  SetItemValue(int iColumn, int iIndex, char* strNewValue);
	char* GetSelectedData();
	char* GetData(int iIndex);
	float GetItemY(int iItem) { return (iItem - m_iTopVisibleItem) * (xFontGetSize(m_pFont) + m_fItemSpacing); }
	char* GetItemValue(int iColumn, int iIndex);
	void  SortListByHeaderColumnClicked(DWORD dwRelativeX);

	xRenderable2DQuad	*m_pBackground;
	xRenderable2DQuad	*m_pHeaderBackgrounds[PE_GUI_LIST_MAX_COLUMNS];
	xRenderable2DText	 m_pHeaderTextObjects[PE_GUI_LIST_MAX_COLUMNS];
	char				*m_strColumnHeaders[PE_GUI_LIST_MAX_COLUMNS];
	xRenderable2DQuad	*m_pSelectedBackground;
	float		 m_fBorder;
	float		 m_fItemSpacing;
	xColor		 m_xFontColor;

	int			 m_iCurrentSortColumn;
	bool		 m_bReverseOrder;
	int          m_iColumnCount;
	int          m_iColumnWidths[PE_GUI_LIST_MAX_COLUMNS];
	xRenderable2DText	*m_pTextObjects[PE_GUI_LIST_MAX_COLUMNS];
	xList<char>  m_xStringItemLists[PE_GUI_LIST_MAX_COLUMNS];
	xList<char>  m_xStringDataList;
	int			 m_iNumItems;
	int			 m_iNumVisibleItems;
	int			 m_iTopVisibleItem;
	int			 m_iSelected;

	bool			 m_bScrollbar;
	int				 m_iScrollbarWidth;
	xGUIScrollbar	*m_pScrollbar;

private:
	void InsertItem(xList<char> *pCurItem);
};

#endif