/////////////////////////////////////////////////////////////////
// Name: xGUIListBox.h
/////////////////////////////////////////////////////////////////
// Description:
// - This handles list boxes for the GUI.
/////////////////////////////////////////////////////////////////
// - 07.11.2004, Note, DayleFlowers (Xienen)
//   File created
/////////////////////////////////////////////////////////////////
#ifndef XGUILISTBOX_H_
#define XGUILISTBOX_H_

#include "xList.h"
#include "xGUIObject.h"
#include "xGUIScrollbar.h"

void xGUIListBoxPreUpdate(xGUIObject* pThis, DWORD dwTime);
void xGUIListBoxUpdate(xGUIObject* pThis, double dTime);
void xGUIListBoxUpdateRenderables(xGUIObject* pThis);
void xGUIListBoxUpdateDepth(xGUIObject* pThis);
void xGUIListBoxRefreshRenderables(xGUIObject* pThis);

class xGUIListBox : public xGUIObject
{
public:
	xGUIListBox()
	{
		pfnPreUpdate			= &xGUIListBoxPreUpdate;
		pfnUpdate				= &xGUIListBoxUpdate;
		pfnUpdateRenderables	= &xGUIListBoxUpdateRenderables;
		pfnUpdateDepth			= &xGUIListBoxUpdateDepth;
		pfnRefreshRenderables	= &xGUIListBoxRefreshRenderables;

		m_pBackground			= 0;
		m_pSelectedBackground	= 0;
		m_pTextObjects			= 0;
		m_iSelected				= 0;
		m_pScrollbar			= 0;
		m_fBorder				= 0.0;
		m_fItemSpacing			= 0.0;
		m_iScrollbarWidth		= 16;

		m_xFontColor.R = m_xFontColor.G = m_xFontColor.B = m_xFontColor.A = 1.0;
	}

	void InitRenderable(const char *strThemeName, const char *strBackground, const char *strSelectedBackground, const char *strScrollbarBG, const char *strScrollbarThumb, const char *strScrollbarUpArrow, const char *strScrollbarDownArrow);
	int  LoadObject(const char *strThemeName);
	void Shutdown();

	void  AddItem(const char *strNewItem);
	float GetItemY(int iItem) { return (iItem - m_iTopVisibleItem) * (xFontGetSize(m_pFont) + m_fItemSpacing); }
	int   GetSelectedIndex() { return m_iSelected; }
	char* GetSelectedString();
	bool  SetSelectedByString(const char *strItem);

	xRenderable2DQuad	*m_pBackground;
	xRenderable2DQuad	*m_pSelectedBackground;
	float				 m_fBorder;
	float				 m_fItemSpacing;
	xColor				 m_xFontColor;

	xRenderable2DText	*m_pTextObjects;
	xList<char>			 m_xStringItemList;
	int					 m_iNumItems;
	int					 m_iNumVisibleItems;
	int					 m_iTopVisibleItem;
	int					 m_iSelected;

	bool			 m_bScrollbar;
	int				 m_iScrollbarWidth;
	float			 m_fScrollbarOffset;
	xGUIScrollbar	*m_pScrollbar;
};

#endif