/////////////////////////////////////////////////////////////////
// Name: xGUIObject.h
/////////////////////////////////////////////////////////////////
// Description:
// - This handles GUI Objects.
/////////////////////////////////////////////////////////////////
// - 07.11.2004, Note, DayleFlowers (Xienen)
//   File created
/////////////////////////////////////////////////////////////////
#ifndef XGUIOBJECT_H_
#define XGUIOBJECT_H_

#include "xResults.h"
#include "xFont.h"
#include "xParser.h"
#include "xGUI.h"
#include "xKeyFrame.h"
#include "xGUISelectionGroup.h"

enum xGUIObjectType
{
	PE_GUI_IMAGE = 0,
	PE_GUI_MODEL,
	PE_GUI_TEXT,
	PE_GUI_TEXTBOX,
	PE_GUI_LISTBOX,
	PE_GUI_COLUMNLISTBOX,
	PE_GUI_COMBOBOX,
	PE_GUI_BUTTON,
	PE_GUI_CHECKBOX,
	PE_GUI_TREELIST,
	PE_GUI_SCROLLBAR,
	PE_GUI_SLIDER
};

typedef class xGUI;
typedef class xGUIObject;
typedef class xGUIButton;
typedef class xGUIWindow;
typedef xList<xGUIObject> xGUIObjectList;
typedef xList<xGUIWindow> xGUIWindowList;

//Callbacks to the Game
extern void ExecTextCommand(char* &strOutput, int iMaxSize, char *strCommand, char **strParams);
void xGUIObjectPreUpdate(xGUIObject* pThis, DWORD dwTime);
void xGUIObjectClearState(xGUIObject* pThis, DWORD dwTime);
void xGUIObjectUpdate(xGUIObject* pThis, double dTime);
void xGUIObjectUpdateRenderables(xGUIObject* pThis);
void xGUIObjectUpdateDepth(xGUIObject* pThis);
void xGUIObjectRefreshRenderables(xGUIObject* pThis);

class xGUIObject : public xButton
{
public:
	xGUIObject()
	{
		pfnPreUpdate			= 0;
		pfnClearState			= 0;
		pfnUpdate				= 0;
		pfnUpdateRenderables	= 0;
		pfnRefreshRenderables	= 0;

		xRenderable2DInit(&m_xRenderable);

		m_strName				= 0;
		m_strLClickCommand		= 0;
		m_strLClickParams		= 0;
		m_strDblClickCommand	= 0;
		m_strDblClickParams		= 0;
		m_strRClickCommand		= 0;
		m_strRClickParams		= 0;
		m_strEnterPressCommand	= 0;
		m_strEnterPressParams	= 0;
		m_strHoverCommand		= 0;
		m_strUnHoverCommand		= 0;
		m_strDragCommand		= 0;
		m_strDropCommand		= 0;
		m_bHasFocus				= 0;
		m_bDisabled				= 0;
		m_bHovered				= 0;
		m_dwHoverDelay			= 500;
		m_dwHoverStateTime		= 0;
		m_bSelected				= 0;
		m_bShouldSelect			= 0;
		m_bClicked				= 0;
		m_bStretch				= 0;
		m_iResult				= 0;
		m_pSceneGraph			= 0;
		m_pFont					= 0;
		m_pParent				= 0;
		m_pParser				= 0;
		m_pGUI					= 0;
		m_iTabIndex				= -1;
		m_pSelectionGroup		= 0;
		m_pHoverSound			= 0;
		m_pClickSound			= 0;
		m_pDblClickSound		= 0;
		m_pEnterPressSound		= 0;
	}

			void		LoadAnimation();
			int			AddChildObject(int iType);
			int			AddChildObject(char *strType);
			xGUIObject*	GetChildByName(const char *strName);
			void		SetName(const char *strName) { if(!strName) return; m_strName = new char[strlen(strName) + 1]; strcpy(m_strName, strName); }
			void		SetHidden(bool bHidden) { m_xRenderable.m_bVisible = !bHidden; }
			void		SetDepth(float fDepth) { m_xRenderable.m_fDepth = fDepth; }
			void		SetX(float fX) { m_xRenderable.m_fX = fX; }
			void		SetY(float fY) { m_xRenderable.m_fY = fY; }
			void		SetWidth(float fWidth) { m_xRenderable.m_fWidth = fWidth; }
			void		SetHeight(float fHeight) { m_xRenderable.m_fHeight = fHeight; }
			void		SetRotation(float fRotation) { m_xRenderable.m_fRotation = fRotation; }
			void		SetScale(float fScaleX, float fScaleY) { m_xRenderable.m_fScaleX = fScaleX; m_xRenderable.m_fScaleY = fScaleY; }
			void		SetColor(xColor *pColor) { xColorCopy(&m_xRenderable.m_xColor, pColor); }
			void		SetAlpha(float fAlpha) { m_xRenderable.m_xColor.A = fAlpha; }
			int			GetCommandAndParams(char *strCommandOut, char** &strParamsOut, char *strLine);
			bool		GetAbsoluteFocus() { if(m_pParent) return m_bHasFocus || m_pParent->GetAbsoluteFocus(); return m_bHasFocus; }
			bool		GetHidden() { return !m_xRenderable.m_bVisible; }
			bool		GetAbsoluteHidden() { if(m_pParent) return GetHidden() || m_pParent->GetAbsoluteHidden(); return GetHidden(); }
			float		GetDepth() { return m_xRenderable.m_fDepth; }
			float		GetAbsoluteDepth() { if(m_pParent) return m_pParent->GetAbsoluteDepth() + GetDepth(); return GetDepth(); }
			float		GetX() { return m_xRenderable.m_fX; }
			float		GetY() { return m_xRenderable.m_fY; }
			float		GetAbsoluteX() { if(m_pParent) return m_pParent->GetAbsoluteX() + (GetX() * m_pParent->GetAbsoluteScaleX()); return GetX(); }
			float		GetAbsoluteY() { if(m_pParent) return m_pParent->GetAbsoluteY() + (GetY() * m_pParent->GetAbsoluteScaleY()); return GetY(); }
			float		GetWidth() { return m_xRenderable.m_fWidth; }
			float		GetHeight() { return m_xRenderable.m_fHeight; }
			float		GetScaleX() { return m_xRenderable.m_fScaleX; }
			float		GetScaleY() { return m_xRenderable.m_fScaleX; }
			float		GetAbsoluteScaleX() { if(m_pParent) return m_pParent->GetAbsoluteScaleX() * GetScaleX(); return GetScaleX(); }
			float		GetAbsoluteScaleY() { if(m_pParent) return m_pParent->GetAbsoluteScaleY() * GetScaleY(); return GetScaleY(); }
			float		GetRotation() { if(m_pParent) return m_pParent->GetRotation() + m_xRenderable.m_fRotation; return m_xRenderable.m_fRotation; }
			xColor*		GetAbsoluteColor(xColor *pOutColor) { if(m_pParent) return xColorMultiply(pOutColor, m_pParent->GetAbsoluteColor(pOutColor), &m_xRenderable.m_xColor); xColorCopy(pOutColor, &m_xRenderable.m_xColor); return pOutColor; }
			float		GetAlpha() { return m_xRenderable.m_xColor.A; }
			void		SetEnabled(bool bEnable) { m_bDisabled = !bEnable; }
	inline	void		SelectPrevious();
	inline	void		SelectNext();
			void		UpdateRenderables() { pfnUpdateRenderables(this); }
			void		UpdateDepth() { pfnUpdateDepth(this); }
			void		RefreshRenderables() { pfnRefreshRenderables(this); }

			void Init();
			int  Init(const char *strThemeName);
	virtual	int  LoadObject(const char *strThemeName);
	virtual void Shutdown();

	//Use function pointers instead of virtual functions
	void (*pfnPreUpdate)(xGUIObject *pThis, DWORD dwTime);
	void (*pfnClearState)(xGUIObject *pThis, DWORD dwTime);
	void (*pfnUpdate)(xGUIObject *pThis, double dTime);
	void (*pfnUpdateRenderables)(xGUIObject *pThis);
	void (*pfnUpdateDepth)(xGUIObject *pThis);
	void (*pfnRefreshRenderables)(xGUIObject *pThis);

			int	   m_iResult;
			char  *m_strName;
	const	char  *m_strThemeName;
			bool   m_bHasFocus;
			bool   m_bDisabled;
			bool   m_bStretch;
			bool   m_bHovered;
			DWORD  m_dwHoverDelay;
			DWORD  m_dwHoverStateTime;
			bool   m_bRanHoveredCommand;
			bool   m_bSelected;
			bool   m_bShouldSelect;
			bool   m_bClicked;
			int    m_iTabIndex;

	//Commands
	char  *m_strLClickCommand;
	char **m_strLClickParams;
	char  *m_strDblClickCommand;
	char **m_strDblClickParams;
	char  *m_strRClickCommand;
	char **m_strRClickParams;
	char  *m_strEnterPressCommand;
	char **m_strEnterPressParams;
	char  *m_strHoverCommand;
	char **m_strHoverParams;
	char  *m_strUnHoverCommand;
	char **m_strUnHoverParams;
	char  *m_strDragCommand;
	char **m_strDragParams;
	char  *m_strDropCommand;
	char **m_strDropParams;

	xRenderable2D		 m_xRenderable;
	xSceneGraph			*m_pSceneGraph;
	xFont				*m_pFont;
	xGUIObject			*m_pParent;
	xParser				*m_pParser;
	xGUI				*m_pGUI;
	xGUIObjectList		 m_xChildObjectList;
	xGUIWindowList		*m_pOwningWindow;
	xAnimation			 m_xAnimation;
	xGUISelectionGroup	*m_pSelectionGroup;

	// Sounds
	xSound	*m_pHoverSound;
	xSound	*m_pClickSound;
	xSound	*m_pDblClickSound;
	xSound	*m_pEnterPressSound;
};

void xGUIObject::SelectPrevious()
{
	xGUIObjectList *sellist = m_pSelectionGroup->m_xSelectableList.m_pNext;

	while(sellist)
	{
		if(sellist->m_pData == this)
			break;

		sellist = sellist->m_pNext;
	}

	if(sellist && sellist->m_pPrev && sellist->m_pPrev->m_pData)
	{
		sellist->m_pData->m_bSelected = false;
		sellist->m_pPrev->m_pData->m_bShouldSelect = true;
	}
}

void xGUIObject::SelectNext()
{
	xGUIObjectList *sellist = m_pSelectionGroup->m_xSelectableList.m_pNext;

	while(sellist)
	{
		if(sellist->m_pData == this)
			break;

		sellist = sellist->m_pNext;
	}

	if(sellist && sellist->m_pNext && sellist->m_pNext->m_pData)
	{
		sellist->m_pData->m_bSelected = false;
		sellist->m_pNext->m_pData->m_bShouldSelect = true;
	}
}

FORCE_INLINE void xGUIObjectSyncChildRenderables(xGUIObject *pObject)
{
	xGUIObjectList *curobj = pObject->m_xChildObjectList.m_pNext;
	while(curobj)
	{
		curobj->m_pData->UpdateRenderables();
		curobj = curobj->m_pNext;
	}
}

#endif