/////////////////////////////////////////////////////////////////
// Name: xGUIText.h
/////////////////////////////////////////////////////////////////
// Description:
// - This handles text for the GUI.
/////////////////////////////////////////////////////////////////
// - 07.11.2004, Note, DayleFlowers (Xienen)
//   File created
/////////////////////////////////////////////////////////////////
#ifndef XGUITEXT_H_
#define XGUITEXT_H_

#include "xGUIObject.h"
#include "xRenderable2DText.h"

enum xGUITextAlignment
{
	PE_GUITEXT_ALIGNMENT_LEFT = PE_TEXTALIGN_LEFT,
	PE_GUITEXT_ALIGNMENT_CENTER = PE_TEXTALIGN_CENTER,
	PE_GUITEXT_ALIGNMENT_RIGHT = PE_TEXTALIGN_RIGHT,
	PE_GUITEXT_ALIGNMENT_JUSTIFIED = PE_TEXTALIGN_JUSTIFIED
};

void xGUITextPreUpdate(xGUIObject* pThis, DWORD dwTime);
void xGUITextClearState(xGUIObject* pThis, DWORD dwTime);
void xGUITextUpdate(xGUIObject* pThis, double dTime);
void xGUITextUpdateRenderable(xGUIObject* pThis);
void xGUITextUpdateDepth(xGUIObject* pThis);
void xGUITextRefreshRenderables(xGUIObject* pThis);

class xGUIText : public xGUIObject
{
public:
	xGUIText()
	{
		pfnPreUpdate			= &xGUITextPreUpdate;
		pfnClearState			= &xGUITextClearState;
		pfnUpdate				= &xGUITextUpdate;
		pfnUpdateRenderables	= &xGUITextUpdateRenderable;
		pfnUpdateDepth			= &xGUITextUpdateDepth;
		pfnRefreshRenderables	= &xGUITextRefreshRenderables;

		m_xAlignment			= PE_GUITEXT_ALIGNMENT_LEFT;
		m_iMaxLength			= -1;
		m_strBuffer				= 0;
		m_iBufferLen			= 0;
		m_bMasked				= 0;
		m_pSelectedEmitter		= 0;
		m_bUniqueColorData		= 0;
		m_bColorized			= 0;

		m_xDisabledColor.R = m_xDisabledColor.G = m_xDisabledColor.B = m_xDisabledColor.A = 1.0;
		m_xHoverColor.R = m_xHoverColor.G = m_xHoverColor.B = m_xHoverColor.A = 1.0;
	}

	void InitRenderable();
	int  LoadObject(const char *strThemeName = NULL);
	void Shutdown();

	inline void SetFont(xFont *pFont) { m_pFont = pFont; m_xText.m_pFont = pFont; }
	inline void SetText(const char *strText);
	inline void SetAlignment(xGUITextAlignment xNewAlignment) { m_xAlignment = xNewAlignment; UpdateRenderable(); }
	inline void UpdateAlignedPosition();
	inline void UpdateRenderable();
	inline void UpdateColor(xColor *pColor);

	xRenderable2DText	 m_xText;
	xGUITextAlignment	 m_xAlignment;
	int					 m_iMaxLength;
	char				*m_strBuffer;
	int					 m_iBufferLen;
	bool				 m_bMasked;
	xEmitter			*m_pSelectedEmitter;
	char				 m_strSelectedEmitter[64];
	bool				 m_bUniqueColorData;
	xColor				 m_xNormalColor;
	xColor				 m_xDisabledColor;
	xColor				 m_xHoverColor;
	bool				 m_bColorized;
	xColor				 m_xOutlineColor;
};

void xGUIText::SetText(const char *strText)
{
	if(!strText)
		return;
	
	if(m_iMaxLength <= 0 || !m_strBuffer)
	{
		if(m_strBuffer)
			delete [] m_strBuffer;
		
		m_strBuffer = new char[strlen(strText) + 1];
	}
	
	strcpy(m_strBuffer, strText);
	
	if(m_xAlignment != PE_GUITEXT_ALIGNMENT_JUSTIFIED)
		m_xRenderable.m_fWidth = xFontGetWidth(m_pFont, m_strBuffer) * GetAbsoluteScaleX();
	
	UpdateRenderable();
}

void xGUIText::UpdateAlignedPosition()
{
	float xchange, ychange;

	if(GetRotation() == 0.0)
	{
		if(!m_pFont || m_xAlignment == PE_GUITEXT_ALIGNMENT_LEFT || m_xAlignment == PE_GUITEXT_ALIGNMENT_JUSTIFIED)
			m_xText.m_xRenderable.m_xRenderable2D.m_fX = GetAbsoluteX();
		else if(m_xAlignment == PE_GUITEXT_ALIGNMENT_CENTER)
			m_xText.m_xRenderable.m_xRenderable2D.m_fX = GetAbsoluteX() - floor(m_xRenderable.m_fWidth / 2.0);
		else
			m_xText.m_xRenderable.m_xRenderable2D.m_fX = GetAbsoluteX() - m_xRenderable.m_fWidth;
	}
	else
	{
		xchange = m_xRenderable.m_fWidth * GetAbsoluteScaleX() * cos(GetRotation() * (PI / 180.0f));
		ychange = m_xRenderable.m_fWidth * GetAbsoluteScaleX() * sin(GetRotation() * (PI / 180.0f));

		if(m_xAlignment == PE_GUITEXT_ALIGNMENT_LEFT || m_xAlignment == PE_GUITEXT_ALIGNMENT_JUSTIFIED)
		{
			m_xText.m_xRenderable.m_xRenderable2D.m_fX = GetAbsoluteX();
			m_xText.m_xRenderable.m_xRenderable2D.m_fY = GetAbsoluteY();
		}
		else if(m_xAlignment == PE_GUITEXT_ALIGNMENT_CENTER)
		{
			m_xText.m_xRenderable.m_xRenderable2D.m_fX = GetAbsoluteX() - xchange * 0.5f;
			m_xText.m_xRenderable.m_xRenderable2D.m_fY = GetAbsoluteY() + ychange * 0.5f;
		}
		else
		{
			m_xText.m_xRenderable.m_xRenderable2D.m_fX = GetAbsoluteX() - xchange;
			m_xText.m_xRenderable.m_xRenderable2D.m_fY = GetAbsoluteY() + ychange;
		}
	}
}

void xGUIText::UpdateRenderable()
{
	UpdateAlignedPosition();
	xRenderable2DTextSetAlignment(&m_xText, (xTextAlignment)m_xAlignment, m_xRenderable.m_fWidth);
	xRenderable2DTextSetText(&m_xText, m_pSceneGraph, m_strBuffer, m_bUniqueColorData);
}

void xGUIText::UpdateColor(xColor *pColor)
{
	xColor tempcolor;

	SetColor(pColor);
	xColorCopy(&m_xNormalColor, pColor);
	xRenderable2DTextUpdateColor(&m_xText, &m_xNormalColor);
}

#endif