#ifndef XGAMENETWORK_H_
#define XGAMENETWORK_H_

#include "xGameOptions.h"

enum xNetPacketType
{
	BB_NETPACKETTYPE_NONE = 0,
	BB_NETPACKETTYPE_CLIENTHANDSHAKE,
	BB_NETPACKETTYPE_SERVERHANDSHAKE,
	BB_NETPACKETTYPE_SECURITY,
	BB_NETPACKETTYPE_APPEARANCE,
	BB_NETPACKETTYPE_READY,
	BB_NETPACKETTYPE_ADDEDBLOCK,
	BB_NETPACKETTYPE_FINISHEDPIECE,
	BB_NETPACKETTYPE_PUSHEDDOWNPIECE,
	BB_NETPACKETTYPE_DROPPEDPIECE,
	BB_NETPACKETTYPE_BLOCKSCLEARED,
	BB_NETPACKETTYPE_FAILEDTOADDBLOCK,
	BB_NETPACKETTYPE_MISSEDBEAT,
	BB_NETPACKETTYPE_MISSEDCOMBO,
	BB_NETPACKETTYPE_REMATCH,
	BB_NETPACKETTYPE_NEXTSONG,
	BB_NETPACKETTYPE_ACCEPT,
	BB_NETPACKETTYPE_MESSAGE,
	BB_NETPACKETTYPE_MAX
};

#define BB_PACKET_NAMESIZE		25
#define	BB_PACKET_BGSIZE		25
#define BB_PACKET_SONGSIZE		25
#define BB_PACKET_MD5SIZE		80
#define BB_PACKET_MESSAGESIZE	80

struct xBBNetPacketClientHandshake
{
	uint8_t	m_ucPacketType;
	uint8_t	m_ucNameSize;
	char	m_strName[BB_PACKET_NAMESIZE];
	int		m_iRank;
	char	m_cNullTerminator;
};

struct xBBNetPacketServerHandshake
{
	uint8_t	m_ucPacketType;
	uint8_t	m_ucNameSize;
	char	m_strName[BB_PACKET_NAMESIZE];
	uint8_t	m_ucBackgroundSize;
	char	m_strBackground[BB_PACKET_BGSIZE];
	uint8_t	m_ucSongSize;
	char	m_strSong[BB_PACKET_SONGSIZE];
	uint8_t	m_ucBeatLevel;
	int		m_iRank;
	char	m_cNullTerminator;
};

struct xBBNetPacketSecurity
{
	uint8_t m_ucPacketType;
	char	m_strEncryptedMD5s[BB_PACKET_MD5SIZE];
};

struct xBBNetPacketAppearance
{
	uint8_t m_ucPacketType;
};

struct xBBNetPacketReady
{
	uint8_t m_ucPacketType;
};

struct xBBNetPacketAddedBlock
{
	uint8_t	m_ucPacketType;
	int16	m_i16ComboCount;
	float	m_fNewBarPosition;
};

struct xBBNetPacketFinishedPiece
{
	uint8_t	m_ucPacketType;
	uint8_t	m_ucBlocksInPiece;
	float	m_fNewBarPosition;
};

struct xBBNetPacketPushedDownPiece
{
	uint8_t	m_ucPacketType;
	uint8_t	m_ucDropDistance;
	float	m_fNewBarPosition;
};

struct xBBNetPacketDroppedPiece
{
	uint8_t	m_ucPacketType;
	uint8_t	m_ucDropDistance;
	float	m_fNewBarPosition;
};

struct xBBNetPacketBlocksCleared
{
	uint8_t	m_ucPacketType;
	uint8_t	m_ucBlockCount;
	float	m_fNewBarPosition;
};

struct xBBNetPacketFailedToAddBlock
{
	uint8_t	m_ucPacketType;
	float	m_fNewBarPosition;
};

struct xBBNetPacketMissedBeat
{
	uint8_t	m_ucPacketType;
	int16	m_i16ComboCount;
	float	m_fNewBarPosition;
};

struct xBBNetPacketMissedCombo
{
	uint8_t	m_ucPacketType;
	int16	m_i16ComboCount;
	float	m_fNewBarPosition;
};

struct xBBNetPacketRematch
{
	uint8_t	m_ucPacketType;
};

struct xBBNetPacketNextSong
{
	uint8_t	m_ucPacketType;
	uint8_t	m_ucBeatLevel;
	uint8_t	m_ucNextSongSize;
	char	m_strNextSong[BB_PACKET_SONGSIZE];
};

struct xBBNetPacketAccept
{
	uint8_t	m_ucPacketType;
};

struct xBBNetPacketTextMessage
{
	uint8_t	m_ucPacketType;
	uint8_t	m_ucTextMessageSize;
	char	m_strTextMessage[BB_PACKET_MESSAGESIZE];
};

class xGameNetwork
{
public:
	static void SendClientHandshake(const char *strName, int iRank);
	static void SendServerHandshake(const char *strName, const char *strBackground, const char *strSong, int iBeatLevel, int iRank);
	static void SendSecurity(DWORD dwTime, const char *strExeMD5, const char *strBeatsMD5);
	static void SendAppearance();
	static void SendReady();
	static void SendAddedBlock(int iComboCount, float fNewBarPosition);
	static void SendFinishedPiece(int iBlocksInPiece, float fNewBarPosition);
	static void SendPushedDownPiece(int iDropDistance, float fNewBarPosition);
	static void SendDroppedPiece(int iDropDistance, float fNewBarPosition);
	static void SendBlocksCleared(int iBlockCount, float fNewBarPosition);
	static void SendFailedToAddBlock(float fNewBarPosition);
	static void SendMissedBeat(int iComboCount, float fNewBarPosition);
	static void SendMissedCombo(int iComboCount, float fNewBarPosition);
	static void SendRematch();
	static void SendNextSong(char *strNextSong, int iBeatLevel);
	static void SendAccept();
	static void SendTextMessage(char *strMessage);

	static bool ReceivedNetPacket(int &iNetPacketType);
	static void ReceiveClientHandshake(char strName[BB_PACKET_SONGSIZE], int &iRank);
	static void ReceiveServerHandshake(char strName[BB_PACKET_SONGSIZE], char strBackground[BB_PACKET_SONGSIZE], char strSong[BB_PACKET_SONGSIZE], int &iBeatLevel, int &iRank);
	static void ReceiveSecurity(char strExeMD5[33], char strBeatsMD5[33]);
	static void ReceiveAppearance();
	static void ReceiveReady();
	static void ReceiveAddedBlock(int &iComboCount, float &fNewBarPosition);
	static void ReceiveFinishedPiece(int &iBlocksInPiece, float &fNewBarPosition);
	static void ReceivePushedDownPiece(int &iDropDistance, float &fNewBarPosition);
	static void ReceiveDroppedPiece(int &iDropDistance, float &fNewBarPosition);
	static void ReceiveBlocksCleared(int &iBlockCount, float &fNewBarPosition);
	static void ReceiveFailedToAddBlock(float &fNewBarPosition);
	static void ReceiveMissedBeat(int &iComboCount, float &fNewBarPosition);
	static void ReceiveMissedCombo(int &iComboCount, float &fNewBarPosition);
	static void ReceiveRematch();
	static void ReceiveNextSong(char strNextSong[BB_PACKET_SONGSIZE], xGameDifficulty &iBeatLevel);
	static void ReceiveAccept();
	static void ReceiveTextMessage(char strMessage[BB_PACKET_MESSAGESIZE]);

private:
	static char m_strBuffer[MAX_NETWORK_BUFFERSIZE + 1];
	static int  m_iBufferOffset;
};

#endif