/////////////////////////////////////////////////////////////////
// Name: xGameOptions.h
/////////////////////////////////////////////////////////////////
// Description:
// - Holds all information for a loaded level and/or current game
/////////////////////////////////////////////////////////////////
// - 7.15.2004, Update, Dayle Flowers(Xienen)
//   File Created
/////////////////////////////////////////////////////////////////
#ifndef XGAMEOPTIONS_H_
#define XGAMEOPTIONS_H_

enum xGameDifficulty
{
	BB_DIFFICULTY_TUTORIAL = 0,
	BB_DIFFICULTY_NOVICE,
	BB_DIFFICULTY_ADVANCED,
	BB_DIFFICULTY_EXPERT
};

struct xPreloadGameOptions
{
	DWORD m_dwBronzeMedalScore;	//Score that must be beaten to achieve the Bronze Medal
	DWORD m_dwSilverMedalScore;	//Score that must be beaten to achieve the Silver Medal
	DWORD m_dwGoldMedalScore;	//Score that must be beaten to achieve the Gold Medal
};

struct xGameOptions
{
	void Delete()
	{
		m_iMaxPieces					= 0;
		m_iLevelIndex					= -1;
		m_bSkipPrePopup					= 0;
		m_strPrePopup[0]				= '\0';
		m_strSenseiPrePopup[0]			= '\0';
		m_strSenseiPrePopupVoice[0]		= '\0';
		m_dwEnemyFirstMoveDelay			= 1;
		m_dwEnemyMoveDelay				= 1;
		m_dwMinEnemyMoveDelay			= 1;
		m_iEnemyDamage					= 10000000;
		m_bTutorial						= 0;
		m_bBoss							= 0;
		m_iBossIndex					= -1;
		m_iCashPrize					= 50;
		m_dwLevelCompletionTime			= -1;
		m_dwBronzeMedalScore			= 0;
		m_dwSilverMedalScore			= 0;
		m_dwGoldMedalScore				= 0;
		m_dwStartOffset					= 0;
		m_fBPM							= 0.1;
		m_bLoopSong						= 0;
		m_iDifficultyLevel				= BB_DIFFICULTY_NOVICE;
		m_iNumColors					= 3;
		m_iNumRandomColors				= 3;
		m_iMaxPieces					= 0;
		m_bShowMissed					= 1;
		m_bCombosRequired				= 0;
		m_bAlwaysAddBlock				= 0;
		m_iBeatMarkerShowTime			= 350;
		m_iHitTime						= 350;
		m_bShowComboCounter				= 0;
		m_bShowClearCounter				= 0;
		memset(m_strLevelFile, 0, 255);
		memset(m_strNextLevelFile, 0, 255);
		memset(m_strSongName, 0, 255);
		memset(m_strBGFile, 0, 255);
		memset(m_strEnemyName, 0, 21);
		memset(m_strServerAddress, 0, 16);
	}
	
	int   m_iLevelIndex;				//Index of this level in the "Story Mode"
	char  m_strLevelFile[256];			//Location of .gbl File
	char  m_strNextLevelFile[256];		//Location of .gbl File to load after this one is done
	char  m_strSongName[256];			//Directory under the Music directory which contains the song for this level
	char  m_strSongDisplayName[256];	//Displayable name of the song
	char  m_strSongArtistName[256];		//Displayable name of Artist that composed the song
	char  m_strBGFile[256];				//Location of Background Image/Movie
	bool  m_bSkipPrePopup;				//If this is set, no PrePopups will ever show
	char  m_strPrePopup[1024];			//Full text to be shown in the PrePopup Window
	char  m_strSenseiPrePopup[1024];	//Full text to be shown in the Sensei PrePopup Window
	char  m_strSenseiPrePopupVoice[256];//Location of the Sensei's PrePopup Voice wave file
	char  m_strEnemyName[24];			//Name of Enemy's directory
	char  m_strEnemyDisplayName[24];	//Name of Enemy as displayed to players
	DWORD m_dwEnemyFirstMoveDelay;		//Milliseconds before the Enemy executes his first special move
	DWORD m_dwEnemyMoveDelay;			//Milliseconds between Enemy's special moves
	DWORD m_dwMinEnemyMoveDelay;		//Minimum milliseconds between Enemy's special moves(used for randomization)
	int   m_iEnemyDamage;				//Amount of Crowd Favoritism(0-1000, prolly shouldn't go over 300)
	bool  m_bTutorial;					//Whether or not this is the Tutorial level(not set for the Tutorial Battle)
	bool  m_bBoss;						//Whether or not this is a Boss Battle(if it is, BossIndex should be set to 1-5)
	int   m_iBossIndex;					//Used to determine what kind of special moves are required to do well against each Boss
	DWORD m_dwBossMoveMinTime;			//Minimum amount of time between Boss Moves(basically just the randomly changing block colors)
	DWORD m_dwBossMoveRandTime;			//Random extra amount of time between Boss Moves(basically just the randomly changing block colors)
	int   m_iCashPrize;					//Amount of money the player gets for completing this level
	DWORD m_dwLevelCompletionTime;		//Amount of time the level is expected to be completed in(High Score stops going up once this has elapsed)
	DWORD m_dwBronzeMedalScore;			//Score that must be beaten to achieve the Bronze Medal
	DWORD m_dwSilverMedalScore;			//Score that must be beaten to achieve the Silver Medal
	DWORD m_dwGoldMedalScore;			//Score that must be beaten to achieve the Gold Medal
	DWORD m_dwStartOffset;				//Offset from 0 to start song(allows fine tuning of song/arrow alignment)
	int   m_iNumColors;					//Number of Colors available on this level
	int   m_iNumRandomColors;			//Number of Colors that can be used when determining a random color(upon missing the beat or hitting the wrong color key)
	float m_fBPM;						//Beats Per Minute
	bool  m_bLoopSong;					//If true, the song and beats loop forever
	xGameDifficulty m_iDifficultyLevel;	//1 = Novice, 2 = Advanced, 3 = Expert
	int   m_iDifficultyLevels[5];		//1 = Novice, 2 = Advanced, 3 = Expert
	int   m_iMaxPieces;					//Number of pieces they're allowed to use to complete level
	bool  m_bShowMissed;				//Show Missed when hitting outside 250ms
	bool  m_bCombosRequired;			//Require Combos?
	bool  m_bAlwaysAddBlock;			//Add block every time you can?
	int   m_iBeatMarkerShowTime;		//How long, in milliseconds, that each Beat Marker should render on the screen
	int   m_iHitTime;					//Amount of time in milliseconds an arrow can be hit
	char  m_strServerAddress[16];		//IP of the Server, even if we are the server(without the Port)
	int   m_iServerPort;				//Port of the Server, even if we are the server
	bool  m_bShowComboCounter;			//Challenge support for showing Combo count
	bool  m_bShowClearCounter;			//Challenge support for showing Blocks Cleared count
};

#endif