/////////////////////////////////////////////////////////////////
// Name: xInput.h
/////////////////////////////////////////////////////////////////
// Description:
// - Core input interface to abstart platform specific input
/////////////////////////////////////////////////////////////////
// - 24,10.2004, Update, Arto Ruotsalainen (Arex)
//   Added to xInput function GetCharacterPressed, it returns
//   pressed character.
//
// - 7.16.2004, Update, Chris Morris(Orb)
//   Added the escape xbutton.
/////////////////////////////////////////////////////////////////
// - 7.6.2004, Update, Arto Ruotsalainen(Arex)
//   Comment stuff added to begin of the file.
//   Add new updates/fixes above this, so the old updates
//   will be at the bottom.
/////////////////////////////////////////////////////////////////
#ifndef XINPUT_H_
#define XINPUT_H_

#define PE_REPEAT_MAX_SPACING	400
#define PE_REPEAT_MIN_SPACING	100
typedef unsigned long DWORD;

class xButton
{
public:
		  ~xButton() { m_iKeyState = 0; m_bEaten = false; }
		  void  UpdateState(bool bDown, DWORD dwTime) { if(!bDown) { m_iKeyState = 0; m_bEaten = false; } else { if(m_iKeyState == 0) { m_dwPressTime = m_dwLastRepeatTime = dwTime; m_dwCurRepeatTime = PE_REPEAT_MAX_SPACING; m_iKeyState = 1;} else m_iKeyState = 2; } }
	bool  IsDown() { return (m_iKeyState == 1 || m_iKeyState == 2); }
	bool  JustPressed(bool bCheckEaten = false) { return m_iKeyState == 1 && (!bCheckEaten || !m_bEaten); }
	void  SetHeld() { m_iKeyState = 2; }
	bool  IsHeld() { return m_iKeyState == 2; }
	bool  ShouldRepeat(DWORD dwTime) { if((dwTime - m_dwLastRepeatTime) >= m_dwCurRepeatTime && m_iKeyState > 0) { m_dwCurRepeatTime -= PE_REPEAT_MAX_SPACING / 4; if(m_dwCurRepeatTime < PE_REPEAT_MIN_SPACING) m_dwCurRepeatTime = PE_REPEAT_MIN_SPACING; m_dwLastRepeatTime = dwTime; return true; } return false;  };
	DWORD PressTime() { return m_dwPressTime; }
	void  SetPressTime(DWORD dwTime) { m_dwPressTime = dwTime; }
	void  Eaten() { if(JustPressed()) m_bEaten = true; }
	
	int   GetState() { return m_iKeyState; }
	void  SetState(int iNewState) { m_iKeyState = iNewState; }

private:
	int		m_iKeyState;
	bool	m_bEaten;
	DWORD	m_dwPressTime;
	DWORD	m_dwLastRepeatTime;
	DWORD	m_dwCurRepeatTime;
};

class xJoystick
{
public:
	xJoystick()
	{
		// Default variables
		m_pNext	= NULL;
		m_pPrev	= NULL;
		m_lJoyX	= 0;
		m_lJoyY	= 0;
		m_lJoyZ	= 0;
	}

	~xJoystick()
	{
		if(m_pNext)
			delete m_pNext;

		if(m_pPrev)
			delete m_pPrev;
	}

	long	  m_lJoyX, m_lJoyY, m_lJoyZ;
	xButton	  m_button1, m_button2, m_button3, m_button4, m_button5, m_button6, m_button7,
			  m_button8, m_button9, m_button10, m_button11, m_button12, m_button13, m_button14;
	xJoystick *m_pNext;
	xJoystick *m_pPrev;
};

class xInput
{
public:
	xInput()
	{
		m_bIgnoreInput		= 0;
		m_pJoysticks		= 0;
		m_bJoystickSupport	= 0;
		m_lMouseX			= 0;
		m_lMouseY			= 0;
		m_lMouseZ			= 0;
		m_lWindowOffsetX	= 0;
		m_lWindowOffsetY	= 0;
		m_bCaps				= 0;
		m_bCapitals			= 0;
	}

	int  Init() { return PE_RES_NOERROR; }
	void PostUpdate();
	void Shutdown() {}

	void HandleKeyboardKeyPressed(unsigned char cKey, bool bPressed);
	void HandleMouseButtonPressed(int iMouseButtonIndex, bool bPressed);

	xButton*	GetKeyFromString(char *strKeyString);
	char*		GetStringFromKey(xButton* pKey);

	bool m_bIgnoreInput;

	xButton m_keyA, m_keyB, m_keyC, m_keyD, m_keyE, m_keyF, m_keyG, m_keyH, m_keyI, m_keyJ, m_keyK, m_keyL, m_keyM, m_keyN, m_keyO, m_keyP,
		    m_keyQ, m_keyR, m_keyS, m_keyT, m_keyU, m_keyV, m_keyW, m_keyX, m_keyY, m_keyZ, 
		    m_key1, m_key2, m_key3, m_key4, m_key5, m_key6, m_key7, m_key8, m_key9, m_key0,
		    m_keyEscape, m_keyF1, m_keyF2, m_keyF3, m_keyF4, m_keyF5, m_keyF6, m_keyF7, m_keyF8, m_keyF9, m_keyF10, m_keyF11, m_keyF12,
		    m_keyTilde, m_keyDash, m_keyEquals, m_keyLBracket, m_keyRBracket, m_keySlash, m_keySemiColon, m_keyApostrophe,
		    m_keyComma, m_keyPeriod, m_keyBackSlash, m_keySpace, m_keyBackspace,
		    m_keyTab, m_keyCaps, m_keyLShift, m_keyRShift, m_keyLControl, m_keyLWindows, m_keyLALT, m_keyRALT, m_keyRWindows, m_keyRControl,
		    m_keyDelete, m_keyInsert, m_keyHome, m_keyEnd, m_keyPageDown, m_keyPageUp, m_keyEnter,
		    m_keyLeft, m_keyRight, m_keyUp, m_keyDown, 
		    m_keyNumLock, m_keyNumDivide, m_keyNumMultiply, m_keyNumMinus, m_keyNumPlus, m_keyNumEnter, m_keyNumDecimal,
		    m_keyNum0, m_keyNum1, m_keyNum2, m_keyNum3, m_keyNum4, m_keyNum5, m_keyNum6, m_keyNum7, m_keyNum8, m_keyNum9;

	long	m_lWindowOffsetX, m_lWindowOffsetY;
	long    m_lMouseX, m_lMouseY, m_lMouseZ;
	xButton m_buttonMouseLeft, m_buttonMouseRight, m_buttonMouseMiddle, m_buttonMouseExtra1, 
			m_buttonMouseExtra2, m_buttonMouseExtra3, m_buttonMouseExtra4, m_buttonMouseExtra5;
	bool	m_bCaps, m_bCapitals;

	FORCE_INLINE xButton* GetKeyPressed();
	FORCE_INLINE char GetCharacterPressed(DWORD dwTime, bool bAllowSpace);

	xJoystick *m_pJoysticks;

	bool m_bJoystickSupport;
protected:
	int  m_iResult;
};

xButton* xInput::GetKeyPressed()
{
	if(m_keyA.JustPressed())
		return &m_keyA;

	if(m_keyB.JustPressed())
		return &m_keyB;

	if(m_keyC.JustPressed())
		return &m_keyC;

	if(m_keyD.JustPressed())
		return &m_keyD;

	if(m_keyE.JustPressed())
		return &m_keyE;

	if(m_keyF.JustPressed())
		return &m_keyF;

	if(m_keyG.JustPressed())
		return &m_keyG;

	if(m_keyH.JustPressed())
		return &m_keyH;

	if(m_keyI.JustPressed())
		return &m_keyI;

	if(m_keyJ.JustPressed())
		return &m_keyJ;

	if(m_keyK.JustPressed())
		return &m_keyK;

	if(m_keyL.JustPressed())
		return &m_keyL;

	if(m_keyM.JustPressed())
		return &m_keyM;

	if(m_keyN.JustPressed())
		return &m_keyN;

	if(m_keyO.JustPressed())
		return &m_keyO;

	if(m_keyP.JustPressed())
		return &m_keyP;

	if(m_keyQ.JustPressed())
		return &m_keyQ;

	if(m_keyR.JustPressed())
		return &m_keyR;

	if(m_keyS.JustPressed())
		return &m_keyS;

	if(m_keyT.JustPressed())
		return &m_keyT;

	if(m_keyU.JustPressed())
		return &m_keyU;

	if(m_keyV.JustPressed())
		return &m_keyV;

	if(m_keyW.JustPressed())
		return &m_keyW;

	if(m_keyX.JustPressed())
		return &m_keyX;

	if(m_keyY.JustPressed())
		return &m_keyY;

	if(m_keyZ.JustPressed())
		return &m_keyZ;

	if(m_key0.JustPressed())
		return &m_key0;

	if(m_key1.JustPressed())
		return &m_key1;

	if(m_key2.JustPressed())
		return &m_key2;

	if(m_key3.JustPressed())
		return &m_key3;

	if(m_key4.JustPressed())
		return &m_key4;

	if(m_key5.JustPressed())
		return &m_key5;

	if(m_key6.JustPressed())
		return &m_key6;

	if(m_key7.JustPressed())
		return &m_key7;

	if(m_key8.JustPressed())
		return &m_key8;

	if(m_key9.JustPressed())
		return &m_key9;

	if(m_keyEscape.JustPressed())
		return &m_keyEscape;

	if(m_keyF1.JustPressed())
		return &m_keyF1;

	if(m_keyF2.JustPressed())
		return &m_keyF2;

	if(m_keyF3.JustPressed())
		return &m_keyF3;

	if(m_keyF4.JustPressed())
		return &m_keyF4;

	if(m_keyF5.JustPressed())
		return &m_keyF5;

	if(m_keyF6.JustPressed())
		return &m_keyF6;

	if(m_keyF7.JustPressed())
		return &m_keyF7;

	if(m_keyF8.JustPressed())
		return &m_keyF8;

	if(m_keyF9.JustPressed())
		return &m_keyF9;

	if(m_keyF10.JustPressed())
		return &m_keyF10;

	if(m_keyF11.JustPressed())
		return &m_keyF11;

	if(m_keyF12.JustPressed())
		return &m_keyF12;

	if(m_keyTilde.JustPressed())
		return &m_keyTilde;

	if(m_keyDash.JustPressed())
		return &m_keyDash;

	if(m_keyEquals.JustPressed())
		return &m_keyEquals;

	if(m_keyLBracket.JustPressed())
		return &m_keyLBracket;

	if(m_keyRBracket.JustPressed())
		return &m_keyRBracket;

	if(m_keyEnter.JustPressed())
		return &m_keyEnter;

	if(m_keyApostrophe.JustPressed())
		return &m_keyApostrophe;

	if(m_keyBackSlash.JustPressed())
		return &m_keyBackSlash;

	if(m_keySemiColon.JustPressed())
		return &m_keySemiColon;

	if(m_keyComma.JustPressed())
		return &m_keyComma;

	if(m_keyPeriod.JustPressed())
		return &m_keyPeriod;

	if(m_keySlash.JustPressed())
		return &m_keySlash;

	if(m_keySpace.JustPressed())
		return &m_keySpace;

	if(m_keyBackspace.JustPressed())
		return &m_keyBackspace;

	if(m_keyTab.JustPressed())
		return &m_keyTab;

	if(m_keyCaps.JustPressed())
		return &m_keyCaps;

	if(m_keyLShift.JustPressed())
		return &m_keyLShift;

	if(m_keyRShift.JustPressed())
		return &m_keyRShift;

	if(m_keyLControl.JustPressed())
		return &m_keyLControl;

	if(m_keyRControl.JustPressed())
		return &m_keyRControl;

	if(m_keyLALT.JustPressed())
		return &m_keyLALT;

	if(m_keyRALT.JustPressed())
		return &m_keyRALT;

	if(m_keyLWindows.JustPressed())
		return &m_keyLWindows;

	if(m_keyRWindows.JustPressed())
		return &m_keyRWindows;

	if(m_keyDelete.JustPressed())
		return &m_keyDelete;

	if(m_keyInsert.JustPressed())
		return &m_keyInsert;

	if(m_keyHome.JustPressed())
		return &m_keyHome;

	if(m_keyEnd.JustPressed())
		return &m_keyEnd;

	if(m_keyPageDown.JustPressed())
		return &m_keyPageDown;

	if(m_keyPageUp.JustPressed())
		return &m_keyPageUp;

	if(m_keyLeft.JustPressed())
		return &m_keyLeft;

	if(m_keyRight.JustPressed())
		return &m_keyRight;

	if(m_keyUp.JustPressed())
		return &m_keyUp;

	if(m_keyDown.JustPressed())
		return &m_keyDown;

	if(m_keyNumLock.JustPressed())
		return &m_keyNumLock;

	if(m_keyNumDivide.JustPressed())
		return &m_keyNumDivide;

	if(m_keyNumMultiply.JustPressed())
		return &m_keyNumMultiply;

	if(m_keyNumMinus.JustPressed())
		return &m_keyNumMinus;

	if(m_keyNumPlus.JustPressed())
		return &m_keyNumPlus;

	//if(m_keyNumEnter.JustPressed())
	//	m_keyNumEnter.SetHeld();

	if(m_keyNumDecimal.JustPressed())
		return &m_keyNumDecimal;

	if(m_keyNum0.JustPressed())
		return &m_keyNum0;

	if(m_keyNum1.JustPressed())
		return &m_keyNum1;

	if(m_keyNum2.JustPressed())
		return &m_keyNum2;

	if(m_keyNum3.JustPressed())
		return &m_keyNum3;

	if(m_keyNum4.JustPressed())
		return &m_keyNum4;

	if(m_keyNum5.JustPressed())
		return &m_keyNum5;

	if(m_keyNum6.JustPressed())
		return &m_keyNum6;

	if(m_keyNum7.JustPressed())
		return &m_keyNum7;

	if(m_keyNum8.JustPressed())
		return &m_keyNum8;

	if(m_keyNum9.JustPressed())
		return &m_keyNum9;

	if(m_buttonMouseLeft.JustPressed())
		return &m_buttonMouseLeft;

	if(m_buttonMouseRight.JustPressed())
		return &m_buttonMouseRight;

	if(m_buttonMouseMiddle.JustPressed())
		return &m_buttonMouseMiddle;

	if(m_buttonMouseExtra1.JustPressed())
		return &m_buttonMouseExtra1;

	if(m_buttonMouseExtra2.JustPressed())
		return &m_buttonMouseExtra2;

	if(m_buttonMouseExtra3.JustPressed())
		return &m_buttonMouseExtra3;

	if(m_buttonMouseExtra4.JustPressed())
		return &m_buttonMouseExtra4;

	if(m_buttonMouseExtra5.JustPressed())
		return &m_buttonMouseExtra5;

	return 0;
}

char xInput::GetCharacterPressed(DWORD dwTime, bool bAllowSpace)
{
	if(m_keyA.JustPressed(true) || m_keyA.ShouldRepeat(dwTime))
		return m_bCapitals ? 'A' : 'a';
	else if(m_keyB.JustPressed(true) || m_keyB.ShouldRepeat(dwTime))
		return m_bCapitals ? 'B' : 'b';
	else if(m_keyC.JustPressed(true) || m_keyC.ShouldRepeat(dwTime))
		return m_bCapitals ? 'C' : 'c';
	else if(m_keyD.JustPressed(true) || m_keyD.ShouldRepeat(dwTime))
		return m_bCapitals ? 'D' : 'd';
	else if(m_keyE.JustPressed(true) || m_keyE.ShouldRepeat(dwTime))
		return m_bCapitals ? 'E' : 'e';
	else if(m_keyF.JustPressed(true) || m_keyF.ShouldRepeat(dwTime))
		return m_bCapitals ? 'F' : 'f';
	else if(m_keyG.JustPressed(true) || m_keyG.ShouldRepeat(dwTime))
		return m_bCapitals ? 'G' : 'g';
	else if(m_keyH.JustPressed(true) || m_keyH.ShouldRepeat(dwTime))
		return m_bCapitals ? 'H' : 'h';
	else if(m_keyI.JustPressed(true) || m_keyI.ShouldRepeat(dwTime))
		return m_bCapitals ? 'I' : 'i';
	else if(m_keyJ.JustPressed(true) || m_keyJ.ShouldRepeat(dwTime))
		return m_bCapitals ? 'J' : 'j';
	else if(m_keyK.JustPressed(true) || m_keyK.ShouldRepeat(dwTime))
		return m_bCapitals ? 'K' : 'k';
	else if(m_keyL.JustPressed(true) || m_keyL.ShouldRepeat(dwTime))
		return m_bCapitals ? 'L' : 'l';
	else if(m_keyM.JustPressed(true) || m_keyM.ShouldRepeat(dwTime))
		return m_bCapitals ? 'M' : 'm';
	else if(m_keyN.JustPressed(true) || m_keyN.ShouldRepeat(dwTime))
		return m_bCapitals ? 'N' : 'n';
	else if(m_keyO.JustPressed(true) || m_keyO.ShouldRepeat(dwTime))
		return m_bCapitals ? 'O' : 'o';
	else if(m_keyP.JustPressed(true) || m_keyP.ShouldRepeat(dwTime))
		return m_bCapitals ? 'P' : 'p';
	else if(m_keyQ.JustPressed(true) || m_keyQ.ShouldRepeat(dwTime))
		return m_bCapitals ? 'Q' : 'q';
	else if(m_keyR.JustPressed(true) || m_keyR.ShouldRepeat(dwTime))
		return m_bCapitals ? 'R' : 'r';
	else if(m_keyS.JustPressed(true) || m_keyS.ShouldRepeat(dwTime))
		return m_bCapitals ? 'S' : 's';
	else if(m_keyT.JustPressed(true) || m_keyT.ShouldRepeat(dwTime))
		return m_bCapitals ? 'T' : 't';
	else if(m_keyU.JustPressed(true) || m_keyU.ShouldRepeat(dwTime))
		return m_bCapitals ? 'U' : 'u';
	else if(m_keyV.JustPressed(true) || m_keyV.ShouldRepeat(dwTime))
		return m_bCapitals ? 'V' : 'v';
	else if(m_keyW.JustPressed(true) || m_keyW.ShouldRepeat(dwTime))
		return m_bCapitals ? 'W' : 'w';
	else if(m_keyX.JustPressed(true) || m_keyX.ShouldRepeat(dwTime))
		return m_bCapitals ? 'X' : 'x';
	else if(m_keyY.JustPressed(true) || m_keyY.ShouldRepeat(dwTime))
		return m_bCapitals ? 'Y' : 'y';
	else if(m_keyZ.JustPressed(true) || m_keyZ.ShouldRepeat(dwTime))
		return m_bCapitals ? 'Z' : 'z';
	else if(m_key1.JustPressed(true) || m_key1.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '!' : '1';
	else if(m_key2.JustPressed(true) || m_key2.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '@' : '2';
	else if(m_key3.JustPressed(true) || m_key3.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '#' : '3';
	else if(m_key4.JustPressed(true) || m_key4.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '$' : '4';
	else if(m_key5.JustPressed(true) || m_key5.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '%' : '5';
	else if(m_key6.JustPressed(true) || m_key6.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '^' : '6';
	else if(m_key7.JustPressed(true) || m_key7.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '&' : '7';
	else if(m_key8.JustPressed(true) || m_key8.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '*' : '8';
	else if(m_key9.JustPressed(true) || m_key9.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '(' : '9';
	else if(m_key0.JustPressed(true) || m_key0.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? ')' : '0';
	else if(m_keyTilde.JustPressed(true) || m_keyTilde.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '~' : '`';
	else if(m_keyDash.JustPressed(true) || m_keyDash.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '_' : '-';
	else if(m_keyEquals.JustPressed(true) || m_keyEquals.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '=' : '+';
	else if(m_keyLBracket.JustPressed(true) || m_keyLBracket.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '{' : '[';
	else if(m_keyRBracket.JustPressed(true) || m_keyRBracket.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '}' : ']';
	else if(m_keySlash.JustPressed(true) || m_keySlash.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '?' : '/';
	else if(m_keySemiColon.JustPressed(true) || m_keySemiColon.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? ':' : ';';
	else if(m_keyApostrophe.JustPressed(true) || m_keyApostrophe.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '\'' : '"';
	else if(m_keyComma.JustPressed(true) || m_keyComma.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '<' : ',';
	else if(m_keyPeriod.JustPressed(true) || m_keyPeriod.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '>' : '.';
	else if(m_keyBackSlash.JustPressed(true) || m_keyBackSlash.ShouldRepeat(dwTime))
		return (m_keyLShift.IsDown() || m_keyRShift.IsDown()) ? '|' : '\\';
	else if(m_keyNumDivide.JustPressed(true) || m_keyNumDivide.ShouldRepeat(dwTime))
		return '/';
	else if(m_keyNumMultiply.JustPressed(true) || m_keyNumMultiply.ShouldRepeat(dwTime))
		return '*';
	else if(m_keyNumMinus.JustPressed(true) || m_keyNumMinus.ShouldRepeat(dwTime))
		return '-';
	else if(m_keyNumPlus.JustPressed(true) || m_keyNumPlus.ShouldRepeat(dwTime))
		return '+';
	else if(m_keyNumDecimal.JustPressed(true) || m_keyNumDecimal.ShouldRepeat(dwTime))
		return '.';
	else if(m_keyNum0.JustPressed(true) || m_keyNum0.ShouldRepeat(dwTime))
		return '0';
	else if(m_keyNum1.JustPressed(true) || m_keyNum1.ShouldRepeat(dwTime))
		return '1';
	else if(m_keyNum2.JustPressed(true) || m_keyNum2.ShouldRepeat(dwTime))
		return '2';
	else if(m_keyNum3.JustPressed(true) || m_keyNum3.ShouldRepeat(dwTime))
		return '3';
	else if(m_keyNum4.JustPressed(true) || m_keyNum4.ShouldRepeat(dwTime))
		return '4';
	else if(m_keyNum5.JustPressed(true) || m_keyNum5.ShouldRepeat(dwTime))
		return '5';
	else if(m_keyNum6.JustPressed(true) || m_keyNum6.ShouldRepeat(dwTime))
		return '6';
	else if(m_keyNum7.JustPressed(true) || m_keyNum7.ShouldRepeat(dwTime))
		return '7';
	else if(m_keyNum8.JustPressed(true) || m_keyNum8.ShouldRepeat(dwTime))
		return '8';
	else if(m_keyNum9.JustPressed(true) || m_keyNum9.ShouldRepeat(dwTime))
		return '9';
	else if(bAllowSpace && m_keySpace.JustPressed(true))
		return ' ';

	return '\0';
}

#endif