#include "Paper.h"

FORCE_INLINE void xAnimationReset(xAnimation *pThis)
{
	pThis->m_pCurKeyFrame			= 0;
	pThis->m_dCurKeyFrameTime		= 0.0;
	pThis->m_bDone					= 1;
	pThis->m_fCurXChange			= 0.0f;
	pThis->m_fCurYChange			= 0.0f;
	pThis->m_fCurWidthChange		= 0.0f;
	pThis->m_fCurHeightChange		= 0.0f;
	pThis->m_fCurAngleChange		= 0.0f;
	pThis->m_fCurScaleXChange		= 0.0f;
	pThis->m_fCurScaleYChange		= 0.0f;
	pThis->m_fCurAlphaChange		= 0.0f;
	pThis->m_pNext					= 0;
	pThis->m_pPrev					= 0;
	pThis->m_xKeyFrameList.m_pNext	= 0;
}

FORCE_INLINE void xAnimationUpdateRenderable(xAnimation *pThis)
{
	switch(pThis->m_iAnimType)
	{
		case PE_ANIMTYPE_QUAD:
			xRenderable2DQuadUpdate((xRenderable2DQuad*)pThis->m_pObject);
			break;
		case PE_ANIMTYPE_QUAD9:
			break;
		case PE_ANIMTYPE_UIOBJECT:
			pThis->pfnUpdateRenderables((xGUIObject*)pThis->m_pObject);
			break;
	}
}

void xAnimationInit(xAnimation *pThis, void* pObject, xRenderable2D *pRenderable, void (*pfnUpdateRenderablesCallback)(xGUIObject *pThis))
{
	xAnimationReset(pThis);

	pThis->m_pObject = pObject;
	pThis->m_pRenderable = pRenderable;
	pThis->m_iAnimType = PE_ANIMTYPE_UIOBJECT;
	pThis->pfnUpdateRenderables = pfnUpdateRenderablesCallback;
}

void xAnimationInitQuad(xAnimation *pThis, xSceneGraph *pSceneGraph, const char *strObjectFile, int iFrame, float fDepth)
{
	xAnimationReset(pThis);

	pThis->m_pObject = xSceneGraphCreateQuad(strObjectFile, iFrame);
	pThis->m_pRenderable = &((xRenderable2DQuad*)pThis->m_pObject)->m_xRenderable2D;
	pThis->m_iAnimType = PE_ANIMTYPE_QUAD;

	xSceneGraphInsertQuad(pSceneGraph, (xRenderable2DQuad*)pThis->m_pObject, 1, fDepth);

	xRenderable2DQuadUpdate((xRenderable2DQuad*)pThis->m_pObject);
}

void xAnimationLoadedAllKeyFrames(xAnimation *pThis)
{
	xRenderable2D *renderable;
	xKeyFrame *curframe, *nextframe;

	renderable = pThis->m_pRenderable;
	curframe = pThis->m_pCurKeyFrame;
	nextframe = pThis->m_pCurKeyFrame->m_pNext;

	renderable->m_fX		= curframe->m_fX;
	renderable->m_fY		= curframe->m_fY;
	renderable->m_fWidth	= curframe->m_fWidth;
	renderable->m_fHeight	= curframe->m_fHeight;
	renderable->m_fRotation	= curframe->m_fAngle;
	renderable->m_fScaleX	= curframe->m_fScaleX;
	renderable->m_fScaleY	= curframe->m_fScaleY;
	renderable->m_xColor.A	= curframe->m_fAlpha;

	pThis->m_iCurFrameChange		= nextframe->m_iFrame - curframe->m_iFrame;
	pThis->m_fCurXChange			= nextframe->m_fX - curframe->m_fX;
	pThis->m_fCurYChange			= nextframe->m_fY - curframe->m_fY;
	pThis->m_fCurWidthChange		= nextframe->m_fWidth - curframe->m_fWidth;
	pThis->m_fCurHeightChange		= nextframe->m_fHeight - curframe->m_fHeight;
	pThis->m_fCurAngleChange		= nextframe->m_fAngle - curframe->m_fAngle;
	pThis->m_fCurScaleXChange		= nextframe->m_fScaleX - curframe->m_fScaleX;
	pThis->m_fCurScaleYChange		= nextframe->m_fScaleY - curframe->m_fScaleY;
	pThis->m_fCurAlphaChange		= nextframe->m_fAlpha - curframe->m_fAlpha;

	pThis->m_dCurKeyFrameTime = 0;
	pThis->m_bDone = 0;

	xAnimationUpdateRenderable(pThis);
}

void xAnimationUpdate(xAnimation *pThis, double dTime)
{
	xRenderable2D *renderable;
	xKeyFrame *curframe, *nextframe;
	float curframechange;

	if(!pThis->m_pCurKeyFrame)
	{
		pThis->m_bDone = true;
		return;
	}

	if(!pThis->m_pCurKeyFrame->m_pNext)
	{
		if(pThis->m_bLoop)
			pThis->m_pCurKeyFrame = pThis->m_xKeyFrameList.m_pNext;
		else
		{
		pThis->m_bDone = true;
		return;
	}
	}

	renderable = pThis->m_pRenderable;
	curframe = pThis->m_pCurKeyFrame;
	nextframe = pThis->m_pCurKeyFrame->m_pNext;

	pThis->m_dCurKeyFrameTime += dTime;

	while(pThis->m_dCurKeyFrameTime * ANIMATION_FPS > pThis->m_iCurFrameChange)
	{
		curframe				= curframe->m_pNext;
		pThis->m_pCurKeyFrame	= curframe;

		renderable->m_fX		= curframe->m_fX;
		renderable->m_fY		= curframe->m_fY;
		renderable->m_fWidth	= curframe->m_fWidth;
		renderable->m_fHeight	= curframe->m_fHeight;
		renderable->m_fRotation	= curframe->m_fAngle;
		renderable->m_fScaleX	= curframe->m_fScaleX;
		renderable->m_fScaleY	= curframe->m_fScaleY;
		renderable->m_xColor.A	= curframe->m_fAlpha;

		if(!curframe->m_pNext)
		{
			xAnimationUpdateRenderable(pThis);

			if(pThis->m_bLoop)
			{
				curframe = pThis->m_xKeyFrameList.m_pNext;
				nextframe = curframe->m_pNext;
			}
			else
			{
			pThis->m_bDone = true;
			return;
		}
		}
		else
			nextframe = curframe->m_pNext;

		pThis->m_dCurKeyFrameTime	-= (float)pThis->m_iCurFrameChange / (float)ANIMATION_FPS;
		pThis->m_iCurFrameChange	= nextframe->m_iFrame - curframe->m_iFrame;
		pThis->m_fCurXChange		= nextframe->m_fX - curframe->m_fX;
		pThis->m_fCurYChange		= nextframe->m_fY - curframe->m_fY;
		pThis->m_fCurWidthChange	= nextframe->m_fWidth - curframe->m_fWidth;
		pThis->m_fCurHeightChange	= nextframe->m_fHeight - curframe->m_fHeight;
		pThis->m_fCurAngleChange	= nextframe->m_fAngle - curframe->m_fAngle;
		pThis->m_fCurScaleXChange	= nextframe->m_fScaleX - curframe->m_fScaleX;
		pThis->m_fCurScaleYChange	= nextframe->m_fScaleY - curframe->m_fScaleY;
		pThis->m_fCurAlphaChange	= nextframe->m_fAlpha - curframe->m_fAlpha;
	}

	curframechange = pThis->m_iCurFrameChange;

	renderable->m_fX		+= (float)(dTime * pThis->m_fCurXChange * ((float)ANIMATION_FPS / curframechange));
	renderable->m_fY		+= (float)(dTime * pThis->m_fCurYChange * ((float)ANIMATION_FPS / curframechange));
	renderable->m_fWidth	+= (float)(dTime * pThis->m_fCurWidthChange * ((float)ANIMATION_FPS / curframechange));
	renderable->m_fHeight	+= (float)(dTime * pThis->m_fCurHeightChange * ((float)ANIMATION_FPS / curframechange));
	renderable->m_fRotation	+= (float)(dTime * pThis->m_fCurAngleChange * ((float)ANIMATION_FPS / curframechange));
	renderable->m_fScaleX	+= (float)(dTime * pThis->m_fCurScaleXChange * ((float)ANIMATION_FPS / curframechange));
	renderable->m_fScaleY	+= (float)(dTime * pThis->m_fCurScaleYChange * ((float)ANIMATION_FPS / curframechange));
	renderable->m_xColor.A	+= (float)(dTime * pThis->m_fCurAlphaChange * ((float)ANIMATION_FPS / curframechange));

	if((pThis->m_fCurXChange > 0.0 && renderable->m_fX > nextframe->m_fX) ||
	   (pThis->m_fCurXChange < 0.0 && renderable->m_fX < nextframe->m_fX) || curframechange == 0.0)
		renderable->m_fX = nextframe->m_fX;

	if((pThis->m_fCurYChange > 0.0 && renderable->m_fY > nextframe->m_fY) || 
	   (pThis->m_fCurYChange < 0.0 && renderable->m_fY < nextframe->m_fY) || curframechange == 0.0)
		renderable->m_fY = nextframe->m_fY;

	if((pThis->m_fCurWidthChange > 0.0 && renderable->m_fWidth > nextframe->m_fWidth) ||
	   (pThis->m_fCurWidthChange < 0.0 && renderable->m_fWidth < nextframe->m_fWidth) || curframechange == 0.0)
		renderable->m_fWidth = nextframe->m_fWidth;

	if((pThis->m_fCurHeightChange > 0.0 && renderable->m_fHeight > nextframe->m_fHeight) || 
	   (pThis->m_fCurHeightChange < 0.0 && renderable->m_fHeight < nextframe->m_fHeight) || curframechange == 0.0)
		renderable->m_fHeight = nextframe->m_fHeight;

	if((pThis->m_fCurAngleChange > 0.0 && renderable->m_fRotation > nextframe->m_fAngle) ||
	   (pThis->m_fCurAngleChange < 0.0 && renderable->m_fRotation < nextframe->m_fAngle) || curframechange == 0.0)
		renderable->m_fRotation = nextframe->m_fAngle;

	if((pThis->m_fCurScaleXChange > 0.0 && renderable->m_fScaleX > nextframe->m_fScaleX) ||
	   (pThis->m_fCurScaleXChange < 0.0 && renderable->m_fScaleX < nextframe->m_fScaleX) || curframechange == 0.0)
		renderable->m_fScaleX = nextframe->m_fScaleX;

	if((pThis->m_fCurScaleYChange > 0.0 && renderable->m_fScaleY > nextframe->m_fScaleY) ||
	   (pThis->m_fCurScaleYChange < 0.0 && renderable->m_fScaleY < nextframe->m_fScaleY) || curframechange == 0.0)
		renderable->m_fScaleY = nextframe->m_fScaleY;

	xAnimationUpdateRenderable(pThis);
}

void xAnimationDelete(xAnimation *pThis, xSceneGraph *pSceneGraph, bool bDeleteTexture)
{
	xKeyFrame *curframe = pThis->m_xKeyFrameList.m_pNext;
	if(curframe)
	{
		while(curframe->m_pNext)
		{
			curframe = curframe->m_pNext;
			dlfree(curframe->m_pPrev);
		}

		dlfree(curframe);
	}

	if(pThis->m_pObject)
	{
		switch(pThis->m_iAnimType)
		{
			case PE_ANIMTYPE_QUAD:
				xSceneGraphDeleteQuad(pSceneGraph, (xRenderable2DQuad*)pThis->m_pObject, bDeleteTexture);
				pThis->m_pObject = 0;
				pThis->m_pRenderable = 0;
				break;
			case PE_ANIMTYPE_QUAD9:
				break;
		}
	}
}
