#include "peGame.h"

FORCE_INLINE void xLocalPlayerResetComboCount(xLocalPlayer *pThis);
FORCE_INLINE void xLocalPlayerUpdateCombo(xLocalPlayer *pThis);
FORCE_INLINE void xLocalPlayerUpdateBlocksCleared(xLocalPlayer *pThis);

bool xLocalPlayer::Init()
{
	xRenderable2DTextInit(&m_xComboText, BB_DEPTH_COMBO);
	xRenderable2DTextInit(&m_xBlocksClearedText, BB_DEPTH_BLOCKSCLEARED);
	return true;
}

void xLocalPlayer::Reset()
{
	xColor tempcolor, tempcolor2;
	float centerx, centery, angle, x, y;
	int i;

	m_bDead						= 0;
	m_bWonGame					= 0;
	m_bViewingPostPopup			= 0;
	m_bReadyToMoveOn			= 0;
	m_iComboCount			= 0;
	m_iNumBlocksCleared		= 0;
	m_iNumPiecesUsed		= 0;
	m_dwColorKeyPressedTime		= 0;
	m_dwLastRedBuildKeyPress	= 0;
	m_bAllowClearing			= 1;
	m_fComboTime				= BB_COMBO_TOTALTIME;
	m_bComboFade				= 0;
	m_fBlocksClearedHangTime	= BB_BLOCKSCLEARED_HANGTIME;
	m_fBlocksClearedTotalTime	= BB_BLOCKSCLEARED_GROWTIME + m_fBlocksClearedHangTime + BB_BLOCKSCLEARED_FADETIME;
	m_fBlocksClearedTime		= m_fBlocksClearedTotalTime;
	m_iBlocksCleared			= 0;

	m_xMultiplier.Init();

	m_xPiece.Init(BB_PIECESTARTX, BB_PIECESTARTY, m_xMultiplier.GetCurrentMultiplier(), 0, g_xGameOptions.m_bTutorial);
	m_xPiece.SetBeatManager(&m_xBeatManager);

	m_xField.Init();

	if(!m_pHitTimeObject)
	{
		xGraphicsGetTextures(BB_IMAGE_HITTIME, &m_pHitTimeTextures);

		m_pHitTimeObject = xSceneGraphCreateQuad(BB_IMAGE_HITTIME, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pHitTimeObject, 1, BB_DEPTH_HITTIME, 0, 1);
		xRenderable2DQuadSetPosition(m_pHitTimeObject, BB_HITTIME_LEFT, BB_HITTIME_TOP);
		xRenderable2DQuadUpdate(m_pHitTimeObject);
	}

	xRenderable2DQuadUpdateVisibility(m_pHitTimeObject, 0);

	if(!m_pCombo)
	{
		xRenderable2DTextSetFont(&m_xComboText, g_xGame.m_xGUI.GetFont(BB_FONT_COMBO));
		xRenderable2DTextSetPosition(&m_xComboText, BB_COMBO_TEXT_LEFT, BB_COMBO_TEXT_TOP);
		xRenderable2DTextSetRotation(&m_xComboText, BB_COMBO_TEXT_ROTATION);

		m_pCombo = xSceneGraphCreateQuad(BB_IMAGE_COMBO, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pCombo, 1, BB_DEPTH_COMBO, 0, 1);
		xRenderable2DQuadSetPosition(m_pCombo, BB_COMBO_LEFT, BB_COMBO_TOP);
		xRenderable2DQuadUpdate(m_pCombo);
	}

	xRenderable2DTextUpdateAlpha(&m_xComboText, 0.0f);
	xRenderable2DQuadUpdateAlpha(m_pCombo, 0.0f);

	if(!m_pBlocksCleared)
	{
		xRenderable2DTextSetFont(&m_xBlocksClearedText, g_xGame.m_xGUI.GetFont(BB_FONT_BLOCKSCLEARED));
		xRenderable2DTextSetPosition(&m_xBlocksClearedText, BB_BLOCKSCLEARED_TEXT_LEFT, BB_BLOCKSCLEARED_TEXT_TOP);
		xRenderable2DTextSetRotation(&m_xBlocksClearedText, BB_BLOCKSCLEARED_TEXT_ROTATION);

		m_pBlocksCleared = xSceneGraphCreateQuad(BB_IMAGE_BLOCKSCLEARED, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pBlocksCleared, 1, BB_DEPTH_BLOCKSCLEARED, 0, 1);
		xRenderable2DQuadSetPosition(m_pBlocksCleared, BB_BLOCKSCLEARED_LEFT, BB_BLOCKSCLEARED_TOP);
		xRenderable2DQuadUpdate(m_pBlocksCleared);

		m_pBlocksClearedDesc = xSceneGraphCreateQuad(BB_IMAGE_BLOCKSCLEAREDDESC, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pBlocksClearedDesc, 1, BB_DEPTH_BLOCKSCLEARED, 0, 1);
		xRenderable2DQuadSetPosition(m_pBlocksClearedDesc, BB_BLOCKSCLEARED_DESC_LEFT, BB_BLOCKSCLEARED_DESC_TOP);
		xGraphicsGetTextures(BB_IMAGE_BLOCKSCLEAREDDESC, &m_pBlocksClearedDescTextures);
		xRenderable2DQuadUpdate(m_pBlocksClearedDesc);
	}

	xRenderable2DTextUpdateVisibility(&m_xBlocksClearedText, 0);
	xRenderable2DQuadUpdateVisibility(m_pBlocksCleared, 0);
	xRenderable2DQuadUpdateVisibility(m_pBlocksClearedDesc, 0);

	if(!m_pHitTimeSounds[0])
	{
		m_pHitTimeSounds[0] = g_xPaperAPI.m_xAudio.LoadSound(BB_SOUND_HITTIME0);
		m_pHitTimeSounds[1] = g_xPaperAPI.m_xAudio.LoadSound(BB_SOUND_HITTIME1);
		m_pHitTimeSounds[2] = g_xPaperAPI.m_xAudio.LoadSound(BB_SOUND_HITTIME2);
		m_pHitTimeSounds[3] = g_xPaperAPI.m_xAudio.LoadSound(BB_SOUND_HITTIME3);
		m_pHitTimeSounds[4] = g_xPaperAPI.m_xAudio.LoadSound(BB_SOUND_HITTIME4);
		m_pHitTimeSounds[5] = g_xPaperAPI.m_xAudio.LoadSound(BB_SOUND_HITTIME5);
		m_pHitTimeSounds[6] = g_xPaperAPI.m_xAudio.LoadSound(BB_SOUND_HITTIME6);

		m_pSlideSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_SLIDE);
		m_pRotateSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_ROTATE);
		m_pDropSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_DROP);
		m_pTouchDownSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_TOUCHDOWN);
		m_pBlocksClearedSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_BLOCKCLEAR);
		m_pBlocksClearedWonSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_BLOCKCLEARWON);
		m_pPlayerMoveSound = g_xPaperAPI.m_xAudio.LoadSoundWithRandom(BB_SOUND_PLAYERDANCEMOVE);
	}

	if(!m_xBlockAddedEmitter.IsLoaded())
		m_xBlockAddedEmitter.Init(BB_EMITTER_BLOCKADDED, BB_DEPTH_BLOCKADDEDEMITTER, 0, &g_xGame.m_xSceneGraph);

	m_xBlockAddedEmitter.Kill();
	m_xBlockAddedEmitter.SetPosition(-10000.0, -10000.0);

#if WITH_BALANCESTATS
	g_xBalanceStats.NewPiece();
#endif
}

bool xLocalPlayer::PreUpdate(DWORD dwTime)
{
	int tempint, tempint2, tempint3;
	bool tempbool;

	if(m_bDead || m_bWonGame)
		return true;

	m_xPiece.PreUpdate(dwTime);

	if(m_xPiece.m_iUnfilled > 0 && m_dwColorKeyPressedTime != 0 && dwTime - m_dwColorKeyPressedTime > 150)
	{
		m_dwColorKeyPressedTime = 0;

		FillInBlock(false);
		HitWrongColor(m_dwColorKeyPressedTime);
	}

	if(!g_xGame.m_bTypingMessage)
	{
		if(g_xOptions.m_pButtons[KEY_RED]->JustPressed(true) && m_xPiece.m_iUnfilled > 0)
		{
			g_xOptions.m_pButtons[KEY_RED]->Eaten();

			if(g_xOptions.m_pButtons[KEY_RED] == g_xOptions.m_pButtons[KEY_DROPPIECE_ALT])
				m_dwLastRedBuildKeyPress = g_xOptions.m_pButtons[KEY_RED]->PressTime();

			if(!m_xPiece.CurBlockContainsRed())
			{
				m_dwColorKeyPressedTime = 0;

				FillInBlock(false);
				HitWrongColor(dwTime);
			}
			else if(m_xPiece.ColorComplete(true, g_xOptions.m_pButtons[KEY_BLUE]->IsDown(), g_xOptions.m_pButtons[KEY_YELLOW]->IsDown(), &tempbool))
			{
				m_dwColorKeyPressedTime = 0;
				m_bRotationKeyPressed = 0;

				g_xOptions.m_pButtons[KEY_BLUE]->Eaten();
				g_xOptions.m_pButtons[KEY_YELLOW]->Eaten();

				if(tempbool && m_xBeatManager.HitBeat(g_xOptions.m_pButtons[KEY_RED]->PressTime()))
				{
#if !EDITINGBEATMARKERS
					FillInBlock(true);
#endif
					SuccessfullyAddedBlock();
				}
				else
				{
#if !EDITINGBEATMARKERS
					FillInBlock(false);
#endif
					MissedBeat();
				}
			}
			else
				m_dwColorKeyPressedTime = g_xOptions.m_pButtons[KEY_RED]->PressTime();
		}
		
		if(g_xOptions.m_pButtons[KEY_BLUE]->JustPressed(true) && m_xPiece.m_iUnfilled > 0)
		{
			g_xOptions.m_pButtons[KEY_BLUE]->Eaten();

			if(g_xOptions.m_pButtons[KEY_BLUE] == g_xOptions.m_pButtons[KEY_ROTATEBOARDCLOCKWISE_ALT])
				m_dwLastBlueBuildKeyPress = g_xOptions.m_pButtons[KEY_BLUE]->PressTime();

			m_bRotationKeyPressed = 0;

			if(!m_xPiece.CurBlockContainsBlue())
			{
				m_dwColorKeyPressedTime = 0;

				FillInBlock(false);
				HitWrongColor(dwTime);
			}
			else if(m_xPiece.ColorComplete(g_xOptions.m_pButtons[KEY_RED]->IsDown(), true, g_xOptions.m_pButtons[KEY_YELLOW]->IsDown(), &tempbool))
			{
				g_xOptions.m_pButtons[KEY_YELLOW]->Eaten();

				m_dwColorKeyPressedTime = 0;

				if(tempbool && m_xBeatManager.HitBeat(g_xOptions.m_pButtons[KEY_BLUE]->PressTime()))
				{
#if !EDITINGBEATMARKERS
					FillInBlock(true);
#endif
					SuccessfullyAddedBlock();
				}
				else
				{
#if !EDITINGBEATMARKERS
					FillInBlock(false);
#endif
					MissedBeat();
				}
			}
			else
				m_dwColorKeyPressedTime = g_xOptions.m_pButtons[KEY_BLUE]->PressTime();
		}
		
		if(g_xOptions.m_pButtons[KEY_YELLOW]->JustPressed(true) && m_xPiece.m_iUnfilled > 0)
		{
			g_xOptions.m_pButtons[KEY_YELLOW]->Eaten();

			if(g_xOptions.m_pButtons[KEY_YELLOW] == g_xOptions.m_pButtons[KEY_ROTATEBOARDCOUNTERCLOCKWISE_ALT])
				m_dwLastYellowBuildKeyPress = g_xOptions.m_pButtons[KEY_YELLOW]->PressTime();

			m_bRotationKeyPressed = 0;

			if(!m_xPiece.CurBlockContainsYellow())
			{
				m_dwColorKeyPressedTime = 0;

				FillInBlock(false);
				HitWrongColor(dwTime);
			}
			else if(m_xPiece.ColorComplete(g_xOptions.m_pButtons[KEY_RED]->IsDown(), g_xOptions.m_pButtons[KEY_BLUE]->IsDown(), true, &tempbool))
			{
				m_dwColorKeyPressedTime = 0;

				if(tempbool && m_xBeatManager.HitBeat(g_xOptions.m_pButtons[KEY_YELLOW]->PressTime()))
				{
#if !EDITINGBEATMARKERS
					FillInBlock(true);
#endif
					SuccessfullyAddedBlock();
				}
				else
				{
#if !EDITINGBEATMARKERS
					FillInBlock(false);
#endif
					MissedBeat();
				}
			}
			else
				m_dwColorKeyPressedTime = g_xOptions.m_pButtons[KEY_YELLOW]->PressTime();
		}

		if(((m_xPiece.m_iUnfilled <= 0 && (g_xOptions.m_pButtons[KEY_ROTATEBOARDCLOCKWISE_ALT]->JustPressed(true) || (m_bRotationKeyPressed && g_xOptions.m_pButtons[KEY_ROTATEBOARDCLOCKWISE_ALT]->ShouldRepeat(dwTime)))) ||
			 g_xOptions.m_pButtons[KEY_ROTATEBOARDCLOCKWISE]->JustPressed(true) || (m_bRotationKeyPressed && g_xOptions.m_pButtons[KEY_ROTATEBOARDCLOCKWISE]->ShouldRepeat(dwTime))) &&
			(!g_xGameOptions.m_bTutorial || (!g_xGame.m_bPrePopup && g_xGame.m_bAllowFieldRotation)))
		{
			if(!g_xOptions.m_pButtons[KEY_ROTATEBOARDCLOCKWISE_ALT]->JustPressed(true) || dwTime - m_dwLastBlueBuildKeyPress > 200)
			{
				DEBUGSHADOWPIECE("RotateBoardClockwise - %u\n", dwTime);
				m_pSlideSound->Play(g_xOptions.m_fSoundVolume);
				m_xField.RotateFieldClockwise();
				g_xGame.m_iBoardRotateKeyPressed += 1;
				m_bRotationKeyPressed = 1;
			}
		}
		else if(((m_xPiece.m_iUnfilled <= 0 && (g_xOptions.m_pButtons[KEY_ROTATEBOARDCOUNTERCLOCKWISE_ALT]->JustPressed(true) || (m_bRotationKeyPressed && g_xOptions.m_pButtons[KEY_ROTATEBOARDCOUNTERCLOCKWISE_ALT]->ShouldRepeat(dwTime)))) ||
				  g_xOptions.m_pButtons[KEY_ROTATEBOARDCOUNTERCLOCKWISE]->JustPressed(true) || (m_bRotationKeyPressed && g_xOptions.m_pButtons[KEY_ROTATEBOARDCOUNTERCLOCKWISE]->ShouldRepeat(dwTime))) &&
				 (!g_xGameOptions.m_bTutorial || (!g_xGame.m_bPrePopup && g_xGame.m_bAllowFieldRotation)))
		{
			if(!g_xOptions.m_pButtons[KEY_ROTATEBOARDCOUNTERCLOCKWISE_ALT]->JustPressed(true) || dwTime - m_dwLastYellowBuildKeyPress > 200)
			{
				DEBUGSHADOWPIECE("RotateBoardCounterClockwise - %u\n", dwTime);
				m_pSlideSound->Play(g_xOptions.m_fSoundVolume);
				m_xField.RotateFieldCounterClockwise();
				g_xGame.m_iBoardRotateKeyPressed += 2;
				m_bRotationKeyPressed = 1;
			}
		}

		if((g_xOptions.m_pButtons[KEY_DROPPIECE]->JustPressed(true) || g_xOptions.m_pButtons[KEY_DROPPIECE_ALT]->JustPressed(true)) && m_xPiece.m_iUnfilled <= 0)
		{
			if(!g_xOptions.m_pButtons[KEY_DROPPIECE_ALT]->JustPressed(true) || dwTime - m_dwLastRedBuildKeyPress > 200)
			{
				g_xGame.m_bDropKeyPressed = 1;
				m_xField.DropPiece();
				m_pDropSound->Play(g_xOptions.m_fSoundVolume);
			}
		}

		if(m_bAllowClearing && g_xOptions.m_pButtons[KEY_CLEAR]->JustPressed())
		{
			tempint = m_xField.ClearBlocks(&tempint2, &tempint3);

			m_iNumBlocksCleared += tempint;

			if(!g_xGameOptions.m_bTutorial)
				BlocksCleared(tempint, tempint2, tempint3);

			if(g_xGame.m_xEnemyMoveState == BB_ENEMYMOVE_UNDECIDED)
			{
				g_xGame.m_dwPlayerVoiceTime = dwTime + 2000;
				DEBUGVOICE("ENEMY VOICE: %u LocalPlayer setting Voice Time after Clear %u\n", dwTime, g_xGame.m_dwPlayerVoiceTime);
			}
		}

		if(g_xGame.m_dwPlayerVoiceTime != 0)
		{
			if(g_xGame.m_dwPlayerVoiceTime < 2000000000)
			{
				if(dwTime > g_xGame.m_dwPlayerVoiceTime)
				{
					if(g_xGame.m_pEnemyReactionSound)
						g_xGame.PlayVoiceSound(g_xGame.m_pEnemyReactionSound);

					g_xGame.m_dwPlayerVoiceTime = 2000000000 + dwTime + 2000;

					DEBUGVOICE("ENEMY VOICE: %u Played Player Move Voice and set Voice Time to %u\n", dwTime, g_xGame.m_dwPlayerVoiceTime);
				}
			}
			else if(2000000000 + dwTime > g_xGame.m_dwPlayerVoiceTime)
			{
				DEBUGVOICE("ENEMY VOICE: %u Setting Player Voice Time to 0 from %u\n", dwTime, g_xGame.m_dwPlayerVoiceTime);
				g_xGame.m_dwPlayerVoiceTime = 0;
			}
		}

		if(!m_xPiece.m_bDropping && (g_xOptions.m_pButtons[KEY_ROTATEPIECE]->JustPressed() || g_xOptions.m_pButtons[KEY_ROTATEPIECE_ALT]->JustPressed()))
		{
			DEBUGSHADOWPIECE("Rotate Piece %u\n", dwTime);
			g_xGame.m_bPieceRotateKeyPressed = 1;
			m_xField.ForceFinishRotation();
			m_xPiece.Rotate();
			m_xField.UpdateShadow();
			m_pRotateSound->Play(g_xOptions.m_fSoundVolume);
		}

		if(!g_xGame.m_bPrePopup && g_xOptions.m_pButtons[KEY_PAUSE]->JustPressed() && g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER)
		{
			if(!g_xGame.m_bVersusScreenAnimating)
			{
				g_xGame.Pause(g_xPaperAPI.m_bHasFocus);
				g_xGame.m_xGUI.AddWindow(BB_WINDOW_GAMEPAUSED);
			}
		}

		if(g_xPaperAPI.m_iNetMode == PE_NETMODE_SINGLEPLAYER)
		{
			if(g_xOptions.m_pButtons[KEY_GOTOMENU]->JustPressed(true))
			{
				g_xOptions.m_pButtons[KEY_GOTOMENU]->Eaten();

				if(!g_xGame.m_xGUI.GetWindow(BB_WINDOW_GAMEPAUSED))
				{
					if(!g_xGame.m_bVersusScreenAnimating)
					{
						g_xGame.Pause(g_xPaperAPI.m_bHasFocus);
						g_xGame.m_xGUI.AddWindow(BB_WINDOW_GAMEPAUSED);
					}
				}
				else
				{
					g_xGame.m_xGUI.DelayedRemoveWindow(BB_WINDOW_GAMEPAUSED);
					g_xGame.UnPause();
				}
			}

			m_bShowEscapeMessage = false;
		}
		else
		{
			if(g_xOptions.m_pButtons[KEY_GOTOMENU]->JustPressed(true) && g_xPaperAPI.m_iNetMode == PE_NETMODE_CLIENT && g_xGame.m_iConnectingStatus < 6)
			{
				g_xOptions.m_pButtons[KEY_GOTOMENU]->Eaten();
				g_xPaperAPI.m_xNetwork.Close();
			}
			else if(g_xOptions.m_pButtons[KEY_GOTOMENU]->IsHeld() && g_xPaperAPI.m_dwCurrentTime - g_xOptions.m_pButtons[KEY_GOTOMENU]->PressTime() > 2000)
			{
				g_xPaperAPI.m_xNetwork.Close();
				g_xPaperAPI.m_xPingNetwork.Close();
				g_xGame.m_xSong.Stop();

				g_xMenu.PlayMusic();
				g_xMenu.RemoveAllMenuWindows();
				g_xMenu.AddMainMenu();
				g_pCurGameState = &g_xMenu;
				xGraphicsSetSceneGraph(&g_xMenu.m_xSceneGraph);
			}
			else if(!g_xGame.GameOver() && g_xOptions.m_pButtons[KEY_GOTOMENU]->IsDown())
				m_bShowEscapeMessage = true;
			else
				m_bShowEscapeMessage = false;
		}

		if(m_xPiece.m_bSkippedBeat && m_xPiece.m_iUnfilled > 0)
		{
			if(g_xGameOptions.m_bCombosRequired && m_iComboCount > 0 && m_xPiece.m_iNumBlocks > 0)
			{
				//DEBUGLOG("ResetComboCount - Combos Required\n");
				xLocalPlayerResetComboCount(this);

				m_xPiece.m_bMissedCombo = true;

				MissedCombo();
			}
		}
	}
	
	return true;
}

bool xLocalPlayer::Update(double dTime)
{
	xColor color;
	float hittimeduration, percent, width, height;
	int i;

	if(!g_xGameOptions.m_bShowComboCounter)
	{
		hittimeduration = (g_xPaperAPI.m_dwCurrentTime - g_xGame.m_dwGameStartTime) - m_xBeatManager.m_dwLastHitTime;
		if(hittimeduration < BB_HITTIME_SCALEIN)
		{
			hittimeduration /= BB_HITTIME_SCALEIN;

			if(m_xBeatManager.m_iHitTimeIndex == 6)
				xRenderable2DQuadUpdateColor(m_pHitTimeObject, xColorInit(&color, 1.0f, 0.0f, 0.0f, 0.8f * hittimeduration));
			else
				xRenderable2DQuadUpdateColor(m_pHitTimeObject, xColorInit(&color, 1.0f - ((m_xBeatManager.m_fHitTimeScale - 1.0f) / 0.5f), 1.0f, 0.0f, 0.8f * hittimeduration));

			xRenderable2DQuadUpdateVisibility(m_pHitTimeObject, 1);
			xRenderable2DQuadUpdateTexture(m_pHitTimeObject, m_pHitTimeTextures[m_xBeatManager.m_iHitTimeIndex]);
			xRenderable2DQuadSetPosition(m_pHitTimeObject, BB_HITTIME_LEFT - BB_HITTIME_WIDTH * (m_xBeatManager.m_fHitTimeScale * 0.8f * hittimeduration - 1.0f) / 2.0f, BB_HITTIME_TOP - BB_HITTIME_HEIGHT * (m_xBeatManager.m_fHitTimeScale * 0.8f * hittimeduration - 1.0f) / 2.0f);
			xRenderable2DQuadSetSize(m_pHitTimeObject, BB_HITTIME_WIDTH * m_xBeatManager.m_fHitTimeScale * 0.8f * hittimeduration, BB_HITTIME_HEIGHT * m_xBeatManager.m_fHitTimeScale * 0.8f * hittimeduration);
			xRenderable2DQuadUpdate(m_pHitTimeObject);
		}
		else if(hittimeduration < BB_HITTIME_GROW)
		{
			if(m_xBeatManager.m_iHitTimeIndex == 6)
				xRenderable2DQuadUpdateColor(m_pHitTimeObject, xColorInit(&color, 1.0f, 0.0f, 0.0f, 0.8f));
			else
				xRenderable2DQuadUpdateColor(m_pHitTimeObject, xColorInit(&color, 1.0f - ((m_xBeatManager.m_fHitTimeScale - 1.0f) / 0.5f), 1.0f, 0.0f, 0.8f));

			hittimeduration = (0.8f + ((hittimeduration - BB_HITTIME_SCALEIN) / (BB_HITTIME_GROW - BB_HITTIME_SCALEIN)) * 0.2f) * m_xBeatManager.m_fHitTimeScale;

			xRenderable2DQuadUpdateVisibility(m_pHitTimeObject, 1);
			xRenderable2DQuadSetPosition(m_pHitTimeObject, BB_HITTIME_LEFT - BB_HITTIME_WIDTH * (hittimeduration - 1.0f) / 2.0f, BB_HITTIME_TOP - BB_HITTIME_HEIGHT * (hittimeduration - 1.0f) / 2.0f);
			xRenderable2DQuadSetSize(m_pHitTimeObject, BB_HITTIME_WIDTH * hittimeduration, BB_HITTIME_HEIGHT * hittimeduration);
			xRenderable2DQuadUpdate(m_pHitTimeObject);
		}
		else if(hittimeduration < BB_HITTIME_SCALEOUT)
		{
			hittimeduration = (1.0f + ((hittimeduration - BB_HITTIME_GROW) / (BB_HITTIME_SCALEOUT - BB_HITTIME_GROW))) * m_xBeatManager.m_fHitTimeScale;
			xRenderable2DQuadUpdateAlpha(m_pHitTimeObject, (1.0f - (hittimeduration - m_xBeatManager.m_fHitTimeScale) / m_xBeatManager.m_fHitTimeScale) * 0.8f);
			xRenderable2DQuadSetPosition(m_pHitTimeObject, BB_HITTIME_LEFT - BB_HITTIME_WIDTH * (hittimeduration - 1.0f) / 2.0f, BB_HITTIME_TOP - BB_HITTIME_HEIGHT * (hittimeduration - 1.0f) / 2.0f);
			xRenderable2DQuadSetSize(m_pHitTimeObject, BB_HITTIME_WIDTH * hittimeduration, BB_HITTIME_HEIGHT * hittimeduration);
			xRenderable2DQuadUpdate(m_pHitTimeObject);
		}
		else if(m_pHitTimeObject->m_xRenderable2D.m_bVisible)
			xRenderable2DQuadUpdateVisibility(m_pHitTimeObject, 0);
	}

	//DEBUGLOG("Time: %u %u\n", g_xGame.m_xSong.GetTime(), m_xPiece.m_dwSongLength);

	if(!g_bCreditsRolling)
	{
		if(!g_xGameOptions.m_bLoopSong && g_xGame.m_strTime[0] == 32 && g_xGame.m_strTime[2] == 32 && g_xGame.m_strTime[3] == 32) // if time is 0:00
		{
			if(!g_xGame.GameOver() && !m_bWonGame)
			{
				if(g_xGame.m_fCrowdBarPercentage > 0.5f)
					g_xGame.Failed();
				else
					WonGame();
			}
		}
		else if(m_xPiece.m_bTouchedDown)
		{
			if(!m_xPiece.m_bPlayedTouchDown)
			{
				if(!g_xGame.m_bStopSounds)
					m_pTouchDownSound->Play(g_xOptions.m_fSoundVolume);

				m_xPiece.m_bPlayedTouchDown = 1;
			}

			if(!g_xGameOptions.m_bTutorial)
			{
				if(g_xGameOptions.m_iMaxPieces <= 0 || ++m_iNumPiecesUsed < g_xGameOptions.m_iMaxPieces)
				{
					// Init the New Next Piece
					m_xPiece.Init(BB_PIECESTARTX, BB_PIECESTARTY, m_xMultiplier.GetCurrentMultiplier(), false, g_xGameOptions.m_bTutorial);

					// Let the field know we're onto a new piece
					m_xField.NewPiece();
				}
				else if(!g_xGame.GameOver())
					g_xGame.Failed();
			}
		}
	}

	if(m_dPlayerMoveApplauseDelay < BB_PLAYERMOVE_APPLAUSEDELAY)
	{
		m_dPlayerMoveApplauseDelay += dTime;

		if(!g_xGame.m_bStopSounds && m_dPlayerMoveApplauseDelay >= BB_PLAYERMOVE_APPLAUSEDELAY)
			g_xGame.m_pCrowdPositiveSound->Play(g_xOptions.m_fSoundVolume);
	}

	if(m_dEnemyMoveApplauseDelay < BB_PLAYERMOVE_APPLAUSEDELAY)
	{
		m_dEnemyMoveApplauseDelay += dTime;

		if(!g_xGame.m_bStopSounds && m_dEnemyMoveApplauseDelay >= BB_PLAYERMOVE_APPLAUSEDELAY)
			g_xGame.m_pCrowdPositiveSound->Play(g_xOptions.m_fSoundVolume);
	}

	if(!m_xBlockAddedEmitter.IsDone())
	{
		m_xBlockAddedEmitter.Update(dTime);

		if(m_xBlockAddedEmitter.m_dLifeTime > BB_BLOCKADDED_TIME)
			m_xBlockAddedEmitter.Kill();
	}

	m_xMultiplier.Update(g_xGame.m_bPaused ? 0.0 : dTime);

	if(g_xGame.GameOver() && !m_bViewingPostPopup)
	{
		m_bViewingPostPopup = true;
		m_bDead = true;
		g_xGame.Pause(g_xPaperAPI.m_bHasFocus);

		if(g_xPaperAPI.m_iNetMode == PE_NETMODE_CLIENT)
			g_xGame.AddClientPostPopupWindow();
		else if(g_xPaperAPI.m_iNetMode == PE_NETMODE_LISTENSERVER)
			g_xGame.AddServerPostPopupWindow();
		else
			g_xGame.AddFailedPostPopupWindow();

#if WITH_BALANCESTATS
		g_xBalanceStats.SetScore(((0.5f - g_xGame.m_fCrowdBarPercentage) / 0.5f) * 100.0f, g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][g_xGameOptions.m_iLevelIndex]);
		g_xBalanceStats.SetWonGame(false, g_xGame.m_xSong.GetLength() - g_xGame.m_xSong.GetTime());

		g_xBalanceStats.Commit();
#endif
	}

	if(m_bComboFade)
	{
		if(m_fComboTime < BB_COMBO_TOTALTIME)
		{
			m_fComboTime += dTime;

			if(m_fComboTime > BB_COMBO_TOTALTIME)
				m_fComboTime = BB_COMBO_TOTALTIME;

			xLocalPlayerUpdateCombo(this);
		}
	}
	else if(m_fComboTime < BB_COMBO_GROWTIME + BB_COMBO_HANGTIME)
	{
		m_fComboTime += dTime;

		if(m_fComboTime > BB_COMBO_GROWTIME + BB_COMBO_HANGTIME)
			m_fComboTime = BB_COMBO_GROWTIME + BB_COMBO_HANGTIME;

		xLocalPlayerUpdateCombo(this);
	}

	if(m_fBlocksClearedTime < m_fBlocksClearedTotalTime)
	{
		m_fBlocksClearedTime += dTime;

		if(m_fBlocksClearedTime > m_fBlocksClearedTotalTime)
			m_fBlocksClearedTime = m_fBlocksClearedTotalTime;

		xLocalPlayerUpdateBlocksCleared(this);
	}

	//if(m_bShowEscapeMessage)
	//	m_pEscapeMessage->Render();

	return true;
}

void xLocalPlayer::RefreshAllRenderables()
{
	int i;

	if(m_pHitTimeObject)
	{
		xRenderable2DQuadUpdateTexture(m_pHitTimeObject, m_pHitTimeObject->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pHitTimeObject);
		xGraphicsGetTextures(BB_IMAGE_HITTIME, &m_pHitTimeTextures);
	}

	if(m_pCombo)
	{
		xRenderable2DTextRefreshTextures(&m_xComboText);
		xRenderable2DTextSetText(&m_xComboText, &g_xGame.m_xSceneGraph, m_strCombo);

		xRenderable2DQuadUpdateTexture(m_pCombo, m_pCombo->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pCombo);
	}

	if(m_pBlocksCleared)
	{
		xRenderable2DTextRefreshTextures(&m_xBlocksClearedText);
		xRenderable2DTextSetText(&m_xBlocksClearedText, &g_xGame.m_xSceneGraph, m_strBlocksCleared);

		xRenderable2DQuadUpdateTexture(m_pBlocksCleared, m_pBlocksCleared->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pBlocksCleared);

		xRenderable2DQuadUpdateTexture(m_pBlocksClearedDesc, m_pBlocksClearedDesc->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pBlocksClearedDesc);
		xGraphicsGetTextures(BB_IMAGE_BLOCKSCLEAREDDESC, &m_pBlocksClearedDescTextures);
	}

	if(m_xBlockAddedEmitter.m_pTextures)
		m_xBlockAddedEmitter.RefreshRenderables(BB_EMITTER_BLOCKADDED);

	m_xField.RefreshAllRenderables();
	m_xPiece.RefreshAllRenderables();
	m_xMultiplier.RefreshAllRenderables();
}

void xLocalPlayer::Shutdown()
{
	g_xGame.m_xGUI.RemoveAllWindows();

	m_xPiece.Delete(true, true, true);

	m_xField.Shutdown();
}

void xLocalPlayer::SuccessfullyAddedBlock()
{
	char temptext[16];
	float width;
	int i;

	m_iComboCount++;
	
	//DEBUGLOG("Added - %i\n", m_xBeatManager.m_iHitTimeIndex);
	m_pHitTimeSounds[m_xBeatManager.m_iHitTimeIndex]->Play(g_xOptions.m_fSoundVolume);

	m_xMultiplier.AddedBlock(m_iComboCount);

	if(m_iComboCount >= 10)
	{
		g_xGame.UnlockAchievement(BB_ACH_COMBO_10);

		if(m_iComboCount >= 100)
		{
			g_xGame.UnlockAchievement(BB_ACH_COMBO_100);

			if(m_iComboCount >= 500)
			{
				g_xGame.UnlockAchievement(BB_ACH_COMBO_500);

				if(m_iComboCount >= 1000)
					g_xGame.UnlockAchievement(BB_ACH_COMBO_1000);
			}
		}
	}

	if(g_xGameOptions.m_bShowComboCounter && m_iComboCount >= 2)
	{
		itoa(m_iComboCount, temptext, 10);

		for(i = 0; temptext[i] != '\0'; i++)
			m_strCombo[i] = 32 + temptext[i] - '0';

		m_strCombo[i] = '\0';

		m_fComboTextSize = xFontGetWidth(m_xComboText.m_pFont, m_strCombo);

		xRenderable2DTextUpdateVisibility(&m_xComboText, 1);
		xRenderable2DQuadUpdateVisibility(m_pCombo, 1);

		if(m_xPiece.m_iUnfilled <= 0)
		{
			m_bComboFade = 1;

			if(m_pCombo->m_xRenderable2D.m_xColor.A >= 1.0f)
				m_fComboTime = BB_COMBO_GROWTIME + BB_COMBO_HANGTIME;
			else
				m_fComboTime = 0.0f;
		}
		else if(m_pCombo->m_xRenderable2D.m_xColor.A < 0.1f)
		{
			m_bComboFade = 0;
			m_fComboTime = 0.0f;
		}

		xLocalPlayerUpdateCombo(this);
	}

	if(g_xPaperAPI.m_iNetMode != PE_NETMODE_SINGLEPLAYER)
		xGameNetwork::SendAddedBlock(m_iComboCount, g_xGame.GetCrowdBarPosition());

	if(!m_xPiece.m_iUnfilled && g_xPaperAPI.m_iNetMode != PE_NETMODE_SINGLEPLAYER)
		xGameNetwork::SendFinishedPiece(m_xPiece.m_iNumBlocks, g_xGame.GetCrowdBarPosition());

#if !EDITINGBEATMARKERS
	m_xBlockAddedEmitter.Reset();
	m_xBlockAddedEmitter.SetPosition(m_xPiece.m_pPrevBlock->m_pBlockObject->m_xRenderable2D.m_fX + BB_BLOCKOFFSET / 2.0, m_xPiece.m_pPrevBlock->m_pBlockObject->m_xRenderable2D.m_fY + BB_BLOCKOFFSET / 2.0);
	m_xBlockAddedEmitter.SetColorIndex(m_xPiece.m_iColorIndex);
#endif

#if WITH_BALANCESTATS
	g_xBalanceStats.AddedBlock(m_iComboCount);

	if(!m_xPiece.m_iUnfilled)
		g_xBalanceStats.NewPiece();
#endif
}

FORCE_INLINE void xLocalPlayerUpdateCombo(xLocalPlayer *pThis)
{
	float scale, alpha, temptime;

	if(pThis->m_bComboFade && pThis->m_fComboTime >= BB_COMBO_GROWTIME + BB_COMBO_HANGTIME)
	{
		if( pThis->m_fComboTime < BB_COMBO_TOTALTIME)
		{
			alpha = (BB_COMBO_TOTALTIME - pThis->m_fComboTime) / BB_COMBO_FADETIME;
			scale = 1.0f + (1.0f - alpha) * 0.4f;
		}
		else
		{
			alpha = 0.0f;
			scale = 1.4f;
		}
	}
	else if(pThis->m_fComboTime < BB_COMBO_GROWTIME)
	{
		alpha = pThis->m_fComboTime / BB_COMBO_GROWTIME;
		scale = 0.5f + (alpha) * 0.5f;
	}
	else
	{
		alpha = 1.0f;
		scale = 1.0f;
	}

	xRenderable2DQuadSetPosition(pThis->m_pCombo, BB_COMBO_LEFT + 0.5f * (BB_COMBO_SIZEX * (1.0f - scale)), BB_COMBO_TOP + 0.5f * (BB_COMBO_SIZEY * (1.0f - scale)));
	xRenderable2DQuadSetSize(pThis->m_pCombo, BB_COMBO_SIZEX * scale, BB_COMBO_SIZEY * scale);
	xRenderable2DQuadUpdateAlpha(pThis->m_pCombo, alpha);
	xRenderable2DQuadUpdate(pThis->m_pCombo);

	xRenderable2DTextSetPosition(&pThis->m_xComboText, pThis->m_pCombo->m_xRenderable2D.m_fX - (pThis->m_fComboTextSize + 44.0f) * scale, BB_COMBO_TEXT_TOP + 0.5f * (xFontGetSize(pThis->m_xComboText.m_pFont) * (1.0f - scale)));
	xRenderable2DTextSetScale(&pThis->m_xComboText, scale, scale);
	xRenderable2DTextUpdateAlpha(&pThis->m_xComboText, alpha);
	xRenderable2DTextSetText(&pThis->m_xComboText, &g_xGame.m_xSceneGraph, pThis->m_strCombo);
}

void xLocalPlayer::BlocksCleared(int iBlockCount, int iWhiteBlockCount, int iRingCount)
{
	char temptext[4];
	float crowdadjust;
	int i;

	if(iBlockCount > 0)
	{
		//AddToScore(g_xGameOptions.m_fBPM * iBlockCount * BB_SCORE_BLOCKSCLEAREDMULTIPLIER);
		//DEBUGLOG("Blocks Cleared Adding To Score - BlockCount=%i\n", iBlockCount);

		if(!g_xGameOptions.m_bBoss || (g_xGameOptions.m_iBossIndex == 2 || g_xGameOptions.m_iBossIndex == 4) ||
			(g_xGameOptions.m_iBossIndex == 1 && iBlockCount >= g_iPlayerChargeClearableCount[2]) ||
			(g_xGameOptions.m_iBossIndex == 3 && iBlockCount >= g_iPlayerChargeClearableCount[3]) ||
			(g_xGameOptions.m_iBossIndex == 5 && iBlockCount >= g_iPlayerChargeClearableCount[4]))
		{
			crowdadjust = g_xGame.GetCrowdBarBlocksClearedAdjustment(iBlockCount);

			if(g_xGame.m_fCrowdBarPercentage + crowdadjust <= 0.0)
			{
				g_xPaperAPI.m_xAudio.StopAllSounds();
				g_xGame.m_bStopSounds = 1;
				g_xGame.m_xCrowdState = BB_CROWD_NONE;
				m_pBlocksClearedWonSound->Play(g_xOptions.m_fSoundVolume);
			}
			else
				m_pBlocksClearedSound->Play(g_xOptions.m_fSoundVolume);

			g_xGame.AdjustCrowdBar(crowdadjust);

			if(!g_xGameOptions.m_bTutorial)
			{
				m_iBlocksCleared += iBlockCount;

				if(m_iBlocksCleared >= 200)
				{
					g_xGame.UnlockAchievement(BB_ACH_BLOCKS_200);

					if(m_iBlocksCleared >= 250)
					{
						g_xGame.UnlockAchievement(BB_ACH_BLOCKS_250);

						if(m_iBlocksCleared >= 300)
							g_xGame.UnlockAchievement(BB_ACH_BLOCKS_300);
					}
				}

				if(iBlockCount < 20)
				{
					g_xGame.m_pPlayerModel->SetAnimation(BB_PLAYERANIMATION_MOVE1, g_xGameOptions.m_fBPM);
					xRenderable2DQuadUpdateTexture(m_pBlocksClearedDesc, m_pBlocksClearedDescTextures[0]);
				}
				else if(iBlockCount < 40)
				{
					g_xGame.m_pPlayerModel->SetAnimation(BB_PLAYERANIMATION_MOVE2, g_xGameOptions.m_fBPM);
					xRenderable2DQuadUpdateTexture(m_pBlocksClearedDesc, m_pBlocksClearedDescTextures[1]);
					g_xGame.UnlockAchievement(BB_ACH_LEVEL_2_MOVE);
				}
				else if(iBlockCount < 60)
				{
					g_xGame.m_pPlayerModel->SetAnimation(BB_PLAYERANIMATION_MOVE3, g_xGameOptions.m_fBPM);
					xRenderable2DQuadUpdateTexture(m_pBlocksClearedDesc, m_pBlocksClearedDescTextures[2]);
					g_xGame.UnlockAchievement(BB_ACH_LEVEL_3_MOVE);
				}
				else
				{
					g_xGame.m_pPlayerModel->SetAnimation(BB_PLAYERANIMATION_MOVE4, g_xGameOptions.m_fBPM);
					xRenderable2DQuadUpdateTexture(m_pBlocksClearedDesc, m_pBlocksClearedDescTextures[3]);
					g_xGame.UnlockAchievement(BB_ACH_LEVEL_4_MOVE);
				}

				if(iRingCount >= 1)
				{
					g_xGame.UnlockAchievement(BB_ACH_RINGS_1);

					if(iRingCount >= 3)
					{
						g_xGame.UnlockAchievement(BB_ACH_RINGS_3);

						if(iRingCount >= 6)
							g_xGame.UnlockAchievement(BB_ACH_RINGS_6);
					}
				}

				if(!g_xGame.m_bStopSounds)
				{
					m_pPlayerMoveSound->Play(g_xOptions.m_fSoundVolume);
					m_dPlayerMoveApplauseDelay = 0.0;
				}

				if(g_xGameOptions.m_bShowClearCounter)
				{
					itoa(m_iBlocksCleared, temptext, 10);
					m_fBlocksClearedHangTime = BB_BLOCKSCLEARED_HANGTIME_CHAL;
					xRenderable2DQuadUpdateVisibility(m_pBlocksClearedDesc, 0);
					//m_bBlocksClearedShowDesc = 0;
				}
				else
				{
					itoa(iBlockCount, temptext, 10);
					m_fBlocksClearedHangTime = BB_BLOCKSCLEARED_HANGTIME;
					xRenderable2DQuadUpdateVisibility(m_pBlocksClearedDesc, 1);
					//m_bBlocksClearedShowDesc = 1;
				}

				m_fBlocksClearedTotalTime = BB_BLOCKSCLEARED_GROWTIME + m_fBlocksClearedHangTime + BB_BLOCKSCLEARED_FADETIME;

				m_strBlocksCleared[0] = 32 + temptext[0] - '0';
				m_strBlocksCleared[1] = 32 + temptext[1] - '0';
				m_strBlocksCleared[2] = '\0';

				m_fBlocksClearedTextSize = xFontGetWidth(m_xBlocksClearedText.m_pFont, m_strBlocksCleared);
				m_fBlocksClearedTime = 0.0;

				xRenderable2DTextUpdateVisibility(&m_xBlocksClearedText, 1);
				xRenderable2DQuadUpdateVisibility(m_pBlocksCleared, 1);

				xLocalPlayerUpdateBlocksCleared(this);
			}

			m_xMultiplier.MultiplierAddTime(pow((float)iBlockCount, 1.2f) / 7.92); // 7.92 = (10^1.2) / 2.0, which gives us 2 seconds of white block time for clearing 10 blocks and 10.5 seconds for clearing 40 blocks

			if(g_xPaperAPI.m_iNetMode != PE_NETMODE_SINGLEPLAYER)
				xGameNetwork::SendBlocksCleared(iBlockCount, g_xGame.GetCrowdBarPosition());
		}
		else
			m_pBlocksClearedSound->Play(g_xOptions.m_fSoundVolume);

		g_xGame.m_dwPlayerMoveFlashTime = g_xPaperAPI.m_dwCurrentTime - g_xGame.m_dwGameStartTime;
		DEBUGLOG("Player Move Flash Time Set To %u\n", g_xGame.m_dwPlayerMoveFlashTime);

#if WITH_BALANCESTATS
		g_xBalanceStats.BlocksCleared(iBlockCount);
#endif
	}
}

FORCE_INLINE void xLocalPlayerUpdateBlocksCleared(xLocalPlayer *pThis)
{
	float scale, alpha, temptime;

	if(pThis->m_fBlocksClearedTime < BB_BLOCKSCLEARED_GROWTIME)
	{
		alpha = pThis->m_fBlocksClearedTime / BB_BLOCKSCLEARED_GROWTIME;
		scale = 0.5f + (alpha) * 0.5f;
	}
	else if(pThis->m_fBlocksClearedTime < BB_BLOCKSCLEARED_GROWTIME + pThis->m_fBlocksClearedHangTime)
	{
		alpha = 1.0f;
		scale = 1.0f + ((pThis->m_fBlocksClearedTime - BB_BLOCKSCLEARED_GROWTIME) / pThis->m_fBlocksClearedHangTime) * 0.1f;
	}
	else
	{
		alpha = (pThis->m_fBlocksClearedTotalTime - pThis->m_fBlocksClearedTime) / BB_BLOCKSCLEARED_FADETIME;
		scale = 1.1f + (1.0f - alpha) * 0.4f;
	}

	xRenderable2DQuadSetPosition(pThis->m_pBlocksCleared, BB_BLOCKSCLEARED_LEFT + 0.5f * (BB_BLOCKSCLEARED_SIZEX * (1.0f - scale)), BB_BLOCKSCLEARED_TOP + 0.5f * (BB_BLOCKSCLEARED_SIZEY * (1.0f - scale)));
	xRenderable2DQuadSetSize(pThis->m_pBlocksCleared, BB_BLOCKSCLEARED_SIZEX * scale, BB_BLOCKSCLEARED_SIZEY * scale);
	xRenderable2DQuadUpdateAlpha(pThis->m_pBlocksCleared, alpha);
	xRenderable2DQuadUpdate(pThis->m_pBlocksCleared);

	xRenderable2DTextSetPosition(&pThis->m_xBlocksClearedText, pThis->m_pBlocksCleared->m_xRenderable2D.m_fX - (pThis->m_fBlocksClearedTextSize + 44.0f) * scale, BB_BLOCKSCLEARED_TEXT_TOP + 0.5f * (xFontGetSize(pThis->m_xBlocksClearedText.m_pFont) * (1.0f - scale)));
	xRenderable2DTextSetScale(&pThis->m_xBlocksClearedText, scale, scale);
	xRenderable2DTextUpdateAlpha(&pThis->m_xBlocksClearedText, alpha);
	xRenderable2DTextSetText(&pThis->m_xBlocksClearedText, &g_xGame.m_xSceneGraph, pThis->m_strBlocksCleared);

	temptime = pThis->m_fBlocksClearedTime - BB_BLOCKSCLEARED_GROWTIME;

	if(temptime < BB_BLOCKSCLEARED_GROWTIME)
	{
		alpha = temptime / BB_BLOCKSCLEARED_GROWTIME;
		scale = 0.5f + (alpha) * 0.5f;
	}
	else if(temptime < (BB_BLOCKSCLEARED_GROWTIME + pThis->m_fBlocksClearedHangTime) - BB_BLOCKSCLEARED_FADETIME)
	{
		alpha = 1.0f;
		scale = 1.0f + ((temptime - BB_BLOCKSCLEARED_GROWTIME) / pThis->m_fBlocksClearedHangTime) * 0.1f;
	}
	else
	{
		alpha = ((pThis->m_fBlocksClearedTotalTime - temptime) - BB_BLOCKSCLEARED_GROWTIME) / BB_BLOCKSCLEARED_FADETIME;
		scale = 1.1f + (1.0f - alpha) * 0.4f;
	}

	xRenderable2DQuadSetPosition(pThis->m_pBlocksClearedDesc, BB_BLOCKSCLEARED_DESC_LEFT + 0.5f * (BB_BLOCKSCLEARED_DESC_SIZEX * (1.0f - scale)), BB_BLOCKSCLEARED_DESC_TOP + 0.5f * (BB_BLOCKSCLEARED_DESC_SIZEY * (1.0f - scale)));
	xRenderable2DQuadSetSize(pThis->m_pBlocksClearedDesc, BB_BLOCKSCLEARED_DESC_SIZEX * scale, BB_BLOCKSCLEARED_DESC_SIZEY * scale);
	xRenderable2DQuadUpdateAlpha(pThis->m_pBlocksClearedDesc, alpha);
	xRenderable2DQuadUpdate(pThis->m_pBlocksClearedDesc);
}

void xLocalPlayer::HitWrongColor(DWORD dwHitTime)
{
	m_xBeatManager.m_iHitTimeIndex = 0;

	//DEBUGLOG("ResetComboCount - HitWrongColor\n");
	xLocalPlayerResetComboCount(this);

	m_xMultiplier.WrongColor();

	g_xGame.m_pCrowdNegativeSound->Play(g_xOptions.m_fSoundVolume);

	//DEBUGLOG("Failed - 0\n");
	m_pHitTimeSounds[0]->Play(g_xOptions.m_fSoundVolume);

	m_xBeatManager.m_dwLastHitTime = dwHitTime - g_xGame.m_dwGameStartTime;
	m_xBeatManager.m_fHitTimeScale = 1.0f;
	m_xBeatManager.m_iHitTimeIndex = 6;
			
	g_xGame.AdjustCrowdBar(g_xGame.GetCrowdBarFailedToAddBlockAdjustment(g_xGameOptions.m_iDifficultyLevel));

	if(g_xPaperAPI.m_iNetMode != PE_NETMODE_SINGLEPLAYER)
		xGameNetwork::SendFailedToAddBlock(g_xGame.GetCrowdBarPosition());

#if WITH_BALANCESTATS
	g_xBalanceStats.HitWrongColor();

	if(!m_xPiece.m_iUnfilled)
		g_xBalanceStats.NewPiece();
#endif
}

void xLocalPlayer::MissedBeat()
{
	m_xMultiplier.MissedBeat();

	g_xGame.m_pCrowdNegativeSound->Play(g_xOptions.m_fSoundVolume);

	//DEBUGLOG("Failed - 0\n");
	m_pHitTimeSounds[0]->Play(g_xOptions.m_fSoundVolume);

	g_xGame.AdjustCrowdBar(g_xGame.GetCrowdBarMissedBeatAdjustment(g_xGameOptions.m_iDifficultyLevel));

	if(g_xPaperAPI.m_iNetMode != PE_NETMODE_SINGLEPLAYER)
		xGameNetwork::SendMissedBeat(m_iComboCount, g_xGame.GetCrowdBarPosition());

	//DEBUGLOG("ResetComboCount - MissedBeat\n");
	xLocalPlayerResetComboCount(this);

#if WITH_BALANCESTATS
	g_xBalanceStats.MissedBeat();

	if(!m_xPiece.m_iUnfilled)
		g_xBalanceStats.NewPiece();
#endif
}

void xLocalPlayer::MissedCombo()
{
	m_xMultiplier.MissedRequiredCombo();

	g_xGame.m_pCrowdNegativeSound->Play(g_xOptions.m_fSoundVolume);

	g_xGame.AdjustCrowdBar(g_xGame.GetCrowdBarMissedComboAdjustment(g_xGameOptions.m_iDifficultyLevel));

	if(g_xPaperAPI.m_iNetMode != PE_NETMODE_SINGLEPLAYER)
		xGameNetwork::SendMissedCombo(m_iComboCount, g_xGame.GetCrowdBarPosition());
}

void xLocalPlayer::WonGame()
{
	if(!m_bWonGame)
	{
		g_xGame.m_pPlayerModel->SetAnimation(BB_PLAYERANIMATION_WON, g_xGameOptions.m_fBPM);

		if(g_xGame.m_pEnemyModel)
			g_xGame.m_pEnemyModel->SetAnimation(BB_ENEMYANIMATION_LOST, g_xGameOptions.m_fBPM);

		g_xGame.Pause(g_xPaperAPI.m_bHasFocus);
		m_bWonGame = 1;
		m_bViewingPostPopup = 0;
		m_bReadyToMoveOn = 0;
		g_xGame.m_pCrowdGameWonSound->Play(g_xOptions.m_fSoundVolume);

		if(g_xGame.m_pWonVoiceSound)
			g_xGame.m_pWonVoiceSound->Play(0.7);

		if(g_xGame.m_xCrowdState != BB_CROWD_EXCITED || !g_xGame.m_pCrowdLoopExcitedSound->IsPlaying())
		{
			g_xGame.m_pCrowdLoopSound->Stop();
			g_xGame.m_pCrowdLoopExcitedSound->Play(g_xOptions.m_fSoundVolume, 1);
			g_xGame.m_xCrowdState = BB_CROWD_EXCITED;
		}
	}
}

void xLocalPlayer::LevelComplete(int iLevelIndex, DWORD dwGameCompletionTime)
{
	DWORD score = ((0.5f - g_xGame.m_fCrowdBarPercentage) / 0.5f) * 100.0f;
	float completion, averagescore;
	uint32_t totaltime;
	int i, j;

	if(score == 0)
		score = 1;

	if(iLevelIndex >= g_xCurrentProfile.m_iCurrentLevel)
		g_xCurrentProfile.m_iCurrentLevel = iLevelIndex + 1;

	if(g_xCurrentProfile.m_iCurrentLevel > 2)
		g_xGame.UnlockAchievement(BB_ACH_DOMINATE_TRAINING);

	if(score > g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][iLevelIndex])
	{
		g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][iLevelIndex] = score;

		if(iLevelIndex <= 2)
		{
			for(i = 0; i < BB_NUMDIFFICULTIES; i++)
				if(g_xCurrentProfile.m_u32HighScores[i][iLevelIndex] < score)
					g_xCurrentProfile.m_u32HighScores[i][iLevelIndex] = score;
		}
		else
		{
			for(i = 0; i < g_xCurrentProfile.m_byteSelectedDifficulty; i++)
				if(g_xCurrentProfile.m_u32HighScores[i][iLevelIndex] < score)
					g_xCurrentProfile.m_u32HighScores[i][iLevelIndex] = score;
	}
	}

	for(i = 1, j = 0; i < BB_LEVELS_MAX + 1; i++)
	{
		if(g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][iLevelIndex] == 0)
			break;

		if(g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][iLevelIndex] >= 100)
			j = i;
	}

	if(g_xCurrentProfile.m_u32LevelTimes[g_xGameOptions.m_iLevelIndex] == 0 || dwGameCompletionTime < g_xCurrentProfile.m_u32LevelTimes[g_xGameOptions.m_iLevelIndex])
		g_xCurrentProfile.m_u32LevelTimes[g_xGameOptions.m_iLevelIndex] = dwGameCompletionTime;

	if(g_xCurrentProfile.m_strLoginName)
	{
		completion = 0.0;
		averagescore = 0.0;
		totaltime = 0;

		for(j = 1; j <= BB_LEVELS_MAX; j++)
		{
			if(g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][j] > 0)
			{
				completion += 1.0;
				totaltime += g_xCurrentProfile.m_u32LevelTimes[j];
			}

			if(g_xCurrentProfile.m_u32HighScores[g_xCurrentProfile.m_byteSelectedDifficulty][j] >= 100)
				averagescore += g_xCurrentProfile.m_u32HighScores[i][j];
		}

		averagescore /= completion;
		completion *= 100.0 / 44.0;
		totaltime /= 1000;

		g_xGame.m_xHTTPQuery.UpdateSPLeaderboard(g_xCurrentProfile.m_strLoginName, g_xCurrentProfile.m_strLoginPassword, g_xCurrentProfile.m_byteSelectedDifficulty, completion, averagescore, totaltime);
	}

	if(i == BB_LEVELS_MAX)
	{
		for(i = 0; i < g_xCurrentProfile.m_byteSelectedDifficulty; i++)
			g_xGame.UnlockAchievement(BB_ACH_WIN_SIMPLE + i);

		if(j == BB_LEVELS_MAX)
		{
			for(i = 0; i < g_xCurrentProfile.m_byteSelectedDifficulty; i++)
				g_xGame.UnlockAchievement(BB_ACH_DOMINATE_SIMPLE + i);
		}
	}

	g_xOptions.SavePlayerProfile(&g_xCurrentProfile);
}

FORCE_INLINE void xLocalPlayerResetComboCount(xLocalPlayer *pThis)
{
	pThis->m_iComboCount = 0;
	pThis->m_bComboFade = 1;
	pThis->m_fComboTime = BB_COMBO_TOTALTIME;
}
