/////////////////////////////////////////////////////////////////
// Name: xMD3Model.h
/////////////////////////////////////////////////////////////////
// Description:
// - An MD3 model, which may be linked to other 3D Models
//   behind the scenes
/////////////////////////////////////////////////////////////////
// - 1.27.2010, Update, Dayle Flowers(Xienen)
//   Created the file
/////////////////////////////////////////////////////////////////
#ifndef XMD3MODEL_H_
#define XMD3MODEL_H_

#include "xModel.h"
#include "xMD3Structs.h"

void xMD3ModelPreUpdate(xModel* pThis, DWORD dwTime);
void xMD3ModelUpdate(xModel* pThis, double dTime);

class xMD3Model : public xModel
{
public:
	xMD3Model()
	{
		pfnPreUpdate = &xMD3ModelPreUpdate;
		pfnUpdate = &xMD3ModelUpdate;

		pfnOnAnimationCompleted = 0;
		m_pAnimCompleteCallbackObject = 0;

		m_pMeshes = 0;
		m_pRenderables = 0;
		m_pAnimations = 0;
		m_iCurrentAnim = 0;
		m_iCurrentFrame = -1;
		m_iNextFrame = 1;
		m_dwFrameStartTime = 0;
		m_fCurrentFPS = 30.0;
		m_pTags = 0;
	}

	void Init(const char* strName, int iAnimationCount, xSceneGraph *pSceneGraph, float fRenderDepth);
	void RefreshTextures();
	void Delete();

	void SetRotation(float fX, float fY, float fZ);
	void SetAnimation(int iAnimIndex, float fBPM) { m_iCurrentAnim = iAnimIndex; if ( m_iCurrentFrame == -1 ) m_iCurrentFrame = m_pAnimations[iAnimIndex].m_iFirstFrame; m_iNextFrame = m_pAnimations[iAnimIndex].m_iFirstFrame; m_fCurrentFPS = ((float)(m_pAnimations[iAnimIndex].m_iLastFrame + 1 - m_pAnimations[iAnimIndex].m_iFirstFrame) * 100.0f) / (45.0f * m_pAnimations[iAnimIndex].m_iNumBeats); }

	void (*pfnOnAnimationCompleted)(xMD3Model *pModel, void *pCallbackObject);
	void *m_pAnimCompleteCallbackObject;

	xMesh			 *m_pMeshes;
	xRenderable3D	**m_pRenderables;
	int				  m_iMeshCount;
	xMD3Animation	 *m_pAnimations;
	int				  m_iCurrentAnim;
	int				  m_iCurrentFrame;
	float			  m_fCurrentFPS;
	int				  m_iNextFrame;
	DWORD			  m_dwFrameStartTime;
	float			  m_fFrameTimePct;

private:
	void LoadMD3File(char* strFile);
	void LoadSkinFile(char* strFile);
	void LoadAnimationFile(char* strFile, int iAnimatonCount);

	xFileIO			 m_xFile;
	xMD3Header		 m_xHeader;
	xMD3Tag			*m_pTags;
	int				 m_iTagCount;
};

#endif
