/////////////////////////////////////////////////////////////////
// Name: xMultiplier.h
/////////////////////////////////////////////////////////////////
// Description:
// - Tracks the current Multiplier and progress toward the next.
/////////////////////////////////////////////////////////////////
// - 11.29.2009, Update, Dayle Flowers(Xienen)
//   File Created
/////////////////////////////////////////////////////////////////
#ifndef XMULTIPLIER_H_
#define XMULTIPLIER_H_

class xMultiplier
{
public:
	xMultiplier()
	{
		m_pWhiteBlockTimer			= 0;
		m_pMultiplierObject			= 0;
		m_pWhiteMultiplierObject	= 0;
		
		Reset();
	}

	void Reset()
	{
		m_iCurrentMultiplier = 2;
		m_fMultiplierTimer = 0.0;
		m_fMultiplierPercentage = 0.0f;
		m_fTargetMultiplierPercentage = 0.0f;
		m_fMaxMultiplierTime = 0.0f;
		m_bMultiplierIncreased = 0;
		m_bMultiplierDecreased = 0;
		m_bNewMultiplierTime = 0;
		m_fFadeToWhiteTime = 0.0f;
		m_fFadeOutTime = 0.0f;
	}

	void	Init();
	void	Update(double dTime);
	void	RefreshAllRenderables();
	bool	ShouldAddBlock() { return m_bMultiplierIncreased; }
	bool	ShouldRemoveBlock() { return m_bMultiplierDecreased; }
	int		GetCurrentMultiplier() { return m_iCurrentMultiplier; }
	bool	TimerRunning() { return m_fMultiplierTimer > 0.0; }
	void	MultiplierAddTime(float fAdditionalTime);

	FORCE_INLINE	void	AddedBlock(int iComboCount);
	FORCE_INLINE	void	PlayerMoveExecuted(float fWhiteBlockTime);
	FORCE_INLINE	void	WrongColor();
	FORCE_INLINE	void	MissedBeat();
	FORCE_INLINE	void	MissedRequiredCombo();

private:
	xRenderable2DQuad	*m_pWhiteBlockTimer;
	xRenderable2DQuad	*m_pMultiplierObject;
	xRenderable2DQuad	*m_pWhiteMultiplierObject;
	int					 m_iCurrentMultiplier;
	float				 m_fMultiplierPercentage;
	float				 m_fMultiplierTimer;
	float				 m_fMaxMultiplierTime;

	// Animation Helpers
	bool	m_bMultiplierIncreased;
	bool	m_bMultiplierDecreased;
	bool	m_bNewMultiplierTime;
	float	m_fMultiplierAnimationTime;
	float	m_fOldMultiplierTime;
	float	m_fTargetMultiplierPercentage;
	float	m_fFadeToWhiteTime;
	float	m_fFadeOutTime;
};

void xMultiplier::AddedBlock(int iComboCount)
{
	//DEBUGLOG("AddedBlock %i %i %i %f\n", m_iCurrentMultiplier, iComboCount, iComboCount % 10, m_fTargetMultiplierPercentage);

	if(m_iCurrentMultiplier < 9)
	{
		if(iComboCount % 10 == 0)
		{
			m_iCurrentMultiplier++;
			m_bMultiplierIncreased = 1;
		}

		m_fTargetMultiplierPercentage += .10f;
		m_fFadeToWhiteTime = 0.0f;
		m_fFadeOutTime = 0.0f;

		//DEBUGLOG("New Multiplier Target Percentage: %f\n", m_fTargetMultiplierPercentage);
	}
}

void xMultiplier::WrongColor()
{
	if(m_iCurrentMultiplier > 2)
	{
		m_iCurrentMultiplier--;
		m_bMultiplierDecreased = 1;
	}

	m_fTargetMultiplierPercentage = 0.0f;
}

void xMultiplier::MissedBeat()
{
	if(m_iCurrentMultiplier > 2)
	{
		m_iCurrentMultiplier--;
		m_bMultiplierDecreased = 1;
	}

	m_fTargetMultiplierPercentage = 0.0f;
}

void xMultiplier::MissedRequiredCombo()
{
	if(m_iCurrentMultiplier > 2)
	{
		m_iCurrentMultiplier--;
		m_bMultiplierDecreased = 1;
	}

	m_fTargetMultiplierPercentage = 0.0f;
}

#endif