#include "Paper.h"

// Support for grabbing MAC Address
#include <net/if_dl.h>
#include <ifaddrs.h>

inline fd_set* CreateSet(int iSocket, fd_set *pSet)
{
	if(!FD_ISSET(iSocket, pSet))
	{
		FD_SET(iSocket, pSet);
	}
	
	return pSet;
}

int xNetworkClient::Init(bool bTCP, int iWinProcMessage)
{
	m_iWinProcMessage = iWinProcMessage;

	m_bTCP = bTCP;
	
	FD_ZERO(&m_setConnection);
	FD_ZERO(&m_setListen);

	return PE_RES_NOERROR;
}

int xNetworkClient::Listen(int iPort)
{
	memset(&m_sockAddrIn, 0, sizeof(m_sockAddrIn));
	m_sockAddrIn.sin_family		 = AF_INET;
	m_sockAddrIn.sin_port		 = htons(iPort);
	m_sockAddrIn.sin_addr.s_addr = htonl(INADDR_ANY);

	if(m_bTCP)
	{
		if(m_sockListen == INVALID_SOCKET)
		{
			m_sockListen = socket(AF_INET, SOCK_STREAM, 0);
			
			int flags = fcntl(m_sockListen, F_GETFL, 0);
			fcntl(m_sockListen, F_SETFL, flags | O_NONBLOCK);

			if(bind(m_sockListen, (sockaddr*)&m_sockAddrIn, sizeof(m_sockAddrIn)) == SOCKET_ERROR || errno)
			{
				return PE_RES_WSINTERNALERROR;
			}

			if(listen(m_sockListen, 5) == SOCKET_ERROR || errno)
			{
				return PE_RES_WSINTERNALERROR;
			}
			
			m_xStatus = PE_SOCKET_LISTENING;
		}
	}
	else
	{
		if(m_bUDPClientBound)
			Close();

		m_sockConnection = socket(AF_INET, SOCK_DGRAM, 0);

		int flags = fcntl(m_sockConnection, F_GETFL, 0);
		fcntl(m_sockConnection, F_SETFL, flags | O_NONBLOCK);

		if(bind(m_sockConnection, (sockaddr*)&m_sockAddrIn, sizeof(m_sockAddrIn)) == SOCKET_ERROR || errno)
		{
			return PE_RES_WSINTERNALERROR;
		}

		m_bUDPClientBound = true;
	}

	return PE_RES_NOERROR;
}

int xNetworkClient::Connect(const char* strIP, int iPort)
{
	int error;
	
	if(m_sockConnection == INVALID_SOCKET)
	{
		if(m_bTCP)
			m_sockConnection = socket(AF_INET, SOCK_STREAM, 0);
		else
			m_sockConnection = socket(AF_INET, SOCK_DGRAM, 0);

		errno;
		
		fcntl(m_sockConnection, F_SETFL, O_NONBLOCK);
	}

	memset(&m_sockAddrIn, 0, sizeof(m_sockAddrIn));
	m_sockAddrIn.sin_family		 = AF_INET;
	m_sockAddrIn.sin_port		 = htons(iPort);
	m_sockAddrIn.sin_addr.s_addr = inet_addr(strIP);

	if(m_bTCP)
	{
		if(connect(m_sockConnection, (sockaddr*)&m_sockAddrIn, sizeof(sockaddr_in)) == SOCKET_ERROR)
		{
			error = errno;
			if(error && error != EINPROGRESS)
				return PE_RES_WSINTERNALERROR;
		}
		
		m_xStatus = PE_SOCKET_CONNECTING;
	}
	else
	{
		if(m_bUDPClientBound)
			Close();

		sockaddr_in bindaddr = {0};
		bindaddr.sin_family		 = AF_INET;
		bindaddr.sin_port		 = htons(0);
		bindaddr.sin_addr.s_addr = htonl(INADDR_ANY);

		if(bind(m_sockConnection, (sockaddr*)&bindaddr, sizeof(bindaddr)) == SOCKET_ERROR || errno)
		{
			return PE_RES_WSINTERNALERROR;
		}

		m_bUDPClientBound = true;

		m_bConnected = true;
	}

	return PE_RES_NOERROR;
}

int xNetworkClient::Accept()
{
	if(m_sockConnection == INVALID_SOCKET)
	{
		int sizeaddr = sizeof(m_sockAddress);
		m_sockConnection = accept(m_sockListen, &m_sockAddress, (socklen_t*)&sizeaddr);
		if(m_sockConnection == INVALID_SOCKET || errno)
			return PE_RES_WSINTERNALERROR;
		
		m_xStatus = PE_SOCKET_CONNECTED;
	}
	else
	{
		sockaddr tempaddr;
		int sizeaddr = sizeof(tempaddr);
		int tempconnection = accept(m_sockListen, &tempaddr, (socklen_t*)&sizeaddr);
		close(tempconnection);
	}

	return PE_RES_NOERROR;
}

int xNetworkClient::Read()
{
	if(m_bTCP)
	{
		char buffer[MAX_NETWORK_PACKETSIZE] = {0};

		int readsize = recv(m_sockConnection, buffer, MAX_NETWORK_PACKETSIZE, 0);

		if(readsize > 0)
		{
			memcpy(&m_strReceiveBuffer[m_iReadBufferOffset], buffer, readsize);

			m_iReadBufferOffset += readsize;
		}
	}
	else
	{
		xUDPPacketWinsock* packet = &m_xUDPPackets;

		while(packet->m_pNext)
			packet = packet->m_pNext;

		packet->m_pNext = new xUDPPacketWinsock;
		memset(packet->m_pNext->m_strData, 0, MAX_NETWORK_PACKETSIZE);

		int sockaddrsize = sizeof(packet->m_pNext->m_sockAddrIn);

		int readsize = recvfrom(m_sockConnection, packet->m_pNext->m_strData, MAX_NETWORK_PACKETSIZE, 0, (sockaddr*)&packet->m_pNext->m_sockAddrIn, (socklen_t*)&sockaddrsize);

		if(readsize <= 0)
		{
			delete packet->m_pNext;
			packet->m_pNext = NULL;
		}
		else
		{
			packet = packet->m_pNext;
			packet->m_pNext = NULL;
			packet->m_dwTimeReceived = g_xPaperAPI.m_xPlatform.GetTime();
		}
	}

	return PE_RES_NOERROR;
}

int xNetworkClient::Update()
{
	struct timeval timeout = {0, 100};
	int error, sent, i;
	
	if(m_bTCP)
	{
		if(m_xStatus == PE_SOCKET_CONNECTING)
		{
			if(select(m_sockConnection + 1, 0, 0, CreateSet(m_sockConnection, &m_setConnection), &timeout))
				m_xStatus = PE_SOCKET_UNUSED;
			else if((connect(m_sockConnection, (sockaddr*)&m_sockAddrIn, sizeof(sockaddr_in)) && errno == EISCONN) ||
					 select(m_sockConnection + 1, 0, CreateSet(m_sockConnection, &m_setConnection), 0, &timeout))
			{
				m_xStatus = PE_SOCKET_CONNECTED;
				m_bConnected = 1;
			}
		}
		else if(m_xStatus == PE_SOCKET_LISTENING)
		{
			if(select(m_sockListen + 1, 0, 0, CreateSet(m_sockListen, &m_setListen), &timeout))
				m_xStatus = PE_SOCKET_UNUSED;
			else if(select(m_sockListen + 1, CreateSet(m_sockListen, &m_setListen), 0, 0, &timeout))
				Accept();
		}
		else if(m_xStatus == PE_SOCKET_CONNECTED)
		{
			if(select(m_sockConnection + 1, CreateSet(m_sockConnection, &m_setConnection), 0, 0, &timeout))
				Read();

			if(m_iWriteBufferOffset > 0 && select(m_sockConnection + 1, 0, CreateSet(m_sockConnection, &m_setConnection), 0, &timeout))
			{
				if(m_iWriteBufferOffset >= MAX_NETWORK_PACKETSIZE)
				{
					i = 0;
					sent = send(m_sockConnection, m_strSendBuffer, MAX_NETWORK_PACKETSIZE, 0);

					if(sent > 0)
					{
						for(; i < m_iWriteBufferOffset; i++)
						{
							m_strSendBuffer[i - sent] = m_strSendBuffer[i];
							m_strSendBuffer[i] = 0;
						}

						m_iWriteBufferOffset -= sent;
					}
				}
				else
				{
					sent = send(m_sockConnection, m_strSendBuffer, m_iWriteBufferOffset, 0);

					if(sent == m_iWriteBufferOffset)
					{
						memset(m_strSendBuffer, 0, MAX_NETWORK_BUFFERSIZE);
						m_iWriteBufferOffset = 0;
					}
					else if(sent > 0)
					{
						for(i = 0; i < m_iWriteBufferOffset; i++)
						{
							m_strSendBuffer[i - sent] = m_strSendBuffer[i];
							m_strSendBuffer[i] = 0;
						}

						m_iWriteBufferOffset -= sent;
					}
				}
			}
		}
	}

	return PE_RES_NOERROR;
}

void xNetworkClient::Close()
{
	if(m_sockConnection != INVALID_SOCKET)
		FD_CLR(m_sockConnection, &m_setConnection);
	
	if(m_sockConnection != INVALID_SOCKET)
		close(m_sockConnection);

	m_bConnected		= 0;
	m_bWritable			= 0;
	m_bUDPClientBound	= 0;
	m_sockConnection	= INVALID_SOCKET;
}

void xNetworkClient::Shutdown()
{
	//Send disconnect message?
}

int xNetworkClient::Send(const char *strData, int iSize)
{
	int sent;

	if(m_bTCP)
	{
		memcpy(&m_strSendBuffer[m_iWriteBufferOffset], strData, iSize);

		m_iWriteBufferOffset += iSize;
	}
	else
	{
		sent = sendto(m_sockConnection, strData, iSize, 0, (sockaddr*)&m_sockAddrIn, sizeof(m_sockAddrIn));
	}

	return PE_RES_NOERROR;
}

int xNetworkClient::SendResponse(const char *strData, int iSize)
{
	if(!m_bTCP)
	{
		sendto(m_sockConnection, strData, iSize, 0, (sockaddr*)&m_xRespondAddress, sizeof(m_xRespondAddress));
	}

	return PE_RES_NOERROR;
}

char* xNetworkClient::Receive(char strBuffer[MAX_NETWORK_BUFFERSIZE + 1], DWORD &dwTimeReceived)
{
	if(m_bTCP)
	{
		memcpy(strBuffer, m_strReceiveBuffer, MAX_NETWORK_BUFFERSIZE);
		strBuffer[MAX_NETWORK_BUFFERSIZE] = '\0';

		memset(m_strReceiveBuffer, 0, MAX_NETWORK_BUFFERSIZE);

		m_iReadBufferOffset = 0;
	}
	else
	{
		xUDPPacketWinsock* packet = m_xUDPPackets.m_pNext;
		if(packet)
		{
			m_xUDPPackets.m_pNext = packet->m_pNext;

			memcpy(strBuffer, packet->m_strData, MAX_NETWORK_PACKETSIZE);

			dwTimeReceived = packet->m_dwTimeReceived;

			memcpy(&m_xRespondAddress, &packet->m_sockAddrIn, sizeof(sockaddr_in));

			delete packet;
		}
		else
		{
			return NULL;
		}
	}

	return strBuffer;
}

void xNetworkClient::GetMacAddress(uint8_t *pMAC)
{
	ifaddrs   *ifaphead;
	ifaddrs   *ifap;
	sockaddr_dl *sdl = NULL;
	
	if(getifaddrs(&ifaphead))
	{
		memset(pMAC, 0, 8);
		return;
	}
	
	for(ifap = ifaphead; ifap; ifap = ifap->ifa_next)
	{
		if(ifap->ifa_addr->sa_family == AF_LINK)
		{
			sdl = (struct sockaddr_dl*)ifap->ifa_addr;
			if(sdl && sdl->sdl_alen && LLADDR(sdl))
			{
				pMAC[0] = LLADDR(sdl)[0];
				pMAC[1] = LLADDR(sdl)[1];
				pMAC[2] = LLADDR(sdl)[2];
				pMAC[3] = LLADDR(sdl)[3];
				pMAC[4] = LLADDR(sdl)[4];
				pMAC[5] = LLADDR(sdl)[5];
				pMAC[6] = LLADDR(sdl)[6];
				pMAC[7] = LLADDR(sdl)[7];

				if(ifaphead)
					freeifaddrs(ifaphead);
				
				return;
			}
		}
	}

	if(ifaphead)
		freeifaddrs(ifaphead);

	memset(pMAC, 0, 8);
}
