#include "Paper.h"

const float g_DegToRadMultiplier = 0.017444f;

extern void GameInitEngineSettings();
extern void GameRefreshAllRenderables();

#ifndef FINAL_RELEASE
	char g_strTempLog[10240];
#endif

int xPaper::Init()
{
	float tempfloat;
	WindowPtr window;

	m_bChangingScreenResolution	= 0;
	m_bOutlineModels			= 1;
	m_bLimitParticles			= 0;

	GameInitEngineSettings();

	if(!(window = m_xPlatform.MakeWindow(m_xEngineSettings.m_iScreenWidth, m_xEngineSettings.m_iScreenHeight, 32, m_xEngineSettings.m_bFullScreen)))
		assert(0);

	m_iNetMode		= PE_NETMODE_SINGLEPLAYER;
	m_bRunning		= 1;
	m_bHasFocus		= 1;
	m_bDragged		= 0;
	m_pWindow		= window;
	m_iFPS			= 0;
	m_iCurrentFPS	= 60;

	m_strExitCommand[0]	= '\0';

	UpdateScreenOffsetsAndRatios();

	if((m_iResult = xGraphicsInit(m_pWindow, m_xEngineSettings.m_iScreenWidth, m_xEngineSettings.m_iScreenHeight, m_xEngineSettings.m_bFullScreen, 32)))
	{
		if(m_iResult != 11)
			assert(0);
	}

	if((m_iResult = m_xAudio.Init()))
		assert(0);

	if((m_iResult = m_xInput.Init()))
		assert(0);

	if(m_iResult = m_xNetwork.Init(1, WM_PE_WSASYNC))
		assert(0);

	if(m_iResult = m_xHTTPNetwork.Init(1, WM_PE_HTTPASYNC))
		assert(0);

	if(m_iResult = m_xPingNetwork.Init(0, WM_PE_PINGASYNC))
		assert(0);

	if(m_iResult = m_xFTPNetwork.Init(1, WM_PE_FTPASYNC))
		assert(0);

	if(m_iResult = m_xFTPFileTransfer.Init(1, WM_PE_FTPTRANSFERASYNC))
		assert(0);

	return PE_RES_NOERROR;
}

void xPaper::PreUpdate()
{
	m_dwCurrentTime = m_xPlatform.GetTime();

	m_iFPS++;

	xGraphicsPreUpdate(m_dwCurrentTime);
}

void xPaper::Update()
{
	static double fpstime = 0.0;

	xGraphicsUpdate((double)(m_dwCurrentTime - m_dwLastTime) * 0.001);

	//Handle FPS Counting
	fpstime += (double)(m_dwCurrentTime - m_dwLastTime) * 0.001;
	if(fpstime > 1.0)
	{
		m_iCurrentFPS = m_iFPS;
		m_iFPS = 0;
		fpstime = 0.0;
	}

	m_dwLastTime = m_dwCurrentTime;
}

void xPaper::Shutdown()
{
	m_xPlatform.Shutdown();

	m_bRunning = false;
}

void xPaper::UpdateScreenOffsetsAndRatios()
{
	float tempfloat;

	if(m_xEngineSettings.m_bForce4x3WhenWideScreen)
	{
		tempfloat = (float)m_xEngineSettings.m_iScreenWidth / (float)m_xEngineSettings.m_iScreenHeight;

		if(tempfloat < 1.3333330f) // Tall Screen
		{
			m_iScreenOffsetX = 0;
			m_iScreenOffsetY = (m_xEngineSettings.m_iScreenHeight - ((int)((float)m_xEngineSettings.m_iScreenWidth / 1.333333333333f))) / 2;
		}
		else if(tempfloat > 1.3333334f) // Wide Screen
		{
			m_iScreenOffsetX = (m_xEngineSettings.m_iScreenWidth - ((int)((float)m_xEngineSettings.m_iScreenHeight * 1.333333333333f))) / 2;
			m_iScreenOffsetY = 0;
		}
		else
		{
			m_iScreenOffsetX = 0;
			m_iScreenOffsetY = 0;
		}
	}
	
	m_fScreenRatioX	= ((float)(m_xEngineSettings.m_iScreenWidth - (m_iScreenOffsetX * 2))) / 1024.0;
	m_fScreenRatioY	= ((float)(m_xEngineSettings.m_iScreenHeight - (m_iScreenOffsetY * 2))) / 768.0;
}

void xPaper::ChangeScreenResolution(int iScreenWidth, int iScreenHeight, bool bFullscreen)
{
	xTexture *texture;

	m_bChangingScreenResolution = true;

	texture = g_xGraphics.m_xTexturePool.m_pNext;

	while(texture)
	{
		glDeleteTextures(1, &texture->m_uiID);

		texture = texture->m_pNext;
	}	

	aglDestroyContext(g_xGraphics.m_hContext);

	CFRelease(g_xPaperAPI.m_pWindow);

	//if(m_xEngineSettings.m_bFullScreen && !bFullscreen)
	//	ChangeDisplaySettings(&g_xGraphics.m_dmOldSettings, 0);

	m_xEngineSettings.m_iScreenWidth = iScreenWidth;
	m_xEngineSettings.m_iScreenHeight = iScreenHeight;
	m_xEngineSettings.m_bFullScreen = bFullscreen;

	if(!(m_pWindow = m_xPlatform.MakeWindow(m_xEngineSettings.m_iScreenWidth, m_xEngineSettings.m_iScreenHeight, 32, m_xEngineSettings.m_bFullScreen)))
		assert(0);

	UpdateScreenOffsetsAndRatios();

	if((m_iResult = m_xAudio.Init()))
		assert(0);

	if((m_iResult = xGraphicsInit(m_pWindow, m_xEngineSettings.m_iScreenWidth, m_xEngineSettings.m_iScreenHeight, m_xEngineSettings.m_bFullScreen, 32)))
	{
		if(m_iResult != 11)
			assert(0);
	}

	xGraphicsGoto2DMode();

	xGraphicsReloadTexturePool();

	GameRefreshAllRenderables();

	m_bChangingScreenResolution = false;
}

void xPaper::ValidateCDKey(char *strCDKey)
{
	char randomkey62[4][7] = {"aaaaaa", "aaaaaa", "aaaaaa", "aaaaaa"};
	char privatekey62[2][7] = {"aaaaaa", "aaaaaa"};
	uint32_t privatekey[2] = {PRIVATE_KEY_ENCRYPTED0, PRIVATE_KEY_ENCRYPTED1};
	uint32_t privatekeykey[4] = {PRIVATE_KEY_KEY0, PRIVATE_KEY_KEY1, PRIVATE_KEY_KEY2, PRIVATE_KEY_KEY3};
	uint32_t privatekeyinput[2], randomkey[4];

	xDecrypt(privatekey, 2, privatekeykey);

	privatekey62[0][0] = strCDKey[0];
	randomkey62[0][3] = strCDKey[1];
	privatekey62[1][0] = strCDKey[2];
	randomkey62[3][3] = strCDKey[3];
	privatekey62[0][1] = strCDKey[4];

	randomkey62[2][3] = strCDKey[6];
	privatekey62[1][1] = strCDKey[7];
	randomkey62[1][3] = strCDKey[8];
	privatekey62[0][2] = strCDKey[9];
	randomkey62[0][4] = strCDKey[10];

	privatekey62[1][2] = strCDKey[12];
	randomkey62[3][4] = strCDKey[13];
	privatekey62[0][3] = strCDKey[15];
	randomkey62[2][4] = strCDKey[16];

	privatekey62[1][3] = strCDKey[18];
	randomkey62[1][4] = strCDKey[19];
	privatekey62[0][4] = strCDKey[20];
	randomkey62[0][5] = strCDKey[21];
	privatekey62[1][4] = strCDKey[22];

	randomkey62[3][5] = strCDKey[24];
	privatekey62[0][5] = strCDKey[25];
	randomkey62[2][5] = strCDKey[26];
	privatekey62[1][5] = strCDKey[27];
	randomkey62[1][5] = strCDKey[28];

	randomkey[0] = xFromBase62(randomkey62[0]);
	randomkey[1] = xFromBase62(randomkey62[1]);
	randomkey[2] = xFromBase62(randomkey62[2]);
	randomkey[3] = xFromBase62(randomkey62[3]);

	privatekeyinput[0] = xFromBase62(privatekey62[0]);
	privatekeyinput[1] = xFromBase62(privatekey62[1]);

	xDecrypt(privatekeyinput, 2, randomkey);

	if(privatekeyinput[0] != privatekey[0] || privatekeyinput[1] != privatekey[1])
		strCDKey[0] = '+';
}