#include "peGame.h"

FORCE_INLINE void xPieceCreateBuildArrowObject(xBlock *pBlock);
FORCE_INLINE void xPieceUpdateBeatMarkerLocation(xPiece *pThis);
FORCE_INLINE void xPieceUpdateBeatMarker(xRenderable2DQuad *pBeatMarker, float fBeatTime, int iBeatTime, float fBeatMarkerX, float fBeatMarkerY);

#define BB_PIECELIST_LENGTH1	16
#define BB_PIECELIST_LENGTH2	21
#define BB_PIECELIST_LENGTH3	22

const xColor g_colUnfilledBlockColors[BB_PIECECOLORS_MAX] = {{255.0 / 255.0,   0.0 / 255.0,   0.0 / 255.0, 1.0},
															 { 50.0 / 255.0, 102.0 / 255.0, 255.0 / 255.0, 1.0},
															 {255.0 / 255.0, 204.0 / 255.0,   0.0 / 255.0, 1.0},
															 {240.0 / 255.0,   0.0 / 255.0, 255.0 / 255.0, 1.0},
															 {255.0 / 255.0, 102.0 / 255.0,   0.0 / 255.0, 1.0},
															 {  6.0 / 255.0, 255.0 / 255.0,   0.0 / 255.0, 1.0},
															 {255.0 / 255.0, 255.0 / 255.0, 255.0 / 255.0, 1.0}};

const xColor g_colFilledBlockColors[BB_PIECECOLORS_MAX] = {{255.0 / 255.0,   0.0 / 255.0,   0.0 / 255.0, 1.0},
														   { 50.0 / 255.0, 102.0 / 255.0, 255.0 / 255.0, 1.0},
														   {255.0 / 255.0, 204.0 / 255.0,   0.0 / 255.0, 1.0},
														   {240.0 / 255.0,   0.0 / 255.0, 255.0 / 255.0, 1.0},
														   {255.0 / 255.0, 102.0 / 255.0,   0.0 / 255.0, 1.0},
														   {  6.0 / 255.0, 255.0 / 255.0,   0.0 / 255.0, 1.0},
														   {255.0 / 255.0, 255.0 / 255.0, 255.0 / 255.0, 1.0}};

const xColor g_colBlockGlowColors[BB_PIECECOLORS_MAX] = {{255.0 / 255.0, 128.0 / 255.0, 128.0 / 255.0, 1.0},
														 {128.0 / 255.0, 136.0 / 255.0, 255.0 / 255.0, 1.0},
														 {255.0 / 255.0, 235.0 / 255.0, 128.0 / 255.0, 1.0},
														 {250.0 / 255.0, 128.0 / 255.0, 255.0 / 255.0, 1.0},
														 {255.0 / 255.0, 192.0 / 255.0, 150.0 / 255.0, 1.0},
														 {128.0 / 255.0, 255.0 / 255.0, 128.0 / 255.0, 1.0},
														 {255.0 / 255.0, 255.0 / 255.0, 255.0 / 255.0, 1.0}};

xPiece::xPiece()
{
	m_iColorIndex			= -1;
	m_pBlockList			= NULL;
	m_pCurBlock				= NULL;
	m_iNumBlocks			= 0;
	m_bSkippedBeat			= false;
	m_bMissedCombo			= false;
	m_pBeatManager			= NULL;
	m_iCurFieldX			= 0;
	m_iCurFieldY			= -2;
	m_pBlockTextures		= 0;
	m_pBlockAnimation		= 0;
	m_bDoneAnimatingBlocks	= 1;
	m_pBeatMarkers[0]		= 0;
}

void xPiece::Init(float fFirstBlockX, float fFirstBlockY, int iBlockCount, bool bCenterAroundPoint, bool bNoBlocks)
{
	xBlock *debugblock;
	const char* piecestring;
	float fHighX = 0.0, fLowX = 0.0;
	float fHighY = 0.0, fLowY = 0.0;
	bool triedleft, triedright, triedup, trieddown;
	int newcolor, i;

	//DEBUGLOG("BlockCount: %i\n", iBlockCount);

	m_bDropping				= 0;
	m_bTouchedDown			= 0;
	m_bSkippedBeat			= 0;
	m_bMissedCombo			= 0;
	m_bPlayedTouchDown		= 0;
	m_dwCurTime				= 0;
	m_pPrevBlock			= 0;
	m_fBuildAnimationTime	= BB_PIECE_ANIMBUILDSUCCESSFUL + 0.1f;
	m_bDoneAnimatingBlocks	= 1;

	m_iNumBlocks = iBlockCount;

	//FIXME - server crashes on this, probably something messed up in difficulty settings?
	if(g_xGameOptions.m_iNumColors == 0)
		g_xGameOptions.m_iNumColors = 3;

	m_iColorIndex = rand() % g_xGameOptions.m_iNumColors;

	if(m_iColorIndex > BB_COLOR_YELLOW)
	{
		g_xGame.m_iSecondaryPiecesInARow++;

		if(g_xGame.m_iSecondaryPiecesInARow > 1)
		{
			m_iColorIndex = m_iColorIndex = rand() % 3;
			g_xGame.m_iSecondaryPiecesInARow = 0;
		}
	}
	else
		g_xGame.m_iSecondaryPiecesInARow = 0;

	if(m_pBlockList)
		Delete(1, 1, 0);

	m_pBlockList				= new xBlock;
	DEBUGCURBLOCK("Init setting CurBlock to %x\n", m_pBlockList);
	m_pCurBlock					= m_pBlockList;
	m_pBlockList->m_iColumnID	= 0;
	m_pBlockList->m_iRowID		= bNoBlocks ? 1 : 0;

	if(!m_pBlockAnimation)
	{
		m_pBlockAnimation = xSceneGraphCreateQuad(BB_IMAGE_PIECEBLOCKANIMATION, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pBlockAnimation, 1, BB_DEPTH_PIECEBLOCKANIMATION, 0, 1);
		xRenderable2DQuadSetSize(m_pBlockAnimation, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
		xRenderable2DQuadUpdateVisibility(m_pBlockAnimation, 0);
	}

	if(!m_pBlockTextures)
		xGraphicsGetTextures(BB_IMAGE_PIECEBLOCKS, &m_pBlockTextures);

	m_pCurBlock->m_pBlockObject = xSceneGraphCreateQuad(m_pBlockTextures[4]);
	xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pCurBlock->m_pBlockObject, 1, BB_DEPTH_CURPIECEBLOCK, 0, 1);
	xRenderable2DQuadSetPosition(m_pCurBlock->m_pBlockObject, fFirstBlockX, fFirstBlockY - (bNoBlocks ? BB_BLOCKOFFSET : 0));
	xRenderable2DQuadSetSize(m_pCurBlock->m_pBlockObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(m_pCurBlock->m_pBlockObject);

	m_pCurBlock->m_pBlockOverlayObject = xSceneGraphCreateQuad(BB_IMAGE_PIECEBLOCKOVERLAY, 0);
	xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pCurBlock->m_pBlockOverlayObject, 1, BB_DEPTH_CURPIECEBLOCKOVERLAY, 0, 0);
	xRenderable2DQuadSetPosition(m_pCurBlock->m_pBlockOverlayObject, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY);
	xRenderable2DQuadSetSize(m_pCurBlock->m_pBlockOverlayObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(m_pCurBlock->m_pBlockOverlayObject);

	m_pCurBlock->m_iUnfilledColorIndex = bNoBlocks ? 0 : rand() % g_xGameOptions.m_iNumColors;

	xRenderable2DQuadUpdateColor(m_pCurBlock->m_pBlockObject, &g_colUnfilledBlockColors[m_pCurBlock->m_iUnfilledColorIndex]);

	xPieceCreateBuildArrowObject(m_pCurBlock);
	xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pCurBlock->m_pBuildArrowObject, 1, BB_DEPTH_CURPIECEBUILDARROW, 0, 0);
	xRenderable2DQuadSetPosition(m_pCurBlock->m_pBuildArrowObject, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY);
	xRenderable2DQuadSetSize(m_pCurBlock->m_pBuildArrowObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(m_pCurBlock->m_pBuildArrowObject);

	m_dwSongLength = g_xGame.m_xSong.GetLength();

	if(!m_pBeatMarkers[0])
	{
		for(i = 0; i < BB_BEATMARKERS_MAX; i++)
		{
			m_pBeatMarkers[i] = xSceneGraphCreateQuad(BB_IMAGE_BEATMARKER, 0);
			xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pBeatMarkers[i], 1, BB_DEPTH_BEATMARKER, 0, 1);
			xRenderable2DQuadUpdate(m_pBeatMarkers[i]);
		}

		m_pBeatMarkerDimmer = xSceneGraphCreateQuad(BB_IMAGE_BEATMARKER_DIMMER, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pBeatMarkerDimmer, 1, BB_DEPTH_BEATMARKER_DIMMER, 0, 1);
		xRenderable2DQuadSetPosition(m_pBeatMarkerDimmer, BB_BEATMARKERDIMMERX, BB_BEATMARKERDIMMERY);
		xRenderable2DQuadUpdate(m_pBeatMarkerDimmer);
		xRenderable2DQuadUpdateAlpha(m_pBeatMarkerDimmer, 0.5f);

		m_pBeatMarkerHitzone = xSceneGraphCreateQuad(BB_IMAGE_BEATMARKER_HITZONE, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pBeatMarkerHitzone, 1, BB_DEPTH_BEATMARKER_HITZONE, 0, 1);
		xRenderable2DQuadSetPosition(m_pBeatMarkerHitzone, BB_BEATMARKERHITZONEX, BB_BEATMARKERHITZONEY);
		xRenderable2DQuadUpdate(m_pBeatMarkerHitzone);
	}

	if(!bNoBlocks)
	{
		m_iUnfilled = 1;

		i = m_iNumBlocks - 1;
		while(i > 0)
		{
			triedleft = false;
			triedright = false;
			triedup = false;
			trieddown = false;
			newcolor = rand() % g_xGameOptions.m_iNumColors;

			while(1)
			{
				switch(rand() % 4)
				{
					case UP:
					{
						if(triedup)
							continue;
						else
							triedup = true;
						
						if(!AddBlockUp(newcolor))
							continue;

						if(m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY - fFirstBlockY < fLowY)
							fLowY = m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY - fFirstBlockY;

						break;
					}

					case DOWN:
					{
						if(trieddown)
							continue;
						else
							trieddown = true;

						if(!AddBlockDown(newcolor))
							continue;

						if(m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY - fFirstBlockY > fHighY)
							fHighY = m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY - fFirstBlockY;

						break;
					}

					case LEFT:
					{
						if(triedleft)
							continue;
						else
							triedleft = true;

						if(!AddBlockLeft(newcolor))
							continue;

						if(m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX - fFirstBlockX < fLowX)
							fLowX = m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX - fFirstBlockX;

						break;
					}

					case RIGHT:
					{
						if(triedright)
							continue;
						else
							triedright = true;

						if(!AddBlockRight(newcolor))
							continue;

						if(m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX - fFirstBlockX > fHighX)
							fHighX = m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX - fFirstBlockX;

						break;
					}
				}

				break;
			}

			m_iUnfilled++;

			xRenderable2DQuadUpdateColor(m_pCurBlock->m_pBlockObject, &g_colUnfilledBlockColors[newcolor]);

			i--;
		}
	}

	if(bCenterAroundPoint)
	{
		// Calculate the Offsets to Center the Piece
		fLowX += ((fHighX - fLowX) + BB_BLOCKOFFSET) / 2.0;
		fLowY += ((fHighY - fLowY) + BB_BLOCKOFFSET) / 2.0;

		for(m_pCurBlock = m_pBlockList; m_pCurBlock; m_pCurBlock = m_pCurBlock->m_pNext)
		{
			DEBUGCURBLOCK("Init2 set CurBlock to %x\n", m_pCurBlock);
			m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX -= fLowX;
			m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY -= fLowY;

			m_pCurBlock->m_pBlockOverlayObject->m_xRenderable2D.m_fX -= fLowX;
			m_pCurBlock->m_pBlockOverlayObject->m_xRenderable2D.m_fY -= fLowY;

			m_pCurBlock->m_pBuildArrowObject->m_xRenderable2D.m_fX -= fLowX;
			m_pCurBlock->m_pBuildArrowObject->m_xRenderable2D.m_fY -= fLowY;			

			xRenderable2DQuadUpdate(m_pCurBlock->m_pBlockObject);
			xRenderable2DQuadUpdate(m_pCurBlock->m_pBlockOverlayObject);
			xRenderable2DQuadUpdate(m_pCurBlock->m_pBuildArrowObject);
		}
	}

	for(debugblock = m_pBlockList; debugblock; debugblock = debugblock->m_pNext)
		DEBUGCURBLOCK("Init Piece List: %i %i %x\n", debugblock->m_iRowID, debugblock->m_iColumnID, debugblock);

	DEBUGCURBLOCK("Init3 setting CurBlock to %x\n", m_pBlockList);
	m_pCurBlock = m_pBlockList;

	if(!g_xGameOptions.m_bTutorial)
	{
		xRenderable2DQuadSetPosition(m_pCurBlock->m_pBlockObject, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX - (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) / 2.0f, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY - (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) / 2.0f);
		xRenderable2DQuadSetSize(m_pCurBlock->m_pBlockObject, BB_LARGEBLOCKOFFSET, BB_LARGEBLOCKOFFSET);
		xRenderable2DQuadUpdate(m_pCurBlock->m_pBlockObject);

		xRenderable2DQuadSetPosition(m_pCurBlock->m_pBlockOverlayObject, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY);
		xRenderable2DQuadSetSize(m_pCurBlock->m_pBlockOverlayObject, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fWidth, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fHeight);
		xRenderable2DQuadUpdate(m_pCurBlock->m_pBlockOverlayObject);

		xRenderable2DQuadSetPosition(m_pCurBlock->m_pBuildArrowObject, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY);
		xRenderable2DQuadSetSize(m_pCurBlock->m_pBuildArrowObject, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fWidth, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fHeight);
		xRenderable2DQuadUpdate(m_pCurBlock->m_pBuildArrowObject);
	}

	xPieceUpdateBeatMarkerLocation(this);
}

void xPiece::PreUpdate(DWORD dwTime)
{
	m_dwCurTime = g_xGame.m_xSong.GetTime();

	//DEBUGLOG("%i %u %u\n", m_pBeatManager->m_iCurBeatIndex, m_dwCurTime, m_pBeatManager->m_dwBeatTimes[m_pBeatManager->m_iCurBeatIndex]);

	if(m_dwCurTime > m_pBeatManager->m_dwBeatTimes[m_pBeatManager->m_iCurBeatIndex] + g_xGameOptions.m_iHitTime)
	{
		if(!m_pBeatManager->m_bAddedBlockAlready)
			m_bSkippedBeat = true;

		m_pBeatManager->m_bAddedBlockAlready = false;

		m_pBeatManager->m_iCurBeatIndex++;

		if(g_xGameOptions.m_bLoopSong && m_pBeatManager->m_iCurBeatIndex == m_pBeatManager->m_iNumBeats)
			m_pBeatManager->m_iCurBeatIndex = 0;

		//DEBUGLOG("NextBeat: %i %i %u %u %i\n", m_pBeatManager->m_iCurBeatIndex, m_pBeatManager->m_iNumBeats, m_dwCurTime, m_pBeatManager->m_dwBeatTimes[m_pBeatManager->m_iCurBeatIndex], g_xGameOptions.m_iHitTime);
	}
}

void xPiece::Update(double dTime, bool bIsActivePiece)
{
	xColor blockcolor;
	const xColor *ptrcolor;
	xBlock *curblock;
	int beatmarkerindex, oldframe, i;
	float beattime, prevblocksize, curblocksize;

	if(g_xGame.m_xPlayer.m_bDead || g_xGame.m_xPlayer.m_bWonGame)
		m_dwCurTime = 0;

	m_bSkippedBeat = false;

	if(bIsActivePiece)
	{
		if(!m_bTouchedDown && m_iUnfilled > 0)
		{
			beatmarkerindex = 0;

			if(m_dwCurTime > 200)
			{
				ptrcolor = &g_colUnfilledBlockColors[m_pCurBlock->m_iUnfilledColorIndex];
				xRenderable2DQuadUpdateColor(m_pCurBlock->m_pBlockObject, xColorInit(&blockcolor,
																					 ptrcolor->R + (1.0f - ptrcolor->R) * g_xGame.m_fBeatPercentage,
																					 ptrcolor->G + (1.0f - ptrcolor->G) * g_xGame.m_fBeatPercentage,
																					 ptrcolor->B + (1.0f - ptrcolor->B) * g_xGame.m_fBeatPercentage,
																					 1.0f));
			}

			for(i = m_pBeatManager->m_iCurBeatIndex; beatmarkerindex < BB_BEATMARKERS_MAX && i < m_pBeatManager->m_iNumBeats && m_pBeatManager->m_dwBeatTimes[i] < m_dwCurTime + g_xGameOptions.m_iBeatMarkerShowTime; i++)
			{
				if(m_pBeatManager->m_dwBeatTimes[i] - m_dwCurTime <= 0 || m_pBeatManager->m_dwBeatTimes[i] - m_dwCurTime >= g_xGameOptions.m_iBeatMarkerShowTime)
				{
					xPieceUpdateBeatMarker(m_pBeatMarkers[beatmarkerindex], 0.0f, (int)m_pBeatManager->m_dwBeatTimes[i] - (int)m_dwCurTime, BB_BEATMARKERX, BB_BEATMARKERY);
					beatmarkerindex++;
					continue;
				}

				beattime = 1.0 - (((float)(g_xGameOptions.m_iBeatMarkerShowTime - (m_pBeatManager->m_dwBeatTimes[i] - m_dwCurTime))) / ((float)g_xGameOptions.m_iBeatMarkerShowTime));
				beattime *= beattime;

				//DEBUGLOG("beatmarker%i - beattime:%1.4f\t\t", beatmarkerindex, beattime);

				xPieceUpdateBeatMarker(m_pBeatMarkers[beatmarkerindex], beattime, m_pBeatManager->m_dwBeatTimes[i] - m_dwCurTime, BB_BEATMARKERX, BB_BEATMARKERY);

				beatmarkerindex++;
			}

			if((m_pBeatManager->m_iCurBeatIndex == 0 || i == m_pBeatManager->m_iNumBeats) && beatmarkerindex < BB_BEATMARKERS_MAX)
			{
				int showtime = g_xGameOptions.m_iBeatMarkerShowTime - ((int)(m_dwSongLength - m_dwCurTime));
				if(showtime > 0)
				{
					DWORD dwshowtime = showtime;
					for(i = 0; m_pBeatManager->m_dwBeatTimes[i] < dwshowtime; i++)
					{
						beattime = ((float)(showtime - m_pBeatManager->m_dwBeatTimes[i])) / ((float)g_xGameOptions.m_iBeatMarkerShowTime);
						
						xPieceUpdateBeatMarker(m_pBeatMarkers[beatmarkerindex], beattime, (int)m_pBeatManager->m_dwBeatTimes[i] - (int)(m_dwSongLength - m_dwCurTime), BB_BEATMARKERX, BB_BEATMARKERY);

						beatmarkerindex++;
					}
				}
			}

			for(i = beatmarkerindex; i < BB_BEATMARKERS_MAX; i++)
			{
				if(m_pBeatMarkers[i]->m_xRenderable2D.m_bVisible)
					xRenderable2DQuadUpdateVisibility(m_pBeatMarkers[i], false);
			}
		}
		else if(m_pBeatMarkers[0]->m_xRenderable2D.m_bVisible)
		{
			for(i = 0; i < BB_BEATMARKERS_MAX; i++)
				xRenderable2DQuadUpdateVisibility(m_pBeatMarkers[i], false);
		}
	}

	if(m_pPrevBlock && (!m_bDoneAnimatingBlocks || m_fBuildAnimationTime < BB_PIECE_ANIMBUILDSUCCESSFUL))
	{
		m_bDoneAnimatingBlocks = 0;

		blockcolor.A = 1.0f;

		DEBUGBLOCKANIM("Block Anim Running %f %f\n", m_fBuildAnimationTime, dTime);

		if(m_bLastBuildSuccessful)
		{
			if(m_fBuildAnimationTime < BB_PIECE_ANIMBUILDGROW)
			{
				m_fBuildAnimationTime += dTime;

				curblocksize = BB_BLOCKOFFSET;

				if(m_fBuildAnimationTime >= BB_PIECE_ANIMBUILDGROW)
				{
					DEBUGBLOCKANIM("Block Anim Done Growing\n");
					prevblocksize = BB_GROWNBLOCKOFFSET;
					xColorCopy(&blockcolor, &g_colFilledBlockColors[m_pPrevBlock->m_iColorIndex]);
				}
				else
				{
					DEBUGBLOCKANIM("Block Anim Still Growing - %f\n", m_fBuildAnimationTime / BB_PIECE_ANIMBUILDGROW);
					prevblocksize = BB_LARGEBLOCKOFFSET + (BB_GROWNBLOCKOFFSET - BB_LARGEBLOCKOFFSET) * (m_fBuildAnimationTime / BB_PIECE_ANIMBUILDGROW);
					blockcolor.R = m_xPrevBlockOldColor.R + (g_colFilledBlockColors[m_pPrevBlock->m_iColorIndex].R - m_xPrevBlockOldColor.R) * (m_fBuildAnimationTime / BB_PIECE_ANIMBUILDGROW);
					blockcolor.G = m_xPrevBlockOldColor.G + (g_colFilledBlockColors[m_pPrevBlock->m_iColorIndex].G - m_xPrevBlockOldColor.G) * (m_fBuildAnimationTime / BB_PIECE_ANIMBUILDGROW);
					blockcolor.B = m_xPrevBlockOldColor.B + (g_colFilledBlockColors[m_pPrevBlock->m_iColorIndex].B - m_xPrevBlockOldColor.B) * (m_fBuildAnimationTime / BB_PIECE_ANIMBUILDGROW);
				}
			}
			else
			{
				if((m_pPrevBlock->m_pBlockObject->m_xRenderable2D.m_fDepth > BB_DEPTH_CURPIECEBLOCK - 0.01f || (m_pCurBlock != m_pPrevBlock && m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fDepth < BB_DEPTH_PIECEBLOCKS + 0.01f)) &&
					(m_fBuildAnimationTime - BB_PIECE_ANIMBUILDGROW) + dTime >= (BB_PIECE_ANIMBUILDSUCCESSFUL - BB_PIECE_ANIMBUILDGROW) / 2.0)
				{
					xRenderable2DQuadUpdateDepth(m_pPrevBlock->m_pBlockObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_PIECEBLOCKS);
					xRenderable2DQuadUpdateDepth(m_pPrevBlock->m_pBlockOverlayObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_PIECEBLOCKOVERLAY);
					xRenderable2DQuadUpdateDepth(m_pPrevBlock->m_pBuildArrowObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_PIECEBUILDARROW);

					DEBUGCURBLOCK("Checking update depth %x %x\n", m_pCurBlock, m_pPrevBlock);
					if(m_pCurBlock != m_pPrevBlock)
					{
						xRenderable2DQuadUpdateDepth(m_pCurBlock->m_pBlockObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_CURPIECEBLOCK);
						xRenderable2DQuadUpdateDepth(m_pCurBlock->m_pBlockOverlayObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_CURPIECEBLOCKOVERLAY);
						xRenderable2DQuadUpdateDepth(m_pCurBlock->m_pBuildArrowObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_CURPIECEBUILDARROW);
					}
				}

				m_fBuildAnimationTime += dTime;

				xColorCopy(&blockcolor, &g_colFilledBlockColors[m_pPrevBlock->m_iColorIndex]);

				if(m_fBuildAnimationTime >= BB_PIECE_ANIMBUILDSUCCESSFUL)
				{
					DEBUGBLOCKANIM("Block Anim Done Shrinking\n");
					m_bDoneAnimatingBlocks = 1;
					curblocksize = BB_LARGEBLOCKOFFSET;
					prevblocksize = BB_BLOCKOFFSET;
				}
				else
				{
					DEBUGBLOCKANIM("Block Anim Still Shrinking %f\n", (m_fBuildAnimationTime - BB_PIECE_ANIMBUILDGROW) / (BB_PIECE_ANIMBUILDSUCCESSFUL - BB_PIECE_ANIMBUILDGROW));
					curblocksize = BB_BLOCKOFFSET + (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) * ((m_fBuildAnimationTime - BB_PIECE_ANIMBUILDGROW) / (BB_PIECE_ANIMBUILDSUCCESSFUL - BB_PIECE_ANIMBUILDGROW));
					prevblocksize = BB_GROWNBLOCKOFFSET - (BB_GROWNBLOCKOFFSET - BB_BLOCKOFFSET) * ((m_fBuildAnimationTime - BB_PIECE_ANIMBUILDGROW) / (BB_PIECE_ANIMBUILDSUCCESSFUL - BB_PIECE_ANIMBUILDGROW));
				}
			}
		}
		else
		{
			if(m_fBuildAnimationTime < BB_PIECE_ANIMBUILDFAILED / 2.0 && m_fBuildAnimationTime + dTime >= BB_PIECE_ANIMBUILDFAILED / 2.0)
			{
				xRenderable2DQuadUpdateDepth(m_pPrevBlock->m_pBlockObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_PIECEBLOCKS);
				xRenderable2DQuadUpdateDepth(m_pPrevBlock->m_pBlockOverlayObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_PIECEBLOCKOVERLAY);
				xRenderable2DQuadUpdateDepth(m_pPrevBlock->m_pBuildArrowObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_PIECEBUILDARROW);

				DEBUGCURBLOCK("Checking update depth %x %x\n", m_pCurBlock, m_pPrevBlock);
				if(m_pCurBlock != m_pPrevBlock)
				{
					xRenderable2DQuadUpdateDepth(m_pCurBlock->m_pBlockObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_CURPIECEBLOCK);
					xRenderable2DQuadUpdateDepth(m_pCurBlock->m_pBlockOverlayObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_CURPIECEBLOCKOVERLAY);
					xRenderable2DQuadUpdateDepth(m_pCurBlock->m_pBuildArrowObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_CURPIECEBUILDARROW);
				}
			}

			m_fBuildAnimationTime += dTime;

			if(m_fBuildAnimationTime >= BB_PIECE_ANIMBUILDFAILED)
			{
				curblocksize = BB_LARGEBLOCKOFFSET;
				prevblocksize = BB_BLOCKOFFSET;
				xColorCopy(&blockcolor, &g_colFilledBlockColors[m_pPrevBlock->m_iColorIndex]);

				m_bDoneAnimatingBlocks = 1;
				m_fBuildAnimationTime = BB_PIECE_ANIMBUILDSUCCESSFUL + 0.1f;
			}
			else
			{
				curblocksize = BB_BLOCKOFFSET + (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) * (m_fBuildAnimationTime / BB_PIECE_ANIMBUILDFAILED);
				prevblocksize = BB_LARGEBLOCKOFFSET - (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) * (m_fBuildAnimationTime / BB_PIECE_ANIMBUILDFAILED);
				blockcolor.R = m_xPrevBlockOldColor.R + (g_colFilledBlockColors[m_pPrevBlock->m_iColorIndex].R - m_xPrevBlockOldColor.R) * (m_fBuildAnimationTime / BB_PIECE_ANIMBUILDFAILED);
				blockcolor.G = m_xPrevBlockOldColor.G + (g_colFilledBlockColors[m_pPrevBlock->m_iColorIndex].G - m_xPrevBlockOldColor.G) * (m_fBuildAnimationTime / BB_PIECE_ANIMBUILDFAILED);
				blockcolor.B = m_xPrevBlockOldColor.B + (g_colFilledBlockColors[m_pPrevBlock->m_iColorIndex].B - m_xPrevBlockOldColor.B) * (m_fBuildAnimationTime / BB_PIECE_ANIMBUILDFAILED);
			}
		}

		xRenderable2DQuadSetPosition(m_pPrevBlock->m_pBlockObject, BB_PIECESTARTX - BB_BLOCKOFFSET * m_pPrevBlock->m_iColumnID - (prevblocksize - BB_BLOCKOFFSET) / 2.0, BB_PIECESTARTY - BB_BLOCKOFFSET * m_pPrevBlock->m_iRowID - (prevblocksize - BB_BLOCKOFFSET) / 2.0);
		xRenderable2DQuadSetSize(m_pPrevBlock->m_pBlockObject, prevblocksize, prevblocksize);
		xRenderable2DQuadUpdateColor(m_pPrevBlock->m_pBlockObject, &blockcolor);
		xRenderable2DQuadUpdate(m_pPrevBlock->m_pBlockObject);

		xRenderable2DQuadSetPosition(m_pPrevBlock->m_pBlockOverlayObject, m_pPrevBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pPrevBlock->m_pBlockObject->m_xRenderable2D.m_fY);
		xRenderable2DQuadSetSize(m_pPrevBlock->m_pBlockOverlayObject, prevblocksize, prevblocksize);
		xRenderable2DQuadUpdate(m_pPrevBlock->m_pBlockOverlayObject);

		xRenderable2DQuadSetPosition(m_pPrevBlock->m_pBuildArrowObject, m_pPrevBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pPrevBlock->m_pBlockObject->m_xRenderable2D.m_fY);
		xRenderable2DQuadSetSize(m_pPrevBlock->m_pBuildArrowObject, prevblocksize, prevblocksize);
		xRenderable2DQuadUpdate(m_pPrevBlock->m_pBuildArrowObject);

		DEBUGCURBLOCK("Update check CurBlock %x against PrevBlock %x\n", m_pCurBlock, m_pPrevBlock);
		if(m_pCurBlock != m_pPrevBlock)
		{
			xRenderable2DQuadSetPosition(m_pCurBlock->m_pBlockObject, BB_PIECESTARTX - BB_BLOCKOFFSET * m_pCurBlock->m_iColumnID - (curblocksize - BB_BLOCKOFFSET) / 2.0, BB_PIECESTARTY - BB_BLOCKOFFSET * m_pCurBlock->m_iRowID - (curblocksize - BB_BLOCKOFFSET) / 2.0);
			xRenderable2DQuadSetSize(m_pCurBlock->m_pBlockObject, curblocksize, curblocksize);
			xRenderable2DQuadUpdate(m_pCurBlock->m_pBlockObject);

			xRenderable2DQuadSetPosition(m_pCurBlock->m_pBlockOverlayObject, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY);
			xRenderable2DQuadSetSize(m_pCurBlock->m_pBlockOverlayObject, curblocksize, curblocksize);
			xRenderable2DQuadUpdate(m_pCurBlock->m_pBlockOverlayObject);

			xRenderable2DQuadSetPosition(m_pCurBlock->m_pBuildArrowObject, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY);
			xRenderable2DQuadSetSize(m_pCurBlock->m_pBuildArrowObject, curblocksize, curblocksize);
			xRenderable2DQuadUpdate(m_pCurBlock->m_pBuildArrowObject);
		}
	}

	if(m_fScaleInTime != 0.0f)
	{
		if(m_fScaleInTime > 0.0f)
		{
			m_fScaleInTime -= dTime;

			curblocksize = (1.0f - (m_fScaleInTime / BB_PIECE_NEWBLOCKSCALEINTIME)) * BB_LARGEBLOCKOFFSET;

			if(m_fScaleInTime <= 0.0f)
			{
				if(m_pScaleInBlock != m_pCurBlock)
					m_fScaleInTime = BB_PIECE_NEWBLOCKSCALEDOWNTIME;
				else
					m_fScaleInTime = 0.0f;

				curblocksize = BB_LARGEBLOCKOFFSET;
			}
		}
		else
		{
			m_fScaleInTime += dTime;

			if(m_fScaleInTime > 0.0f)
				m_fScaleInTime = 0.0f;

			curblocksize = BB_BLOCKOFFSET + (m_fScaleInTime / BB_PIECE_NEWBLOCKSCALEDOWNTIME) * (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET);
		}

		xRenderable2DQuadSetPosition(m_pScaleInBlock->m_pBlockObject, m_fScaleInBlockAnimCachedX - (curblocksize - BB_BLOCKOFFSET) / 2.0, m_fScaleInBlockAnimCachedY - (curblocksize - BB_BLOCKOFFSET) / 2.0);
		xRenderable2DQuadSetSize(m_pScaleInBlock->m_pBlockObject, curblocksize, curblocksize);
		xRenderable2DQuadUpdate(m_pScaleInBlock->m_pBlockObject);

		xRenderable2DQuadSetPosition(m_pScaleInBlock->m_pBlockOverlayObject, m_pScaleInBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pScaleInBlock->m_pBlockObject->m_xRenderable2D.m_fY);
		xRenderable2DQuadSetSize(m_pScaleInBlock->m_pBlockOverlayObject, curblocksize, curblocksize);
		xRenderable2DQuadUpdate(m_pScaleInBlock->m_pBlockOverlayObject);

		xRenderable2DQuadSetPosition(m_pScaleInBlock->m_pBuildArrowObject, m_pScaleInBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pScaleInBlock->m_pBlockObject->m_xRenderable2D.m_fY);
		xRenderable2DQuadSetSize(m_pScaleInBlock->m_pBuildArrowObject, curblocksize, curblocksize);
		xRenderable2DQuadUpdate(m_pScaleInBlock->m_pBuildArrowObject);
	}

	if(m_pBlockAnimation->m_xRenderable2D.m_bVisible)
	{
		oldframe = m_pBlockAnimation->m_iCurrentTexture;

		xRenderable2DQuadUpdateAnimation(m_pBlockAnimation, dTime);

		if(m_pBlockAnimation->m_iCurrentTexture < oldframe)
			xRenderable2DQuadUpdateVisibility(m_pBlockAnimation, 0);
	}
	else if(!m_bDropping)
	{
		curblock = m_pBlockList;
		while(curblock && curblock->m_iColorIndex >= 0 && curblock->m_pBlockObject->m_xRenderable2D.m_fWidth <= BB_BLOCKOFFSET + 0.01)
		{
			if(rand() < RAND_MAX / 64)
			{
				m_pBlockAnimation->m_fAnimationTime = 0;
				m_pBlockAnimation->m_iCurrentTexture = 0;

				xRenderable2DQuadSetPosition(m_pBlockAnimation, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
				xRenderable2DQuadUpdateVisibility(m_pBlockAnimation, true);
				xRenderable2DQuadUpdate(m_pBlockAnimation);
				break;
			}

			curblock = curblock->m_pNext;
		}
	}

#if defined _DEBUG && 0
	char temp[20];
	sprintf(temp, "%i, %i", m_iCurFieldX, m_iCurFieldY);
	g_xDebugFont.Render(temp, 0, 0);
	if(m_pBlockList)
	{
		sprintf(temp, "%i, %i", m_pBlockList->m_iColumnID, m_pBlockList->m_iRowID);
		g_xDebugFont.Render(temp, 0, 30);
		//sprintf(temp, "%i, %i", ((int)((m_pBlockList->m_pBlockObject->m_fX - 104.0) / 25.0)), ((int)((m_pBlockList->m_pBlockObject->m_fY - 110.0) / 25.0)));
		//g_xDebugFont.Render(temp, 0, 60);
		//sprintf(temp, "%i/%i", (int)(((float)m_pBlockList->m_pBlockObject->m_iNumFrames) * dTime * ((float)g_xGameOptions.m_fBPM) / 60.0), m_pBlockList->m_pBlockObject->m_iNumFrames);
		//g_xDebugFont.Render(temp, 0, 90);
	}
#endif
}

void xPiece::RefreshAllRenderables()
{
	xBlock *curblock = m_pBlockList;
	int i;

	while(curblock)
	{
		if(curblock->m_pBlockObject)
		{
			xRenderable2DQuadUpdateTexture(curblock->m_pBlockObject, curblock->m_pBlockObject->m_pTextures[0]);
			xRenderable2DQuadUpdate(curblock->m_pBlockObject);

			xRenderable2DQuadUpdateTexture(curblock->m_pBlockOverlayObject, curblock->m_pBlockOverlayObject->m_pTextures[0]);
			xRenderable2DQuadUpdate(curblock->m_pBlockOverlayObject);

			xRenderable2DQuadUpdateTexture(curblock->m_pBuildArrowObject, curblock->m_pBuildArrowObject->m_pTextures[0]);
			xRenderable2DQuadUpdate(curblock->m_pBuildArrowObject);
		}

		curblock = curblock->m_pNext;
	}

	if(m_pBlockAnimation)
	{
		xRenderable2DQuadUpdateTexture(m_pBlockAnimation, m_pBlockAnimation->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pBlockAnimation);
	}

	if(m_pBeatMarkers[0])
	{
		for(i = 0; i < BB_BEATMARKERS_MAX; i++)
		{
			xRenderable2DQuadUpdateTexture(m_pBeatMarkers[i], m_pBeatMarkers[i]->m_pTextures[0]);
			xRenderable2DQuadUpdate(m_pBeatMarkers[i]);
		}

		if(m_pBeatMarkerDimmer)
		{
			xRenderable2DQuadUpdateTexture(m_pBeatMarkerDimmer, m_pBeatMarkerDimmer->m_pTextures[0]);
			xRenderable2DQuadUpdate(m_pBeatMarkerDimmer);
		}

		if(m_pBeatMarkerHitzone)
		{
			xRenderable2DQuadUpdateTexture(m_pBeatMarkerHitzone, m_pBeatMarkerHitzone->m_pTextures[0]);
			xRenderable2DQuadUpdate(m_pBeatMarkerHitzone);
		}
	}
}

void xPiece::Delete(bool bDeleteBlocks, bool bDeleteBlockObjects, bool bDeleteCursor)
{
	if(m_pBlockList)
	{
		xBlock *curblock = m_pBlockList;
		while(curblock->m_pNext)
		{
			curblock = curblock->m_pNext;

			if(bDeleteBlockObjects)
			{
				xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, curblock->m_pPrev->m_pBlockObject, false);
				xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, curblock->m_pPrev->m_pBlockOverlayObject, false);
				xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, curblock->m_pPrev->m_pBuildArrowObject, false);
			}

			if(bDeleteBlocks)
				delete curblock->m_pPrev;
		}

		if(bDeleteBlockObjects)
		{
			xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, curblock->m_pBlockObject, false);
			xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, curblock->m_pBlockOverlayObject, false);
			xSceneGraphDeleteQuad(&g_xGame.m_xSceneGraph, curblock->m_pBuildArrowObject, false);
		}

		if(bDeleteBlocks)
			delete curblock;

		m_pBlockList = 0;
		DEBUGCURBLOCK("Delete setting CurBlock and PrevBlock to NULL was %x\n", m_pCurBlock);
		m_pCurBlock = 0;
		m_pPrevBlock = 0;

		m_bDoneAnimatingBlocks	= 1;
	}
}

void xPiece::ForceUpdateBeatMarkers(DWORD dwTime)
{
	float beattime;

	beattime = 1.0f - (((float)(g_xGameOptions.m_iBeatMarkerShowTime - (m_pBeatManager->m_dwBeatTimes[m_pBeatManager->m_iCurBeatIndex] - dwTime))) / ((float)g_xGameOptions.m_iBeatMarkerShowTime));
	beattime *= beattime;

	xPieceUpdateBeatMarker(m_pBeatMarkers[0], beattime, (int)m_pBeatManager->m_dwBeatTimes[m_pBeatManager->m_iCurBeatIndex] - (int)dwTime, BB_BEATMARKERX, BB_BEATMARKERY);

	xRenderable2DQuadUpdateVisibility(m_pBeatMarkers[1], false);
}

bool xPiece::AddBlockUp(int iBlockColor)
{
	int hirow = INT_MIN;
	xBlock* prevblock;
	xBlock* curblock;
	float x, y;

	for(curblock = m_pBlockList; curblock; curblock = curblock->m_pNext)
	{
		if(curblock->m_iColumnID == m_pCurBlock->m_iColumnID && curblock->m_iRowID > hirow)
			hirow = curblock->m_iRowID;

		prevblock = curblock;
	}

	if(hirow - m_pCurBlock->m_iRowID > 1)
	{
		bool *bFilled = new bool[hirow - m_pCurBlock->m_iRowID];
		memset(bFilled, 0, hirow - m_pCurBlock->m_iRowID);
		for(xBlock* blocklist = m_pBlockList; blocklist; blocklist = blocklist->m_pNext)
		{
			if(blocklist->m_iColumnID == m_pCurBlock->m_iColumnID && blocklist->m_iRowID > m_pCurBlock->m_iRowID)
				bFilled[blocklist->m_iRowID - m_pCurBlock->m_iRowID] = true;
		}

		hirow++;

		for(int i = hirow - m_pCurBlock->m_iRowID - 2; i > 0; i--)
		{
			if(!bFilled[i])
				hirow = m_pCurBlock->m_iRowID + i;
		}
	}
	else
		hirow++;

	if(hirow > BB_PIECEMAXROWID || hirow - m_pCurBlock->m_iRowID > 1 || hirow - m_pCurBlock->m_iRowID < -1)
		return false;

	prevblock->m_pNext				= new xBlock;
	prevblock->m_pNext->m_pPrev		= prevblock;
	curblock						= prevblock->m_pNext;
	curblock->m_iColumnID			= m_pCurBlock->m_iColumnID;
	curblock->m_iRowID				= hirow;
	curblock->m_iType				= NOT_SPECIAL;
	curblock->m_iUnfilledColorIndex	= iBlockColor;

	if(!curblock->m_pBlockObject)
	{
		curblock->m_pBlockObject = xSceneGraphCreateQuad(m_pBlockTextures[4]);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, curblock->m_pBlockObject, 1, BB_DEPTH_PIECEBLOCKS, 0, 1);

		curblock->m_pBlockOverlayObject = xSceneGraphCreateQuad(BB_IMAGE_PIECEBLOCKOVERLAY, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, curblock->m_pBlockOverlayObject, 1, BB_DEPTH_PIECEBLOCKOVERLAY, 0, 0);

		xPieceCreateBuildArrowObject(curblock);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, curblock->m_pBuildArrowObject, 1, BB_DEPTH_PIECEBUILDARROW, 0, 0);
	}

	x = m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX + (m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fWidth - BB_BLOCKOFFSET) / 2.0f;
	y = m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY + (m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fHeight - BB_BLOCKOFFSET) / 2.0f;

	xRenderable2DQuadSetPosition(curblock->m_pBlockObject, x, y + ((m_pCurBlock->m_iRowID - curblock->m_iRowID) * BB_BLOCKOFFSET));
	xRenderable2DQuadSetSize(curblock->m_pBlockObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(curblock->m_pBlockObject);

	xRenderable2DQuadSetPosition(curblock->m_pBlockOverlayObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
	xRenderable2DQuadSetSize(curblock->m_pBlockOverlayObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(curblock->m_pBlockOverlayObject);

	xRenderable2DQuadSetPosition(curblock->m_pBuildArrowObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
	xRenderable2DQuadSetSize(curblock->m_pBuildArrowObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(curblock->m_pBuildArrowObject);

	DEBUGCURBLOCK("AddBlockUp setting CurBlock to %x\n", curblock);
	m_pCurBlock = curblock;

	xPieceUpdateBeatMarkerLocation(this);

	return true;
}

bool xPiece::AddBlockDown(int iBlockColor)
{
	int lowrow = INT_MAX;
	xBlock* prevblock;
	xBlock* curblock;
	float x, y;

	for(curblock = m_pBlockList; curblock; curblock = curblock->m_pNext)
	{
		if(curblock->m_iColumnID == m_pCurBlock->m_iColumnID && curblock->m_iRowID < lowrow)
			lowrow = curblock->m_iRowID;

		prevblock = curblock;
	}

	if(m_pCurBlock->m_iRowID - lowrow > 1)
	{
		bool *bFilled = new bool[m_pCurBlock->m_iRowID - lowrow];
		memset(bFilled, 0, m_pCurBlock->m_iRowID - lowrow);
		for(xBlock* blocklist = m_pBlockList; blocklist; blocklist = blocklist->m_pNext)
		{
			if(blocklist->m_iColumnID == m_pCurBlock->m_iColumnID && blocklist->m_iRowID < m_pCurBlock->m_iRowID)
				bFilled[m_pCurBlock->m_iRowID - blocklist->m_iRowID] = true;
		}

		lowrow--;

		for(int i = m_pCurBlock->m_iRowID - lowrow - 2; i > 0; i--)
		{
			if(!bFilled[i])
				lowrow = m_pCurBlock->m_iRowID - i;
		}
	}
	else
		lowrow--;

	if(lowrow < BB_PIECEMINROWID || lowrow - m_pCurBlock->m_iRowID > 1 || lowrow - m_pCurBlock->m_iRowID < -1)
		return false;

	prevblock->m_pNext				= new xBlock;
	prevblock->m_pNext->m_pPrev		= prevblock;
	curblock						= prevblock->m_pNext;
	curblock->m_iColumnID			= m_pCurBlock->m_iColumnID;
	curblock->m_iRowID				= lowrow;
	curblock->m_iType				= NOT_SPECIAL;
	curblock->m_iUnfilledColorIndex	= iBlockColor;

	if(!curblock->m_pBlockObject)
	{
		curblock->m_pBlockObject = xSceneGraphCreateQuad(m_pBlockTextures[4]);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, curblock->m_pBlockObject, 1, BB_DEPTH_PIECEBLOCKS, 0, 1);

		curblock->m_pBlockOverlayObject = xSceneGraphCreateQuad(BB_IMAGE_PIECEBLOCKOVERLAY, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, curblock->m_pBlockOverlayObject, 1, BB_DEPTH_PIECEBLOCKOVERLAY, 0, 0);

		xPieceCreateBuildArrowObject(curblock);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, curblock->m_pBuildArrowObject, 1, BB_DEPTH_PIECEBUILDARROW, 0, 0);
	}

	x = m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX + (m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fWidth - BB_BLOCKOFFSET) / 2.0f;
	y = m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY + (m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fHeight - BB_BLOCKOFFSET) / 2.0f;

	xRenderable2DQuadSetPosition(curblock->m_pBlockObject, x, y + ((m_pCurBlock->m_iRowID - curblock->m_iRowID) * BB_BLOCKOFFSET));
	xRenderable2DQuadSetSize(curblock->m_pBlockObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(curblock->m_pBlockObject);

	xRenderable2DQuadSetPosition(curblock->m_pBlockOverlayObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
	xRenderable2DQuadSetSize(curblock->m_pBlockOverlayObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(curblock->m_pBlockOverlayObject);

	xRenderable2DQuadSetPosition(curblock->m_pBuildArrowObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
	xRenderable2DQuadSetSize(curblock->m_pBuildArrowObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(curblock->m_pBuildArrowObject);

	DEBUGCURBLOCK("AddBlockDown setting CurBlock to %x\n", curblock);
	m_pCurBlock = curblock;

	xPieceUpdateBeatMarkerLocation(this);

	return true;
}

bool xPiece::AddBlockLeft(int iBlockColor)
{
	int hicol = INT_MIN;
	xBlock* prevblock;
	xBlock* curblock;
	float x, y;

	for(curblock = m_pBlockList; curblock; curblock = curblock->m_pNext)
	{
		if(curblock->m_iRowID == m_pCurBlock->m_iRowID && curblock->m_iColumnID > hicol)
			hicol = curblock->m_iColumnID;

		prevblock = curblock;
	}

	if(hicol - m_pCurBlock->m_iColumnID > 1)
	{
		bool *bFilled = new bool[hicol - m_pCurBlock->m_iColumnID];
		memset(bFilled, 0, hicol - m_pCurBlock->m_iColumnID);
		for(xBlock* blocklist = m_pBlockList; blocklist; blocklist = blocklist->m_pNext)
		{
			if(blocklist->m_iRowID == m_pCurBlock->m_iRowID && blocklist->m_iColumnID > m_pCurBlock->m_iColumnID)
				bFilled[blocklist->m_iColumnID - m_pCurBlock->m_iColumnID] = true;
		}

		hicol++;

		for(int i = hicol - m_pCurBlock->m_iColumnID - 2; i > 0; i--)
		{
			if(!bFilled[i])
				hicol = m_pCurBlock->m_iColumnID + i;
		}
	}
	else
		hicol++;

	if(hicol > BB_PIECEMAXCOLUMNID || hicol - m_pCurBlock->m_iColumnID > 1 || hicol - m_pCurBlock->m_iColumnID < -1)
		return false;

	prevblock->m_pNext				= new xBlock;
	prevblock->m_pNext->m_pPrev		= prevblock;
	curblock						= prevblock->m_pNext;
	curblock->m_iColumnID			= hicol;
	curblock->m_iRowID				= m_pCurBlock->m_iRowID;
	curblock->m_iType				= NOT_SPECIAL;
	curblock->m_iUnfilledColorIndex	= iBlockColor;

	if(!curblock->m_pBlockObject)
	{
		curblock->m_pBlockObject = xSceneGraphCreateQuad(m_pBlockTextures[4]);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, curblock->m_pBlockObject, 1, BB_DEPTH_PIECEBLOCKS, 0, 1);

		curblock->m_pBlockOverlayObject = xSceneGraphCreateQuad(BB_IMAGE_PIECEBLOCKOVERLAY, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, curblock->m_pBlockOverlayObject, 1, BB_DEPTH_PIECEBLOCKOVERLAY, 0, 0);

		xPieceCreateBuildArrowObject(curblock);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, curblock->m_pBuildArrowObject, 1, BB_DEPTH_PIECEBUILDARROW, 0, 0);
	}

	x = m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX + (m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fWidth - BB_BLOCKOFFSET) / 2.0f;
	y = m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY + (m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fHeight - BB_BLOCKOFFSET) / 2.0f;

	xRenderable2DQuadSetPosition(curblock->m_pBlockObject, x + ((m_pCurBlock->m_iColumnID - curblock->m_iColumnID) * BB_BLOCKOFFSET), y);
	xRenderable2DQuadSetSize(curblock->m_pBlockObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(curblock->m_pBlockObject);

	xRenderable2DQuadSetPosition(curblock->m_pBlockOverlayObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
	xRenderable2DQuadSetSize(curblock->m_pBlockOverlayObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(curblock->m_pBlockOverlayObject);

	xRenderable2DQuadSetPosition(curblock->m_pBuildArrowObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
	xRenderable2DQuadSetSize(curblock->m_pBuildArrowObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(curblock->m_pBuildArrowObject);

	DEBUGCURBLOCK("AddBlockLeft setting CurBlock to %x\n", curblock);
	m_pCurBlock = curblock;

	xPieceUpdateBeatMarkerLocation(this);

	return true;
}

bool xPiece::AddBlockRight(int iBlockColor)
{	
	int lowcol = INT_MAX;
	xBlock* prevblock;
	xBlock* curblock;
	float x, y;

	for(curblock = m_pBlockList; curblock; curblock = curblock->m_pNext)
	{
		if(curblock->m_iRowID == m_pCurBlock->m_iRowID && curblock->m_iColumnID < lowcol)
			lowcol = curblock->m_iColumnID;

		prevblock = curblock;
	}

	if(m_pCurBlock->m_iColumnID - lowcol > 1)
	{
		bool *bFilled = new bool[m_pCurBlock->m_iColumnID - lowcol];
		memset(bFilled, 0, m_pCurBlock->m_iColumnID - lowcol);
		for(xBlock* blocklist = m_pBlockList; blocklist; blocklist = blocklist->m_pNext)
		{
			if(blocklist->m_iRowID == m_pCurBlock->m_iRowID && blocklist->m_iColumnID < m_pCurBlock->m_iColumnID)
				bFilled[m_pCurBlock->m_iColumnID - blocklist->m_iColumnID] = true;
		}

		lowcol--;

		for(int i = m_pCurBlock->m_iColumnID - lowcol - 2; i > 0; i--)
		{
			if(!bFilled[i])
				lowcol = m_pCurBlock->m_iColumnID - i;
		}
	}
	else
		lowcol--;

	if(lowcol < BB_PIECEMINCOLUMNID || lowcol - m_pCurBlock->m_iColumnID > 1 || lowcol - m_pCurBlock->m_iColumnID < -1)
		return false;

	prevblock->m_pNext				= new xBlock;
	prevblock->m_pNext->m_pPrev		= prevblock;
	curblock						= prevblock->m_pNext;
	curblock->m_iColumnID			= lowcol;
	curblock->m_iRowID				= m_pCurBlock->m_iRowID;
	curblock->m_iType				= NOT_SPECIAL;
	curblock->m_iUnfilledColorIndex	= iBlockColor;

	if(!curblock->m_pBlockObject)
	{
		curblock->m_pBlockObject = xSceneGraphCreateQuad(m_pBlockTextures[4]);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, curblock->m_pBlockObject, 1, BB_DEPTH_PIECEBLOCKS, 0, 1);

		curblock->m_pBlockOverlayObject = xSceneGraphCreateQuad(BB_IMAGE_PIECEBLOCKOVERLAY, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, curblock->m_pBlockOverlayObject, 1, BB_DEPTH_PIECEBLOCKOVERLAY, 0, 0);

		xPieceCreateBuildArrowObject(curblock);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, curblock->m_pBuildArrowObject, 1, BB_DEPTH_PIECEBUILDARROW, 0, 0);
	}

	x = m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX + (m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fWidth - BB_BLOCKOFFSET) / 2.0f;
	y = m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY + (m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fHeight - BB_BLOCKOFFSET) / 2.0f;

	xRenderable2DQuadSetPosition(curblock->m_pBlockObject, x + ((m_pCurBlock->m_iColumnID - curblock->m_iColumnID) * BB_BLOCKOFFSET), y);
	xRenderable2DQuadSetSize(curblock->m_pBlockObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(curblock->m_pBlockObject);

	xRenderable2DQuadSetPosition(curblock->m_pBlockOverlayObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
	xRenderable2DQuadSetSize(curblock->m_pBlockOverlayObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(curblock->m_pBlockOverlayObject);

	xRenderable2DQuadSetPosition(curblock->m_pBuildArrowObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
	xRenderable2DQuadSetSize(curblock->m_pBuildArrowObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
	xRenderable2DQuadUpdate(curblock->m_pBuildArrowObject);

	DEBUGCURBLOCK("AddBlockRight setting CurBlock to %x\n", curblock);
	m_pCurBlock = curblock;

	xPieceUpdateBeatMarkerLocation(this);

	return true;
}

#define DEBUGEXTRABLOCK 1
void xPiece::AddExtraBlock()
{
	xBlock *oldcurblock, *debugblock;
	bool triedleft, triedright, triedup, trieddown;
	int newcolor, i;

	if(m_bDropping)
		return;

#if DEBUGEXTRABLOCK
	for(debugblock = m_pBlockList; debugblock; debugblock = debugblock->m_pNext)
		DEBUGLOG("AddExtraBlock - Before: %i %i %x\n", debugblock->m_iRowID, debugblock->m_iColumnID, debugblock);
#endif

	oldcurblock = m_pCurBlock;

	while(m_pCurBlock->m_pNext)
		m_pCurBlock = m_pCurBlock->m_pNext;

#if DEBUGEXTRABLOCK
	DEBUGLOG("AddExtraBlock - CurBlock: %x\n", m_pCurBlock);
#endif

	triedleft = false;
	triedright = false;
	triedup = false;
	trieddown = false;
	newcolor = rand() % g_xGameOptions.m_iNumColors;

	while(1)
	{
#if DEBUGEXTRABLOCK
	for(debugblock = m_pBlockList; debugblock; debugblock = debugblock->m_pNext)
		DEBUGLOG("AddExtraBlock - Attempt: %i %i %x\n", debugblock->m_iRowID, debugblock->m_iColumnID, debugblock);
#endif

		switch(rand() % 4)
		{
			case UP:
				if(triedup)
					continue;
				else
					triedup = true;

				if(!AddBlockUp(newcolor))
					continue;
				else
					break;

			case DOWN:
				if(trieddown)
					continue;
				else
					trieddown = true;

				if(!AddBlockDown(newcolor))
					continue;
				else
					break;

			case LEFT:
				if(triedleft)
					continue;
				else
					triedleft = true;

				if(!AddBlockLeft(newcolor))
					continue;
				else
					break;

			case RIGHT:
				if(triedright)
					continue;
				else
					triedright = true;

				if(!AddBlockRight(newcolor))
					continue;
				else
					break;
		}

#if DEBUGEXTRABLOCK
		DEBUGLOG("AddExtraBlock - SUCCESS! New CurBlock: %x\n", m_pCurBlock);
#endif

		i = 0;

		m_iUnfilled++;

		xRenderable2DQuadUpdateColor(m_pCurBlock->m_pBlockObject, &g_colUnfilledBlockColors[newcolor]);

		break;
	}

	if(i != 0)
	{
		m_pCurBlock = oldcurblock;

#if DEBUGEXTRABLOCK
	DEBUGLOG("AddExtraBlock - FAILED! New CurBlock: %x\n", m_pCurBlock);
#endif

		return;
	}

#if DEBUGEXTRABLOCK
	for(debugblock = m_pBlockList; debugblock; debugblock = debugblock->m_pNext)
		DEBUGLOG("AddExtraBlock - After: %i %i %x\n", debugblock->m_iRowID, debugblock->m_iColumnID, debugblock);
#endif

	m_pScaleInBlock = m_pCurBlock;
	m_fScaleInTime = BB_PIECE_NEWBLOCKSCALEINTIME;

	m_fScaleInBlockAnimCachedX = m_pScaleInBlock->m_pBlockObject->m_xRenderable2D.m_fX;
	m_fScaleInBlockAnimCachedY = m_pScaleInBlock->m_pBlockObject->m_xRenderable2D.m_fY;

	if(oldcurblock != m_pPrevBlock)
		m_pCurBlock = oldcurblock;

#if DEBUGEXTRABLOCK
	DEBUGLOG("AddExtraBlock completed set CurBlock to %x\n", m_pCurBlock);
#endif

	xRenderable2DQuadSetPosition(m_pScaleInBlock->m_pBlockObject, m_fScaleInBlockAnimCachedX - (0.0 - BB_BLOCKOFFSET) / 2.0, m_fScaleInBlockAnimCachedY - (0.0 - BB_BLOCKOFFSET) / 2.0);
	xRenderable2DQuadSetSize(m_pScaleInBlock->m_pBlockObject, 0.0, 0.0);
	xRenderable2DQuadUpdate(m_pScaleInBlock->m_pBlockObject);

	xRenderable2DQuadSetPosition(m_pScaleInBlock->m_pBlockOverlayObject, m_pScaleInBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pScaleInBlock->m_pBlockObject->m_xRenderable2D.m_fY);
	xRenderable2DQuadSetSize(m_pScaleInBlock->m_pBlockOverlayObject, 0.0, 0.0);
	xRenderable2DQuadUpdate(m_pScaleInBlock->m_pBlockOverlayObject);

	xRenderable2DQuadSetPosition(m_pScaleInBlock->m_pBuildArrowObject, m_pScaleInBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pScaleInBlock->m_pBlockObject->m_xRenderable2D.m_fY);
	xRenderable2DQuadSetSize(m_pScaleInBlock->m_pBuildArrowObject, 0.0, 0.0);
	xRenderable2DQuadUpdate(m_pScaleInBlock->m_pBuildArrowObject);

	xPieceUpdateBeatMarkerLocation(this);
}

void xPiece::Rotate()
{
	int hirow = INT_MIN, lowrow = INT_MAX, hicol = INT_MIN, lowcol = INT_MAX, column, xchange, ychange;
	xBlock* curblock, curblock2;

	//Find the center block around which to rotate
	curblock = m_pBlockList;
	for(; curblock; curblock = curblock->m_pNext)
	{
		DEBUGPIECEROTATE("Rotate - Start - %x - %i %i\n", curblock, curblock->m_iRowID, curblock->m_iColumnID);

		if(curblock->m_iRowID > hirow)
			hirow = curblock->m_iRowID;

		if(curblock->m_iRowID < lowrow)
			lowrow = curblock->m_iRowID;

		if(curblock->m_iColumnID > hicol)
			hicol = curblock->m_iColumnID;

		if(curblock->m_iColumnID < lowcol)
			lowcol = curblock->m_iColumnID;
	}

	//Rotate the piece
	//m_iCurFieldX = hicol;
	//m_iCurFieldY = hirow;
	curblock = m_pBlockList;
	while(curblock)
	{
		column = curblock->m_iColumnID;
		curblock->m_iColumnID = curblock->m_iRowID;
		curblock->m_iRowID = column * -1;

		DEBUGPIECEROTATE("Rotate - New Location - %x - %f, %f\n", curblock, BB_PIECESTARTX - (BB_BLOCKOFFSET * curblock->m_iColumnID), BB_PIECESTARTY - (BB_BLOCKOFFSET * curblock->m_iRowID));

		if(curblock == m_pCurBlock && curblock != m_pPrevBlock)
			xRenderable2DQuadSetPosition(curblock->m_pBlockObject, BB_PIECESTARTX - (BB_BLOCKOFFSET * curblock->m_iColumnID) - (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) / 2.0, BB_PIECESTARTY - (BB_BLOCKOFFSET * curblock->m_iRowID) - (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) / 2.0);
		else
			xRenderable2DQuadSetPosition(curblock->m_pBlockObject, BB_PIECESTARTX - (BB_BLOCKOFFSET * curblock->m_iColumnID), BB_PIECESTARTY - (BB_BLOCKOFFSET * curblock->m_iRowID));

		xRenderable2DQuadUpdate(curblock->m_pBlockObject);

		xRenderable2DQuadSetPosition(curblock->m_pBlockOverlayObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
		xRenderable2DQuadUpdate(curblock->m_pBlockOverlayObject);

		xRenderable2DQuadSetPosition(curblock->m_pBuildArrowObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
		xRenderable2DQuadUpdate(curblock->m_pBuildArrowObject);

		curblock = curblock->m_pNext;
	}

	//Make sure it's within the min/max
	xchange = 0;
	ychange = 0;
	curblock = m_pBlockList;
	while(curblock)
	{
		DEBUGPIECEROTATE("Rotate - Checking RowID and ColumnID - %x - %i + %i, %i + %i\n", curblock, curblock->m_iColumnID, xchange, curblock->m_iRowID, ychange);
		if(curblock->m_iColumnID + xchange < BB_PIECEMINCOLUMNID)
			xchange++;
		else if(curblock->m_iColumnID + xchange > BB_PIECEMAXCOLUMNID)
			xchange--;

		if(curblock->m_iRowID + ychange < BB_PIECEMINROWID)
			ychange++;
		else if(curblock->m_iRowID + ychange > BB_PIECEMAXROWID)
			ychange--;

		curblock = curblock->m_pNext;
	}

	if(xchange != 0 || ychange != 0)
	{
		curblock = m_pBlockList;
		while(curblock)
		{
			curblock->m_iColumnID += xchange;
			curblock->m_iRowID += ychange;

			DEBUGPIECEROTATE("Rotate - Adjusting Location - %x - %f, %f\n", curblock, BB_PIECESTARTX - (BB_BLOCKOFFSET * curblock->m_iColumnID), BB_PIECESTARTY - (BB_BLOCKOFFSET * curblock->m_iRowID));

			if(curblock == m_pCurBlock && curblock != m_pPrevBlock)
				xRenderable2DQuadSetPosition(curblock->m_pBlockObject, BB_PIECESTARTX - (BB_BLOCKOFFSET * curblock->m_iColumnID) - (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) / 2.0, BB_PIECESTARTY - (BB_BLOCKOFFSET * curblock->m_iRowID) - (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) / 2.0);
			else
				xRenderable2DQuadSetPosition(curblock->m_pBlockObject, BB_PIECESTARTX - (BB_BLOCKOFFSET * curblock->m_iColumnID), BB_PIECESTARTY - (BB_BLOCKOFFSET * curblock->m_iRowID));

			xRenderable2DQuadUpdate(curblock->m_pBlockObject);

			xRenderable2DQuadSetPosition(curblock->m_pBlockOverlayObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
			xRenderable2DQuadUpdate(curblock->m_pBlockOverlayObject);

			xRenderable2DQuadSetPosition(curblock->m_pBuildArrowObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
			xRenderable2DQuadUpdate(curblock->m_pBuildArrowObject);

			curblock = curblock->m_pNext;
		}
	}

	xRenderable2DQuadUpdateVisibility(m_pBlockAnimation, 0);

	InterruptAnimations();

	xPieceUpdateBeatMarkerLocation(this);
}

void xPiece::FillInBlock(bool bUsePieceColor, bool bWhiteBlockTimerActive)
{
	int oldcolorindex = m_pCurBlock->m_iUnfilledColorIndex;

	if(!bUsePieceColor)
	{
		do
		{
			m_pCurBlock->m_iUnfilledColorIndex = rand() % g_xGameOptions.m_iNumRandomColors;
		} while(!bWhiteBlockTimerActive && m_pCurBlock->m_iUnfilledColorIndex == m_iColorIndex);

		m_pCurBlock->m_iColorIndex = m_pCurBlock->m_iUnfilledColorIndex;
	}
	else if(bWhiteBlockTimerActive)
		m_pCurBlock->m_iColorIndex = BB_COLOR_WHITE;
	else
		m_pCurBlock->m_iColorIndex = m_iColorIndex;

	xRenderable2DQuadUpdateTexture(m_pCurBlock->m_pBlockObject, m_pBlockTextures[0]);

	if(!g_xGameOptions.m_bTutorial || bUsePieceColor)
		xRenderable2DQuadUpdateVisibility(m_pCurBlock->m_pBuildArrowObject, 0);

	InterruptAnimations();

	DEBUGCURBLOCK("FillInBlock setting PrevBlock to %x\n", m_pCurBlock);
	m_pPrevBlock = m_pCurBlock;

	xColorCopy(&m_xPrevBlockOldColor, &g_colFilledBlockColors[oldcolorindex]);

	if(m_pCurBlock->m_pNext)
		m_pCurBlock = m_pCurBlock->m_pNext;

	m_bLastBuildSuccessful = bUsePieceColor;

	if(!g_xGameOptions.m_bTutorial || m_bLastBuildSuccessful)
		m_fBuildAnimationTime = 0.0f;

	m_iUnfilled--;

	xPieceUpdateBeatMarkerLocation(this);
}

void xPiece::Slide(float fXChange, float fYChange, bool bUpdateFieldPosition)
{
	xBlock *curblock;
	int i;

	if(bUpdateFieldPosition)
	{
		if(fXChange < 0)
			m_iCurFieldX--;
		else if(fXChange > 0)
			m_iCurFieldX++;

		if(fYChange > 0)
			m_iCurFieldY++;
	}

	for(curblock = m_pBlockList; curblock; curblock = curblock->m_pNext)
	{
		if(curblock->m_pBlockObject)
		{
			curblock->m_pBlockObject->m_xRenderable2D.m_fX += fXChange;
			curblock->m_pBlockObject->m_xRenderable2D.m_fY += fYChange;
			xRenderable2DQuadUpdate(curblock->m_pBlockObject);

			xRenderable2DQuadSetPosition(curblock->m_pBlockOverlayObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
			xRenderable2DQuadUpdate(curblock->m_pBlockOverlayObject);

			xRenderable2DQuadSetPosition(curblock->m_pBuildArrowObject, curblock->m_pBlockObject->m_xRenderable2D.m_fX, curblock->m_pBlockObject->m_xRenderable2D.m_fY);
			xRenderable2DQuadUpdate(curblock->m_pBuildArrowObject);
		}
	}

	xPieceUpdateBeatMarkerLocation(this);
}

void xPiece::InterruptAnimations()
{
	DEBUGBLOCKANIM("InterruptAnimations %f %f\n", m_fBuildAnimationTime, m_fScaleInTime);

	m_bDoneAnimatingBlocks = 1;

	// Clear Animation timers(as they were often undoing our rotations)
	if(m_fBuildAnimationTime < BB_PIECE_ANIMBUILDSUCCESSFUL)
	{
		m_fBuildAnimationTime = BB_PIECE_ANIMBUILDSUCCESSFUL + 0.1f;

		xRenderable2DQuadUpdateDepth(m_pPrevBlock->m_pBlockObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_PIECEBLOCKS);
		xRenderable2DQuadUpdateDepth(m_pPrevBlock->m_pBlockOverlayObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_PIECEBLOCKOVERLAY);
		xRenderable2DQuadUpdateDepth(m_pPrevBlock->m_pBuildArrowObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_PIECEBUILDARROW);

		xRenderable2DQuadSetPosition(m_pPrevBlock->m_pBlockObject, BB_PIECESTARTX - BB_BLOCKOFFSET * m_pPrevBlock->m_iColumnID, BB_PIECESTARTY - BB_BLOCKOFFSET * m_pPrevBlock->m_iRowID);
		xRenderable2DQuadSetSize(m_pPrevBlock->m_pBlockObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
		xRenderable2DQuadUpdateColor(m_pPrevBlock->m_pBlockObject, &g_colFilledBlockColors[m_pPrevBlock->m_iColorIndex]);
		xRenderable2DQuadUpdate(m_pPrevBlock->m_pBlockObject);

		xRenderable2DQuadSetPosition(m_pPrevBlock->m_pBlockOverlayObject, m_pPrevBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pPrevBlock->m_pBlockObject->m_xRenderable2D.m_fY);
		xRenderable2DQuadSetSize(m_pPrevBlock->m_pBlockOverlayObject, m_pPrevBlock->m_pBlockObject->m_xRenderable2D.m_fWidth, m_pPrevBlock->m_pBlockObject->m_xRenderable2D.m_fHeight);
		xRenderable2DQuadUpdate(m_pPrevBlock->m_pBlockOverlayObject);

		if(m_pCurBlock != m_pPrevBlock)
		{
			xRenderable2DQuadSetPosition(m_pCurBlock->m_pBlockObject, BB_PIECESTARTX - BB_BLOCKOFFSET * m_pCurBlock->m_iColumnID - (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) / 2.0, BB_PIECESTARTY - BB_BLOCKOFFSET * m_pCurBlock->m_iRowID - (BB_LARGEBLOCKOFFSET - BB_BLOCKOFFSET) / 2.0);
			xRenderable2DQuadSetSize(m_pCurBlock->m_pBlockObject, BB_LARGEBLOCKOFFSET, BB_LARGEBLOCKOFFSET);
			xRenderable2DQuadUpdateDepth(m_pCurBlock->m_pBlockObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_CURPIECEBLOCK);
			xRenderable2DQuadUpdateDepth(m_pCurBlock->m_pBlockOverlayObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_CURPIECEBLOCKOVERLAY);
			xRenderable2DQuadUpdateDepth(m_pCurBlock->m_pBuildArrowObject, g_xGraphics.m_pSceneGraph, BB_DEPTH_CURPIECEBUILDARROW);
		}
		else
		{
			xRenderable2DQuadSetPosition(m_pCurBlock->m_pBlockObject, BB_PIECESTARTX - BB_BLOCKOFFSET * m_pCurBlock->m_iColumnID, BB_PIECESTARTY - BB_BLOCKOFFSET * m_pCurBlock->m_iRowID);
			xRenderable2DQuadSetSize(m_pCurBlock->m_pBlockObject, BB_BLOCKOFFSET, BB_BLOCKOFFSET);
		}

		xRenderable2DQuadUpdate(m_pCurBlock->m_pBlockObject);

		xRenderable2DQuadSetPosition(m_pCurBlock->m_pBlockOverlayObject, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY);
		xRenderable2DQuadSetSize(m_pCurBlock->m_pBlockOverlayObject, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fWidth, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fHeight);
		xRenderable2DQuadUpdate(m_pCurBlock->m_pBlockOverlayObject);

		xRenderable2DQuadSetPosition(m_pCurBlock->m_pBuildArrowObject, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fX, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fY);
		xRenderable2DQuadSetSize(m_pCurBlock->m_pBuildArrowObject, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fWidth, m_pCurBlock->m_pBlockObject->m_xRenderable2D.m_fHeight);
		xRenderable2DQuadUpdate(m_pCurBlock->m_pBuildArrowObject);
	}

	if(m_fScaleInTime != 0.0f)
	{
		m_fScaleInTime = 0.0f;

		xRenderable2DQuadSetSize(m_pScaleInBlock->m_pBlockObject, BB_LARGEBLOCKOFFSET, BB_LARGEBLOCKOFFSET);
		xRenderable2DQuadUpdate(m_pScaleInBlock->m_pBlockObject);

		xRenderable2DQuadSetSize(m_pScaleInBlock->m_pBlockOverlayObject, BB_LARGEBLOCKOFFSET, BB_LARGEBLOCKOFFSET);
		xRenderable2DQuadUpdate(m_pScaleInBlock->m_pBlockOverlayObject);

		xRenderable2DQuadSetSize(m_pScaleInBlock->m_pBuildArrowObject, BB_LARGEBLOCKOFFSET, BB_LARGEBLOCKOFFSET);
		xRenderable2DQuadUpdate(m_pScaleInBlock->m_pBuildArrowObject);
	}
}

FORCE_INLINE void xPieceCreateBuildArrowObject(xBlock *pBlock)
{
	switch(pBlock->m_iUnfilledColorIndex)
	{
		case 0: // Red
			pBlock->m_pBuildArrowObject = xSceneGraphCreateQuad(BB_IMAGE_BUILDARROWUP, 0);
			xGraphicsGetTextures(BB_IMAGE_BUILDARROWUP, &pBlock->m_pBuildArrowTextures);
			break;

		case 1: // Blue
			pBlock->m_pBuildArrowObject = xSceneGraphCreateQuad(BB_IMAGE_BUILDARROWRIGHT, 0);
			xGraphicsGetTextures(BB_IMAGE_BUILDARROWRIGHT, &pBlock->m_pBuildArrowTextures);
			break;

		case 2: // Yellow
			pBlock->m_pBuildArrowObject = xSceneGraphCreateQuad(BB_IMAGE_BUILDARROWLEFT, 0);
			xGraphicsGetTextures(BB_IMAGE_BUILDARROWLEFT, &pBlock->m_pBuildArrowTextures);
			break;

		case 3: // Purple
			pBlock->m_pBuildArrowObject = xSceneGraphCreateQuad(BB_IMAGE_BUILDARROWUPRIGHT, 0);
			xGraphicsGetTextures(BB_IMAGE_BUILDARROWUPRIGHT, &pBlock->m_pBuildArrowTextures);
			break;

		case 4: // Orange
			pBlock->m_pBuildArrowObject = xSceneGraphCreateQuad(BB_IMAGE_BUILDARROWUPLEFT, 0);
			xGraphicsGetTextures(BB_IMAGE_BUILDARROWUPLEFT, &pBlock->m_pBuildArrowTextures);
			break;

		case 5: // Green
			pBlock->m_pBuildArrowObject = xSceneGraphCreateQuad(BB_IMAGE_BUILDARROWLEFTRIGHT, 0);
			xGraphicsGetTextures(BB_IMAGE_BUILDARROWLEFTRIGHT, &pBlock->m_pBuildArrowTextures);
			break;
	}
}

void xPiece::UpdateBeatMarkerLocation()
{
	xPieceUpdateBeatMarkerLocation(this);
}

FORCE_INLINE void xPieceUpdateBeatMarkerLocation(xPiece *pThis)
{
	xBlock *curblock;
	int i, j;

	if(pThis->m_iUnfilled)
	{
		for(curblock = pThis->m_pCurBlock, i = 1; curblock; curblock = curblock->m_pNext)
		{
			xRenderable2DQuadUpdateColor(curblock->m_pBlockObject, &g_colFilledBlockColors[curblock->m_iUnfilledColorIndex]);
			xRenderable2DQuadUpdateTexture(curblock->m_pBlockObject, pThis->m_pBlockTextures[i]);
			xRenderable2DQuadUpdateTexture(curblock->m_pBuildArrowObject, curblock->m_pBuildArrowTextures[i]);

			if(i < 4)
				i++;
		}

		if(!pThis->m_pBeatMarkerDimmer->m_xRenderable2D.m_bVisible)
		{
			if(!g_xGameOptions.m_bTutorial)
			{
				for(i = 0; i < BB_FIELDWIDTH; i++)
				{
					for(j = 0; j < BB_FIELDHEIGHT; j++)
					{
						if(!g_xGame.m_xPlayer.m_xField.m_pFieldBlocks[i][j].m_bClearable)
							xRenderable2DQuadUpdateAlpha(g_xGame.m_xPlayer.m_xField.m_pFieldBlocks[i][j].m_pBlockGlowObject, g_xGame.m_xPlayer.m_xField.m_pFieldBlocks[i][j].m_pBlockGlowObject->m_xRenderable2D.m_xColor.A * 0.5f);
					}
				}
			}

			xRenderable2DQuadUpdateVisibility(pThis->m_pBeatMarkerDimmer, 1);
			xRenderable2DQuadUpdateVisibility(pThis->m_pBeatMarkerHitzone, 1);
		}
	}
	else if(pThis->m_pBeatMarkerDimmer->m_xRenderable2D.m_bVisible)
	{
		if(!g_xGameOptions.m_bTutorial)
		{
			for(i = 0; i < BB_FIELDWIDTH; i++)
			{
				for(j = 0; j < BB_FIELDHEIGHT; j++)
				{
					if(!g_xGame.m_xPlayer.m_xField.m_pFieldBlocks[i][j].m_bClearable)
						xRenderable2DQuadUpdateAlpha(g_xGame.m_xPlayer.m_xField.m_pFieldBlocks[i][j].m_pBlockGlowObject, g_xGame.m_xPlayer.m_xField.m_pFieldBlocks[i][j].m_pBlockGlowObject->m_xRenderable2D.m_xColor.A * 2.0f);
				}
			}
		}

		xRenderable2DQuadUpdateVisibility(pThis->m_pBeatMarkerDimmer, 0);
		xRenderable2DQuadUpdateVisibility(pThis->m_pBeatMarkerHitzone, 0);
	}
}

FORCE_INLINE void xPieceUpdateBeatMarker(xRenderable2DQuad *pBeatMarker, float fBeatTime, int iBeatTime, float fBeatMarkerX, float fBeatMarkerY)
{
	xColor tempcolor;
	float mainbeatx, mainbeaty, hittime;
	float markersize = BB_BEATMARKERMINSIZE + ((BB_BEATMARKERMAXSIZE - BB_BEATMARKERMINSIZE) * fBeatTime);

	//DEBUGLOG("UpdateBeatMarker - beattime:%1.4f markersize:%1.4f\n", fBeatTime, markersize);
	if(!pBeatMarker->m_xRenderable2D.m_bVisible)
		xRenderable2DQuadUpdateVisibility(pBeatMarker, true);

	mainbeatx = fBeatMarkerX + BB_BEATMARKEROFFSETX * (1.0 - fBeatTime);
	mainbeaty = fBeatMarkerY + BB_BEATMARKEROFFSETY * (1.0 - fBeatTime);

	if(iBeatTime <= g_xGameOptions.m_iHitTime)
	{
		hittime = ((float)iBeatTime / (float)g_xGameOptions.m_iHitTime);

//		// Fade from Yellow to Green, then White Flash, before fading out(stays White)
		if(hittime >= 1.0f)
			xColorInit(&tempcolor, 0.9f, 0.9f, 0.0f, (1.0 - fBeatTime) * (1.0 - fBeatTime));
		else if(hittime >= 0.25f)
			xColorInit(&tempcolor, ((hittime - 0.25f) / 0.75f) * 0.9f * 2.0f, 0.9f, 0.0f, (1.0 - fBeatTime) * (1.0 - fBeatTime));
		else if(hittime >= 0.0f)
			xColorInit(&tempcolor, 1.0f - (hittime / 0.25f), 1.0f - (hittime / 0.25f), 1.0f - (hittime / 0.25f), (1.0 - fBeatTime) * (1.0 - fBeatTime));
		else
			xColorInit(&tempcolor, 1.0f, 1.0f, 1.0f, 1.0f + hittime);
//*/		
/*/		// Fade from Yellow to Green, then fade out(stays Green)
		if(hittime >= 1.0)
			xColorInit(&tempcolor, 0.9f, 0.9f, 0.0f, (1.0 - fBeatTime) * (1.0 - fBeatTime));
		else if(hittime >= 0.0)
			xColorInit(&tempcolor, hittime * 0.9f * 2.0f, 0.9f, 0.0f, (1.0 - fBeatTime) * (1.0 - fBeatTime));
		else
			xColorInit(&tempcolor, 0.0f, 0.9f, 0.0f, 1.0f + hittime);
//*/
/*/		// Fade from Yellow to Green, then fading out to Red
		if(hittime >= 1.0)
			xColorInit(&tempcolor, 0.9f, 0.9f, 0.0f, (1.0 - fBeatTime) * (1.0 - fBeatTime));
		else if(hittime >= 0.0)
			xColorInit(&tempcolor, hittime * 0.9f * 2.0f, 0.9f, 0.0f, (1.0 - fBeatTime) * (1.0 - fBeatTime));
		else
			xColorInit(&tempcolor, -1.0f * hittime, 0.9f + hittime, 0.0f, 1.0f + hittime * 0.5f);
//*/
/*/		// Fade from Yellow to Green, then White Flash, before fading out to Red
		if(hittime >= 1.0f)
			xColorInit(&tempcolor, 0.9f, 0.9f, 0.0f, (1.0 - fBeatTime) * (1.0 - fBeatTime));
		else if(hittime >= 0.25f)
			xColorInit(&tempcolor, ((hittime - 0.25f) / 0.75f) * 0.9f * 2.0f, 0.9f, 0.0f, (1.0 - fBeatTime) * (1.0 - fBeatTime));
		else if(hittime >= 0.0f)
			xColorInit(&tempcolor, 1.0f - (hittime / 0.25f), 1.0f - (hittime / 0.25f), 1.0f - (hittime / 0.25f), (1.0 - fBeatTime) * (1.0 - fBeatTime));
		else
			xColorInit(&tempcolor, 1.0f, 0.9f + hittime, 0.9f + hittime, 1.0f + hittime * 0.5f);
//*/
		//DEBUGLOG("BeatMarkerTiming: %i %i %f %f %f,%f %f (%f,%f,%f,%f)\n", iBeatTime, g_xGameOptions.m_iHitTime, hittime, fBeatTime, mainbeatx, mainbeaty, markersize, tempcolor.R, tempcolor.G, tempcolor.B, tempcolor.A);
	}
	else if(iBeatTime <= g_xGameOptions.m_iHitTime * 3)
	//else if(dwBeatTime <= g_xGameOptions.m_iHitTime * 2)
	{
		hittime = (((float)iBeatTime - (float)g_xGameOptions.m_iHitTime) / ((float)g_xGameOptions.m_iHitTime * 2.0f));
		//hittime = (((float)dwBeatTime - (float)g_xGameOptions.m_iHitTime) / (float)g_xGameOptions.m_iHitTime);

		xColorInit(&tempcolor, 0.9f, (1.0f - hittime) * 0.9f, 0.0f, (1.0 - fBeatTime) * (1.0 - fBeatTime));
	}
	else
		xColorInit(&tempcolor, 0.9f, 0.0f, 0.0f, (1.0 - fBeatTime) * (1.0 - fBeatTime));

	xRenderable2DQuadSetPosition(pBeatMarker, mainbeatx, mainbeaty);
	xRenderable2DQuadSetSize(pBeatMarker, markersize, markersize);
	xRenderable2DQuadUpdateColor(pBeatMarker, &tempcolor);
	xRenderable2DQuadUpdate(pBeatMarker);
}