/////////////////////////////////////////////////////////////////
// Name: xPiece.h
/////////////////////////////////////////////////////////////////
// Description:
// - Serves as a container for a falling group of blocks.
/////////////////////////////////////////////////////////////////
// - 7.15.2004, Update, Dayle Flowers(Xienen)
//   File Created
/////////////////////////////////////////////////////////////////
// - 7.26.2004, Update, Greg Feldman
//   Modified xPiece to use an xList of xBlockRows
/////////////////////////////////////////////////////////////////
// - 8.06.2004, Update, Alayton Norgard (Adept)
//   Added number rendering inside the active piece
/////////////////////////////////////////////////////////////////
// - 8.07.2004, Update, Alayton Norgard (Adept)
//   - Added 'numbers in block'
//   - Added drop piece functionality
/////////////////////////////////////////////////////////////////
#ifndef XPIECE_H_
#define XPIECE_H_

#include "xBlock.h"
#include "xBeatManager.h"
#include "xGameDefines.h"

enum xBBColor
{
	BB_COLOR_RED = 0,
	BB_COLOR_BLUE,
	BB_COLOR_YELLOW,
	BB_COLOR_PURPLE,
	BB_COLOR_ORANGE,
	BB_COLOR_GREEN,
	BB_COLOR_WHITE
};

extern const xColor g_colFilledBlockColors[BB_PIECECOLORS_MAX];
extern const xColor g_colUnfilledBlockColors[BB_PIECECOLORS_MAX];
extern const xColor g_colBlockGlowColors[BB_PIECECOLORS_MAX];

class xPiece
{
public:
	xPiece();

	void Init(float fFirstBlockX, float fFirstBlockY, int iBlockCount, bool bCenterAroundPoint, bool bNoBlocks);
	void PreUpdate(DWORD dwTime);
	void Update(double dTime, bool bIsActivePiece);
	void RefreshAllRenderables();
	void Delete(bool bDeleteBlocks, bool bDeleteBlockObjects, bool bDeleteCursor);

	void ForceUpdateBeatMarkers(DWORD dwTime);

	bool AddBlockUp(int iBlockColor);
	bool AddBlockDown(int iBlockColor);
	bool AddBlockLeft(int iBlockColor);
	bool AddBlockRight(int iBlockColor);
	void AddExtraBlock();
	void Rotate();

	bool CurBlockContainsRed() { return m_pCurBlock->m_iUnfilledColorIndex == 0 || m_pCurBlock->m_iUnfilledColorIndex == 3 || m_pCurBlock->m_iUnfilledColorIndex == 4; }
	bool CurBlockContainsBlue() { return m_pCurBlock->m_iUnfilledColorIndex == 1 || m_pCurBlock->m_iUnfilledColorIndex == 3 || m_pCurBlock->m_iUnfilledColorIndex == 5; }
	bool CurBlockContainsYellow() { return m_pCurBlock->m_iUnfilledColorIndex == 2 || m_pCurBlock->m_iUnfilledColorIndex == 4 || m_pCurBlock->m_iUnfilledColorIndex == 5; }
	bool ColorComplete(bool bRed, bool bBlue, bool bYellow, bool *bOutCorrect)
	{
		switch(m_pCurBlock->m_iUnfilledColorIndex)
		{
			case 0:
				*bOutCorrect = bRed && !bBlue && !bYellow;
				return bRed;
			case 1:
				*bOutCorrect = !bRed && bBlue && !bYellow;
				return bBlue;
			case 2:
				*bOutCorrect = !bRed && !bBlue && bYellow;
				return bYellow;
			case 3:
				*bOutCorrect = bRed && bBlue && !bYellow;
				return bRed && bBlue;
			case 4:
				*bOutCorrect = bRed && !bBlue && bYellow;
				return bRed && bYellow;
			case 5:
				*bOutCorrect = !bRed && bBlue && bYellow;
				return bBlue && bYellow;
		}
	}

	void FillInBlock(bool bUsePieceColor, bool bWhiteBlockTimerActive);
	void Slide(float fXChange, float fYChange, bool bUpdateFieldPosition = true);
	void InterruptAnimations();

	void UpdateBeatMarkerLocation();
	void SetBeatManager(xBeatManager* pBeatManager) { m_pBeatManager = pBeatManager; }

	int					  m_iCurFieldX;
	int					  m_iCurFieldY;
	int					  m_iColorIndex;
	xBlock				 *m_pBlockList;
	xBlock				 *m_pPrevBlock;
	xBlock				 *m_pCurBlock;
	xBlock				 *m_pScaleInBlock;
	int					  m_iNumBlocks;
	int					  m_iUnfilled;
	bool				  m_bDropping;
	bool				  m_bTouchedDown;
	bool				  m_bPlayedTouchDown;
	bool				  m_bSkippedBeat;
	bool				  m_bMissedCombo;
	DWORD				  m_dwCurTime;
	DWORD				  m_dwSongLength;
	xTexture			**m_pBlockTextures;
	xRenderable2DQuad	 *m_pBlockAnimation;

	//Build Animation Helpers
	bool	m_bLastBuildSuccessful;
	bool	m_bDoneAnimatingBlocks;
	float	m_fBuildAnimationTime;
	float	m_fScaleInTime;
	float	m_fScaleInBlockAnimCachedX;
	float	m_fScaleInBlockAnimCachedY;
	xColor	m_xPrevBlockOldColor;

	xBeatManager		*m_pBeatManager;
	xRenderable2DQuad	*m_pBeatMarkerDimmer;
	xRenderable2DQuad	*m_pBeatMarkers[BB_BEATMARKERS_MAX];
	xRenderable2DQuad	*m_pBeatMarkerHitzone;
};

#endif