/////////////////////////////////////////////////////////////////
// Name: xPool.h
/////////////////////////////////////////////////////////////////
// Description:
// - A pool object, used to never reload assets that have already
//   been loaded(textures, sounds, etc.)
/////////////////////////////////////////////////////////////////
// - 7.7.2004, Update, Arto Ruotsalainen (Arex)
//	 Destructor uses delete now.
//
// - 7.6.2004, Update, Arto Ruotsalainen(Arex)
//   Comment stuff added to begin of the file.
//   Add new updates/fixes above this, so the old updates
//   will be at the bottom.
/////////////////////////////////////////////////////////////////
#ifndef XPOOL_H_
#define XPOOL_H_

#include "xPoolItem.h"

template<class T>
class xPool
{
public:
	xPool()
	{
		m_xObjectList = 0;
		m_xCurrentObject = 0;
	}

	T* CreateNewObject(const char* strObjectPathName)
	{
		if(!m_xObjectList)
		{
			m_xObjectList	 = new T;
			m_xCurrentObject = m_xObjectList;
		}
		else
		{
			m_xCurrentObject = m_xObjectList;
			while(m_xCurrentObject->m_pNext)
			{
				if(!stricmp(m_xCurrentObject->GetObjectPath(), strObjectPathName))
				{
					m_xCurrentObject->m_iReferenceCount++;
					return (T*)m_xCurrentObject;
				}

				m_xCurrentObject = m_xCurrentObject->m_pNext;
			}

			if(!stricmp(m_xCurrentObject->GetObjectPath(), strObjectPathName))
			{
				m_xCurrentObject->m_iReferenceCount++;
				return (T*)m_xCurrentObject;
			}

			m_xCurrentObject->m_pNext			= new T;
			m_xCurrentObject->m_pNext->m_pPrev	= m_xCurrentObject;
			m_xCurrentObject					= m_xCurrentObject->m_pNext;
		}
		
		m_xCurrentObject->SetObjectPath(strObjectPathName);
		return (T*)m_xCurrentObject;
	}

	int DeleteObject(char *strObjectPathName)
	{
		xPoolItem<T> *temp = m_xObjectList;

		while (temp)
		{
			if (!stricmp(temp->GetObjectPath(), strObjectPathName))
			{
				if(m_xObjectList == temp)
					m_xObjectList = temp->m_pNext;

				if(m_xCurrentObject == temp)
				{
					if(temp->m_pNext)
						m_xCurrentObject = temp->m_pNext;
					else
						m_xCurrentObject = temp->m_pPrev;
				}

				if(temp->m_pPrev)
					temp->m_pPrev->m_pNext = temp->m_pNext;

				if(temp->m_pNext)
					temp->m_pNext->m_pPrev = temp->m_pPrev;

				delete (T*)temp;
				return PE_RES_NOERROR;
			}

			temp = temp->m_pNext;
		}

		// Error 
		return(1);
	}

	int DeleteObject(T *pObject)
	{
		xPoolItem<T> *temp = m_xObjectList;

		while (temp)
		{
			if ((T*)temp == pObject)
			{
				if(m_xObjectList == temp)
					m_xObjectList = temp->m_pNext;

				if(m_xCurrentObject == temp)
				{
					if(temp->m_pNext)
						m_xCurrentObject = temp->m_pNext;
					else
						m_xCurrentObject = temp->m_pPrev;
				}

				if(temp->m_pPrev)
					temp->m_pPrev->m_pNext = temp->m_pNext;

				if(temp->m_pNext)
					temp->m_pNext->m_pPrev = temp->m_pPrev;

				delete pObject;

				return PE_RES_NOERROR;
			}

			temp = temp->m_pNext;
		}

		// Error 
		return(1);
	}

	xPoolItem<T> *m_xObjectList;
	xPoolItem<T> *m_xCurrentObject;
};

#endif