/////////////////////////////////////////////////////////////////
// Name: xRenderable2DQuad.h
/////////////////////////////////////////////////////////////////
// Description:
// - A renderable object(either a sprite or GUI object)
/////////////////////////////////////////////////////////////////
// - 23.8.2010, Created, Dayle Flowers (Xienen)
//   Created this as base class for xObject2D and xGUIObject to
//   allow for generic Animation system(xKeyFrame)
// - 14.12.2010, Updated, Dayle Flowers (Xienen)
//   Switched to C-style data structures
/////////////////////////////////////////////////////////////////
#ifndef XRENDERABLE2DQUAD_H_
#define XRENDERABLE2DQUAD_H_

#include "xEngineDefines.h"
#include "xRenderable2D.h"

struct xRenderable2DQuad
{
	xRenderable2D	  m_xRenderable2D;
	int				  m_iNumTextures;
	int				  m_iCurrentTexture;
	xTexture		**m_pTextures;
	xColor			  m_xColor2;
	float			  m_fMaskTextureOffsetX;
	float			  m_fMaskTextureOffsetY;
	float			  m_fTextureWidth;
	float			  m_fTextureHeight;
	float			  m_fAnimationTime;
	float			  m_fAnimationRate;
};

FORCE_INLINE void xRenderable2DQuadInit(xRenderable2DQuad *pThis)
{
	xRenderable2DInit(&pThis->m_xRenderable2D);
	
	xColorInit(&pThis->m_xColor2);
	pThis->m_iNumTextures = 0;
	pThis->m_pTextures = 0;
	pThis->m_iCurrentTexture = 0;
	pThis->m_fMaskTextureOffsetX = 0.0;
	pThis->m_fMaskTextureOffsetY = 0.0;
	pThis->m_fTextureWidth = 1.0;
	pThis->m_fTextureHeight = 1.0;
	pThis->m_fAnimationTime = 0;
	pThis->m_fAnimationRate = 1.0 / 30.0;
}

FORCE_INLINE void xRenderable2DQuadSetDepth(xRenderable2DQuad *pThis, float fDepth) { xRenderable2DSetDepth(&pThis->m_xRenderable2D, fDepth); }
FORCE_INLINE void xRenderable2DQuadSetBlendMode(xRenderable2DQuad *pThis, int iBlendMode) { xRenderable2DSetBlendMode(&pThis->m_xRenderable2D, iBlendMode); }
FORCE_INLINE void xRenderable2DQuadSetPosition(xRenderable2DQuad *pThis, float fX, float fY) { xRenderable2DSetPosition(&pThis->m_xRenderable2D, fX, fY); }
FORCE_INLINE void xRenderable2DQuadSetSize(xRenderable2DQuad *pThis, float fWidth, float fHeight) { xRenderable2DSetSize(&pThis->m_xRenderable2D, fWidth, fHeight); }
FORCE_INLINE void xRenderable2DQuadSetRotation(xRenderable2DQuad *pThis, float fRotation) { xRenderable2DSetRotation(&pThis->m_xRenderable2D, fRotation); }
FORCE_INLINE void xRenderable2DQuadSetRotationCenter(xRenderable2DQuad *pThis, float fRotationCenterX, float fRotationCenterY) { xRenderable2DSetRotationCenter(&pThis->m_xRenderable2D, fRotationCenterX, fRotationCenterY); }
FORCE_INLINE void xRenderable2DQuadSetMirror(xRenderable2DQuad *pThis, bool bMirrored) { xRenderable2DSetMirror(&pThis->m_xRenderable2D, bMirrored); }
FORCE_INLINE void xRenderable2DQuadSetScale(xRenderable2DQuad *pThis, float fScaleX, float fScaleY) { xRenderable2DSetScale(&pThis->m_xRenderable2D, fScaleX, fScaleY); }
FORCE_INLINE void xRenderable2DQuadSetColor(xRenderable2DQuad *pThis, const xColor *pSource) { xRenderable2DSetColor(&pThis->m_xRenderable2D, pSource); }
FORCE_INLINE void xRenderable2DQuadSetAlpha(xRenderable2DQuad *pThis, float fAlpha) { xRenderable2DSetAlpha(&pThis->m_xRenderable2D, fAlpha); }
FORCE_INLINE void xRenderable2DQuadSetColor2(xRenderable2DQuad *pThis, const xColor *pSource) { xColorCopy(&pThis->m_xColor2, pSource); }
FORCE_INLINE void xRenderable2DQuadSetTexture(xRenderable2DQuad *pThis, xTexture *pTexture) { if(!pThis->m_pTextures) { pThis->m_pTextures = new xTexture*[2]; pThis->m_iNumTextures = 0; } pThis->m_pTextures[0] = pTexture; }
FORCE_INLINE void xRenderable2DQuadSetTexture2(xRenderable2DQuad *pThis, xTexture *pTexture) { if(!pThis->m_pTextures) { pThis->m_pTextures = new xTexture*[2]; pThis->m_iNumTextures = 0; } pThis->m_pTextures[1] = pTexture; }
FORCE_INLINE void xRenderable2DQuadSetMaskPosition(xRenderable2DQuad *pThis, float fX, float fY) { pThis->m_fMaskTextureOffsetX = pThis->m_xRenderable2D.m_fX - fX; pThis->m_fMaskTextureOffsetY = pThis->m_xRenderable2D.m_fY - fY; }

FORCE_INLINE void xRenderable2DQuadCopy(xRenderable2DQuad *pDest, xRenderable2DQuad *pSource)
{
	int i;

	xRenderable2DCopy(&pDest->m_xRenderable2D, &pSource->m_xRenderable2D);

	xColorCopy(&pDest->m_xColor2, &pSource->m_xColor2);

	if(pSource->m_pTextures)
	{
		pDest->m_pTextures = new xTexture*[pSource->m_iNumTextures];

		for(i = 0; i < pSource->m_iNumTextures; i++)
			pDest->m_pTextures[i] = pSource->m_pTextures[i];
	}

	pDest->m_iNumTextures = pSource->m_iNumTextures;
	pDest->m_iCurrentTexture = pSource->m_iCurrentTexture;
	pDest->m_fMaskTextureOffsetX = pSource->m_fMaskTextureOffsetX;
	pDest->m_fMaskTextureOffsetY = pSource->m_fMaskTextureOffsetY;
	pDest->m_fTextureWidth = pSource->m_fTextureWidth;
	pDest->m_fTextureHeight = pSource->m_fTextureHeight;
	pDest->m_fAnimationTime = pSource->m_fAnimationTime;
	pDest->m_fAnimationRate = pSource->m_fAnimationRate;
}

#define xRenderable2DQuadSetTexture2(pThis, strFilePath, iFrame)	pThis->m_pTextures[1] = g_xPaperAPI.m_xGraphics.GetTexture(strFilePath, iFrame);

FORCE_INLINE void xRenderable2DQuadUpdateShaderParameters(xRenderable2DQuad *pThis);
FORCE_INLINE void xRenderable2DQuadUpdateDepth(xRenderable2DQuad *pThis, xSceneGraph *pSceneGraph, float fDepth) { xRenderable2DUpdateDepth(&pThis->m_xRenderable2D, pSceneGraph, fDepth); xRenderable2DQuadUpdateShaderParameters(pThis); }
FORCE_INLINE void xRenderable2DQuadUpdateColor(xRenderable2DQuad *pThis, const xColor *pSource) { xRenderable2DUpdateColor(&pThis->m_xRenderable2D, pSource); if(pThis->m_xRenderable2D.m_iBlendMode == PE_BLEND_MODE_2COLOR) xRenderable2DQuadUpdateShaderParameters(pThis); }
FORCE_INLINE void xRenderable2DQuadUpdateRGB(xRenderable2DQuad *pThis, const xColor *pSource) { xRenderable2DUpdateRGB(&pThis->m_xRenderable2D, pSource); if(pThis->m_xRenderable2D.m_iBlendMode == PE_BLEND_MODE_2COLOR) xRenderable2DQuadUpdateShaderParameters(pThis); }
FORCE_INLINE void xRenderable2DQuadUpdateBlendMode(xRenderable2DQuad *pThis, int iBlendMode) { xRenderable2DUpdateBlendMode(&pThis->m_xRenderable2D, iBlendMode); }
FORCE_INLINE void xRenderable2DQuadUpdateAlpha(xRenderable2DQuad *pThis, float fAlpha) { xRenderable2DUpdateAlpha(&pThis->m_xRenderable2D, fAlpha); if(pThis->m_xRenderable2D.m_iBlendMode == PE_BLEND_MODE_2COLOR) xRenderable2DQuadUpdateShaderParameters(pThis); }
FORCE_INLINE void xRenderable2DQuadUpdateTexture(xRenderable2DQuad *pThis, xTexture *pTexture) { xRenderable2DQuadSetTexture(pThis, pTexture); pThis->m_xRenderable2D.m_pVertData->m_pTextureData->m_uiTextureIndexArray[0] = pTexture->m_uiID; }
FORCE_INLINE void xRenderable2DQuadUpdateVisibility(xRenderable2DQuad *pThis, bool bVisible) { xRenderable2DUpdateVisibility(&pThis->m_xRenderable2D, bVisible); }

void xRenderable2DQuadUpdate(xRenderable2DQuad *pThis);
void xRenderable2DQuadUpdateAnimation(xRenderable2DQuad *pThis, float fTime);
void xRenderable2DQuadUpdatePartialX(xRenderable2DQuad *pThis, float fLeftPercentage, float fRightPercentage);
void xRenderable2DQuadUpdatePartialY(xRenderable2DQuad *pThis, float fPercentage);
void xRenderable2DQuad3HorizontalUpdate(xRenderable2DQuad *pThis);
void xRenderable2DQuad3VerticalUpdate(xRenderable2DQuad *pThis);
void xRenderable2DQuad4UpdateRadialPercentage(xRenderable2DQuad *pThis, float fPercentage);
void xRenderable2DQuad9Update(xRenderable2DQuad *pThis);

FORCE_INLINE void xRenderable2DQuadUpdateShaderParameters(xRenderable2DQuad *pThis)
{
	xColor *shaderparams;

	shaderparams = pThis->m_xRenderable2D.m_pVertData->m_pTextureData->m_xShaderParameterArray;

	switch(pThis->m_xRenderable2D.m_iBlendMode)
	{
		case PE_BLEND_MODE_MASKED:
			shaderparams[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].R = (pThis->m_fMaskTextureOffsetX) / (float)pThis->m_pTextures[1]->m_iDataWidth;
			shaderparams[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].G = (pThis->m_fMaskTextureOffsetY) / (float)pThis->m_pTextures[1]->m_iDataHeight;
			shaderparams[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].B = 0.0;
			shaderparams[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].A = 1.0;

			shaderparams[PE_SHADERPARAM_MASKED_TEXTURERATIO].R = (float)pThis->m_pTextures[0]->m_iWidth / (float)pThis->m_pTextures[1]->m_iWidth;
			shaderparams[PE_SHADERPARAM_MASKED_TEXTURERATIO].G = (float)pThis->m_pTextures[0]->m_iHeight / (float)pThis->m_pTextures[1]->m_iHeight;
			shaderparams[PE_SHADERPARAM_MASKED_TEXTURERATIO].B = 0.0;
			shaderparams[PE_SHADERPARAM_MASKED_TEXTURERATIO].A = 0.0;
			break;

		case PE_BLEND_MODE_2COLOR:
			shaderparams[PE_SHADERPARAM_2COLOR_COLOR2].R = pThis->m_xColor2.R;
			shaderparams[PE_SHADERPARAM_2COLOR_COLOR2].G = pThis->m_xColor2.G;
			shaderparams[PE_SHADERPARAM_2COLOR_COLOR2].B = pThis->m_xColor2.B;
			shaderparams[PE_SHADERPARAM_2COLOR_COLOR2].A = 0.0;

			shaderparams[PE_SHADERPARAM_2COLOR_COLORDIFF].R = pThis->m_xRenderable2D.m_xColor.R - pThis->m_xColor2.R;
			shaderparams[PE_SHADERPARAM_2COLOR_COLORDIFF].G = pThis->m_xRenderable2D.m_xColor.G - pThis->m_xColor2.G;
			shaderparams[PE_SHADERPARAM_2COLOR_COLORDIFF].B = pThis->m_xRenderable2D.m_xColor.B - pThis->m_xColor2.B;
			shaderparams[PE_SHADERPARAM_2COLOR_COLORDIFF].A = pThis->m_xRenderable2D.m_xColor.A;
			break;
	}
}

#endif