/////////////////////////////////////////////////////////////////
// Name: xRenderable3D.h
/////////////////////////////////////////////////////////////////
// Description:
// - A renderable 3D object
/////////////////////////////////////////////////////////////////
// - 01.01.2011, Created, Dayle Flowers (Xienen)
//   Created as new data structure for 3D game objects
/////////////////////////////////////////////////////////////////
#ifndef XRENDERABLE3D_H_
#define XRENDERABLE3D_H_

#include "xEngineDefines.h"
#include "xVector.h"
#include "xColor.h"
#include "xRenderData.h"

typedef struct xSceneGraph;

struct xRenderable3D
{
	float				 m_fRenderDepth;
	int					 m_iBlendMode;
	xTexture			*m_pTextures[2];
	xColor				 m_xColor;
	xVector				 m_xTranslation;
	xVector				 m_xRotation;
	xVector				 m_xScale;
	bool				 m_bVisible;
	xRenderDataVerts	*m_pVertData;
};

FORCE_INLINE void xRenderable3DInit(xRenderable3D *pThis)
{
	xColorInit(&pThis->m_xColor);
	xVectorInit(&pThis->m_xTranslation);
	xVectorInit(&pThis->m_xRotation);
	xVectorInit(&pThis->m_xScale, 1.0, 1.0, 1.0);

	pThis->m_fRenderDepth	= 0.0f;
	pThis->m_iBlendMode		= 0;
	pThis->m_pTextures[0]	= 0;
	pThis->m_pTextures[1]	= 0;
	pThis->m_bVisible		= 1;
	pThis->m_pVertData		= 0;
}

FORCE_INLINE void xRenderable3DCopy(xRenderable3D *pDest, xRenderable3D *pSource)
{
	xColorCopy(&pDest->m_xColor, &pSource->m_xColor);
	xVectorCopy(&pDest->m_xTranslation, &pSource->m_xTranslation);
	xVectorCopy(&pDest->m_xRotation, &pSource->m_xRotation);
	xVectorCopy(&pDest->m_xScale, &pSource->m_xScale);

	pDest->m_fRenderDepth	= pSource->m_fRenderDepth;
	pDest->m_iBlendMode		= pSource->m_iBlendMode;
	pDest->m_pTextures[0]	= pSource->m_pTextures[0];
	pDest->m_pTextures[1]	= pSource->m_pTextures[1];
	pDest->m_bVisible		= pSource->m_bVisible;
	pDest->m_pVertData		= pSource->m_pVertData;
}

FORCE_INLINE void xRenderable3DCopyTransform(xRenderable3D *pDest, xRenderable3D *pSource)
{
	xColorCopy(&pDest->m_xColor, &pSource->m_xColor);
	xVectorCopy(&pDest->m_xTranslation, &pSource->m_xTranslation);
	xVectorCopy(&pDest->m_xRotation, &pSource->m_xRotation);
	xVectorCopy(&pDest->m_xScale, &pSource->m_xScale);

	pDest->m_fRenderDepth	= pSource->m_fRenderDepth;
	pDest->m_bVisible		= pSource->m_bVisible;
}

FORCE_INLINE void xRenderable3DSetRenderDepth(xRenderable3D *pThis, float fRenderDepth) { pThis->m_fRenderDepth = fRenderDepth; }
FORCE_INLINE void xRenderable3DSetBlendMode(xRenderable3D *pThis, int iBlendMode) { pThis->m_iBlendMode = iBlendMode; }
FORCE_INLINE void xRenderable3DSetPosition(xRenderable3D *pThis, const xVector *pPosition) { xVectorCopy(&pThis->m_xTranslation, pPosition); }
FORCE_INLINE void xRenderable3DSetPosition(xRenderable3D *pThis, float fX, float fY, float fZ) { xVectorInit(&pThis->m_xTranslation, fX, fY, fZ); }
FORCE_INLINE void xRenderable3DSetRotation(xRenderable3D *pThis, const xVector *pRotation) { xVectorCopy(&pThis->m_xRotation, pRotation); }
FORCE_INLINE void xRenderable3DSetColor(xRenderable3D *pThis, const xColor *pSource) { xColorCopy(&pThis->m_xColor, pSource); }
FORCE_INLINE void xRenderable3DSetAlpha(xRenderable3D *pThis, float fAlpha) { pThis->m_xColor.A = fAlpha; }

FORCE_INLINE void xRenderable3DUpdateRenderDepth(xRenderable3D *pThis, xSceneGraph *pSceneGraph, float fRenderDepth)
{
	xRenderable3DSetRenderDepth(pThis, fRenderDepth);

	extern void xSceneGraphDepthChanged(xSceneGraph *pThis, xRenderDataVerts *pVertData, float fDepth);
	xSceneGraphDepthChanged(pSceneGraph, pThis->m_pVertData, fRenderDepth);
}

FORCE_INLINE void xRenderable3DUpdateBlendMode(xRenderable3D *pThis, int iBlendMode)
{
	xRenderable3DSetBlendMode(pThis, iBlendMode);

	if(pThis->m_pVertData->m_pTextureData->m_xShaderParameterArray)
		dlfree(pThis->m_pVertData->m_pTextureData->m_xShaderParameterArray);

	switch(iBlendMode)
	{
		case PE_BLEND_MODE_MASKED:
			pThis->m_pVertData->m_pTextureData->m_xShaderParameterArray = (xColor*)dlmalloc(sizeof(xColor) * 2);
			break;
		case PE_BLEND_MODE_2COLOR:
			pThis->m_pVertData->m_pTextureData->m_xShaderParameterArray = (xColor*)dlmalloc(sizeof(xColor) * 2);
			break;
	}
}

FORCE_INLINE void xRenderable3DUpdateColor(xRenderable3D *pThis, const xColor *pSource)
{
	xRenderable3DSetColor(pThis, pSource);

	xColorCopy(&pThis->m_pVertData->m_xColor, pSource);
}

FORCE_INLINE void xRenderable3DUpdateAlpha(xRenderable3D *pThis, float fAlpha)
{
	xRenderable3DSetAlpha(pThis, fAlpha);

	pThis->m_pVertData->m_xColor.A = fAlpha;
}

FORCE_INLINE void xRenderable3DUpdateVisibility(xRenderable3D *pThis, bool bVisible)
{
	pThis->m_bVisible = bVisible;

	if(!bVisible)
	{
		pThis->m_pVertData->m_bRender = 0;

		if(!pThis->m_pVertData->m_pNext && pThis->m_pVertData->m_pPrev == &pThis->m_pVertData->m_pTextureData->m_xVertDataList)
		{
			pThis->m_pVertData->m_pTextureData->m_bRender = 0;

			if(!pThis->m_pVertData->m_pTextureData->m_pNext && pThis->m_pVertData->m_pTextureData->m_pPrev == &pThis->m_pVertData->m_pShaderData->m_xTextureDataList)
				pThis->m_pVertData->m_pShaderData->m_bRender = 0;
		}
	}
	else
	{
		pThis->m_pVertData->m_pShaderData->m_bRender = 1;
		pThis->m_pVertData->m_pTextureData->m_bRender = 1;
		pThis->m_pVertData->m_bRender = 1;
	}
}

FORCE_INLINE void xRenderable3DUpdateRotation(xRenderable3D *pThis, const xVector *pRotation)
{
	xVectorCopy(&pThis->m_xRotation, pRotation);

	if(pThis->m_pVertData)
		xVectorCopy(&pThis->m_pVertData->m_xRotation, pRotation);
}

FORCE_INLINE void xRenderable3DRefreshTextures(xRenderable3D *pThis)
{
	pThis->m_pVertData->m_pTextureData->m_uiTextureIndexArray[0] = pThis->m_pTextures[0]->m_uiID;

	if(pThis->m_pTextures[1])
		pThis->m_pVertData->m_pTextureData->m_uiTextureIndexArray[1] = pThis->m_pTextures[1]->m_uiID;
}

#endif