#include "Paper.h"

char ErrorMessages[PE_RES_MAXERRORS][256] = 
{
	"There was no error reported, returning 0 means no error.",						//PE_RES_NOERROR,
	"Error creating a platform specific setting.",									//PE_RES_PLATFORMERROR,
	"Error creating the game window on your system.",								//PE_RES_CREATEWINDOWERROR,
	"Error creating DirectInput8. Make sure DirectX 8.0 or higher is installed.",	//PE_RES_DI8CREATEERROR,
	"Error creating the Direct Input Keyboard.",									//PE_RES_DI8CREATEKEYBOARDERROR,
	"Error aquiring the Direct Input Keyboard.",									//PE_RES_DI8AQUIREKEYBOARDERROR,
	"Error getting keyboard state in Direct Input, try reaquiring the keyboard.",	//PE_RES_DI8GETKEYBOARDSTATEERROR,
	"Error creating the Direct Input Mouse.",										//PE_RES_DI8CREATEMOUSEERROR,
	"Error aquiring the Direct Input Mouse.",										//PE_RES_DI8AQUIREMOUSEERROR,
	"Error getting mouse state in Direct Input, try reaquiring the mouse.",			//PE_RES_DI8GETMOUSESTATEERROR,
	"Error getting monitor display information.",									//PE_RES_D3DGETDISPLAYERROR,
	"Error trying to enable hardward vertices: using software vertices.",			//PE_RES_D3DUSINGSOFTWAREVERTICES
	"Error creating Direct3D device. Make sure DirectX 9.0 or higher is installed.",//PE_RES_D3DCREATEDEVICEERROR,
	"Error obtaining the Direct3D back buffer when attempting to render.",			//PE_RES_D3DGETBACKBUFFERERROR
	"Error loading the Direct3D texture from file: invalid pathname.",				//PE_RES_D3DINVALIDTEXTURERESOURCE,
	"Error loading the Direct3D texture from file: exact error unknown.",			//PE_RES_D3DTEXTURELOADERROR,
	"Error rendering the Direct3D scene: the device is not ready.",					//PE_RES_D3DDEVICENOTREADY,
	"Error setting OpenGL into full screen mode using Windows API calls.",			//PE_RES_OGLFULLSCREENERROR,
	"Error setting OpenGL pixel format using Windows API calls."					//PE_RES_OGLPIXELFORMATERROR,
	"Error loading the desired file: the file does not exist.",						//PE_RES_FILEDOESNOTEXIST,
	"Error opening the desired file: exact error unknown.",							//PE_RES_FILEOPENERROR,
	"Error reading or writing data to file: file has not been opened.",				//PE_RES_FILENOTOPEN,
	"Error reading data from file: exact error unknown.",							//PE_RES_FILEREADERROR,
	"Error loading the DMusic audio segment from file.",							//PE_RES_DMUSICLOADFILEERROR
	"Error loading the OpenAL audio segment from file.",							//PE_RES_OPENALMUSICLOADFILEERROR
	"This is a result signifying the end of the script, not an error.",				//PE_RES_ENDOFSCRIPT
	"Error calling requested script function.",										//PE_RES_FUNCTIONNOTFOUND
	"Error finding requested script variable.",										//PE_RES_VARNOTFOUND
	"Error loading font object or font load never attempted.",						//PE_RES_FONTOBJECTNOTLOADED
	"The specified object type does not match any known GUI object types.",			//PE_RES_INVALIDGUIOBJECTTYPE
	"Error creating fragment shader.",												//PE_RES_FRAGMENTERROR
};

int ErrorLevels[PE_RES_MAXERRORS] = 
{
	PE_RESLEVEL_NOACTION,		//PE_RES_NOERROR,
	PE_RESLEVEL_KILLAPP,		//PE_RES_PLATFORMERROR,
	PE_RESLEVEL_KILLAPP,		//PE_RES_CREATEWINDOWERROR,
	PE_RESLEVEL_GOTOSETTINGS,	//PE_RES_DI8CREATEERROR,
	PE_RESLEVEL_GOTOSETTINGS,	//PE_RES_DI8CREATEKEYBOARDERROR,
	PE_RESLEVEL_NOACTION,		//PE_RES_DI8AQUIREKEYBOARDERROR,
	PE_RESLEVEL_NOACTION,		//PE_RES_DI8GETKEYBOARDSTATEERROR,
	PE_RESLEVEL_GOTOSETTINGS,	//PE_RES_DI8CREATEMOUSEERROR,
	PE_RESLEVEL_NOACTION,		//PE_RES_DI8AQUIREMOUSEERROR,
	PE_RESLEVEL_NOACTION,		//PE_RES_DI8GETMOUSESTATEERROR,
	PE_RESLEVEL_KILLAPP,		//PE_RES_D3DGETDISPLAYERROR,
	PE_RESLEVEL_NOACTION,		//PE_RES_D3DUSINGSOFTWAREVERTICES,
	PE_RESLEVEL_GOTOSETTINGS,	//PE_RES_D3DCREATEDEVICEERROR,
	PE_RESLEVEL_KILLAPP,		//PE_RES_D3DGETBACKBUFFERERROR
	PE_RESLEVEL_NOACTION,		//PE_RES_D3DINVALIDTEXTURERESOURCE,
	PE_RESLEVEL_GOTOMENU,		//PE_RES_D3DTEXTURELOADERROR,
	PE_RESLEVEL_NOACTION,		//PE_RES_D3DDEVICENOTREADY,
	PE_RESLEVEL_GOTOSETTINGS,	//PE_RES_OGLFULLSCREENERROR,
	PE_RESLEVEL_GOTOSETTINGS,	//PE_RES_OGLPIXELFORMATERROR,
	PE_RESLEVEL_GOTOMENU,		//PE_RES_FILEDOESNOTEXIST,
	PE_RESLEVEL_GOTOMENU,		//PE_RES_FILEOPENERROR,
	PE_RESLEVEL_NOACTION,		//PE_RES_FILENOTOPEN,
	PE_RESLEVEL_NOACTION,		//PE_RES_FILEREADERROR,
	PE_RESLEVEL_GOTOMENU,		//PE_RES_DMUSICLOADFILEERROR
	PE_RESLEVEL_GOTOMENU,		//PE_RES_OPENALMUSICLOADFILEERROR
	PE_RESLEVEL_NOACTION,		//PE_RES_ENDOFSCRIPT
	PE_RESLEVEL_GOTOMENU,		//PE_RES_FUNCTIONNOTFOUND
	PE_RESLEVEL_GOTOMENU,		//PE_RES_VARNOTFOUND
	PE_RESLEVEL_GOTOMENU,		//PE_RES_FONTOBJECTNOTLOADED
	PE_RESLEVEL_GOTOMENU,		//PE_RES_INVALIDGUIOBJECTTYPE
	PE_RESLEVEL_KILLAPP,		//PE_RES_FRAGMENTERROR
};

void xResults::Init(void *pfMainMenu, void *pfSettings)
{
	m_pfMainMenu = (pfMainMenuCallBack)pfMainMenu;
	m_pfSettings = (pfSettingsCallBack)pfSettings;
}

int xResults::ReactToError(int iResult)
{
	switch(ErrorLevels[iResult])
	{
		case PE_RESLEVEL_GOTOMENU:
			m_pfMainMenu();
			break;
		case PE_RESLEVEL_GOTOSETTINGS:
			m_pfSettings();
			break;
		case PE_RESLEVEL_KILLAPP:
			g_xPaperAPI.m_bRunning = false;
			break;
	}

	return PE_RESLEVEL_NOACTION;
}

int xResults::GetErrorLevel(int iResult)
{
	return ErrorLevels[iResult];
}

char* xResults::GetResultString(int iResult)
{
	return ErrorMessages[iResult];
}